static char rcsid[] = "$Header: inst.c,v 1.6 91/02/13 09:45:46 mara Exp $";

/* inst.c
 *
 *  Written by Mara Helmuth
 *
 *  Description: Instrument class for cmix X graphical interface
 * 
 *  $Log:	inst.c,v $
 * Revision 1.6  91/02/13  09:45:46  mara
 * panel fixes
 * 
 * Revision 1.5  90/10/11  10:31:15  mara
 * *** empty log message ***
 * 
 * Revision 1.4  90/09/07  20:49:50  mara
 * fm
 * 
 * Revision 1.3  90/09/05  22:09:41  mara
 * reset works, +,-,*,/ ugens added
 * 
 * Revision 1.2  90/09/02  18:18:27  mara
 * NeXT fixes
 * 
 * Revision 1.1  90/08/27  17:08:45  mara
 * Initial revision
 * 
*/

#include "plus.h"

instrument::instrument() : cmd_list(50), loop_list(50)
{
	reset();
}

void instrument::process_ugen(int p, int l, int r)
{
	if(ugen[p].type) {
		unit_generator ug;
		switch(ugen[p].type) {
			case OSCIL:
				ug.write_oscil(p, l, r);
				break;
			case EVP:
				ug.write_evp(p, l, r);
				break;
			case BUZZ:
				ug.write_buzz(p, l, r);
				break;
			case ADD:
				ug.write_add(p, l, r);
				break;
			case SUBTRACT:
				ug.write_sub(p, l, r);
				break;
			case MULTIPLY:
				ug.write_mult(p, l, r);
				break;
			case DIVIDE:
				ug.write_div(p, l, r);
				break;
			case RAND:
				ug.write_rand(p, l, r);
				break;
			case OUT:
				ug.write_out(p, l, r);
				break;
			default:
			break;
		}
	}
}

void instrument::writer(ugen_str ugen[])
{
	num_ints = num_sfloats = num_float_ptrs = 0;
	system("mkdir Newinst");
	sprintf(name,"%s",(char *)panelitem_get(panel,pi_name,LXPTEXT_VALUE));
	sprintf(file,"Newinst/%s.c", name); 
	for(int i = 0; i < MAX; i++) {
		showit(i);
	}
	
//	 	find OUT ugen for tree root	
	i = 0;
	while(ugen[i].type != OUT && i < MAX) 
		i++;

	head();
	body(i);
	tail();
	profile();
	make();
}

void instrument::head()
{
	fp = fopen(file,"w");
	if(!fp)
		perror(file);
				// variables & includes for all instr:	
	fprintf(fp, "#include \"../cmix/H/ugens.h\"\n");
 	fprintf(fp, "#include \"../cmix/H/sfheader.h\"\n");
 	fprintf(fp, "#include <stdio.h>\n#include <sys/file.h>\n");
	fprintf(fp, "#include <sys/types.h>\n\n");
       	fprintf(fp, "extern SFHEADER      sfdesc[NFILES];\n\n");
	fprintf(fp, "%s(p,n_args)\n\n", name);
	fprintf(fp, "float *p; /* array of p-fields */\n");
	fprintf(fp, "int n_args; /* number of p-fields */\n\n{\n");

	fprintf(fp, "\tint nsamps,i,chans;\n");
	fclose(fp);
}

// for oscil

void instrument::body(int out_ugen)
{
	ugen_tree ugtree(sizeof(int), out_ugen, get_in0, get_in1);
	tree ntree(sizeof(int), out_ugen);

	fp = fopen(file,"a");
	if(!fp)
		perror(file);

	fprintf(fp, "\tfloat amp;\n");
	fprintf(fp, "\tfloat out[2];\n");

	fprintf(fp, "\tnsamps = setnote(p[0],p[1],0);\n"); 
	fprintf(fp, "\tchans = sfchans(&sfdesc[0]);\n");

// get variables for specific functions (lib)

	command();

	fprintf(fp, "\tfor(i = 0; i < nsamps; i++) {\n");
	loop_command();
	fclose(fp);
}

void instrument::command()
{
	while(cmd = cmd_list.get()) {
		fprintf(fp, "\t%s", cmd);
	}
	cmd_list.clear();
}

void instrument::loop_command()
{
	while(cmd = loop_list.get()) {
		fprintf(fp, "\t\t%s", cmd);
	}
	loop_list.clear();
}

void instrument::tail()
{
	fp = fopen(file,"a");
	if(!fp)
		perror(file);
	fprintf(fp, "\t\tADDOUT(out,0);\n\t}\n");

	fprintf(fp, "\tendnote(0);\n");
	fprintf(fp, "}\n");

	fclose(fp);
}

void instrument::profile()
{

	fp = fopen("Newinst/profile.c","w");
	if(!fp)
		perror("profile.c");

	fprintf(fp,"#include \"..\/cmix\/H\/ugens.h\"\n");
	fprintf(fp,"int FSPACE = 1;\n");
	fprintf(fp,"int NBYTES = 32768;\n\n");
	fprintf(fp,"profile()\n{\n");
	fprintf(fp,"\tUG_INTRO(\"%s\",%s);\n}\n",name,name);
	fclose(fp);
}

void instrument::make()
{
	fp = fopen("Newinst/makefile","w");
	if(!fp)
		errx("write_make");	

#ifdef WOOF
	fprintf(fp,"CFLAGS =  -ffpa -O\n");
#else
	fprintf(fp,"CFLAGS =  -O\n");
#endif
	fprintf(fp,"POBJECTS = %s.o profile.o\n",name);
	fprintf(fp,"LDFLAGS =  ../cmix/bin/cmix.o ../cmix/lib/genlib.a -lm\n\n");
	fprintf(fp,"%s:  ../cmix/H/ugens.h $(POBJECTS) ../cmix/bin/cmix.o\n",name);
#ifdef WOOF
	fprintf(fp,"\tcc -ffpa -o %s $(POBJECTS) $(LDFLAGS)\n",name);
#else
	fprintf(fp,"\tcc -o %s $(POBJECTS) $(LDFLAGS)\n",name);
#endif

	fclose(fp);
}

void instrument::compile()
{
	system("cd Newinst; make");
}

void instrument::set_data()
{
	graph.data_window();
/*
	p[0] = atof((char *)panelitem_get(datpanel,datout,LXPTEXT_VALUE));
	printf("p[0] = %f",p[0]);

	char * pstr = new char[20];
	strcpy(pstr,(char *)panelitem_get(datpanel,datout,LXPTEXT_VALUE));
	printf("pstr with strcpy  = %s",pstr);
	delete pstr;

	p[1] = atof(panelitem_get(datpanel,datdur,LXPTEXT_VALUE));
	p[2] = atof(panelitem_get(datpanel,datamp,LXPTEXT_VALUE));
*/
	
	sprintf(name,"%s",(char *)panelitem_get(panel,pi_name,LXPTEXT_VALUE));
	p[0] = atof(data.outskip);
	p[1] = atof(data.dur);
	p[2] = atof(data.amp);

	fp = fopen("Newinst/data","w");

	if(!fp)
		errx("instrument::data");

	fprintf(fp,"output(\"sf/%s\",0)\n",name);
	fprintf(fp,"sfclean(0)\n");
	fprintf(fp,"makegen(1,10,1024,1)\n"); 	// sine
	fprintf(fp,"makegen(2,7,512,0,512,1)\n");	// linear
	fprintf(fp,"makegen(3,5,512,0,512,1)\n");	// exponential
	fprintf(fp,"%s(",name);
		printf("%d pfields\n",pfield+1);
	for(int i = 0; i < pfield; i++)
		fprintf(fp,"%.3f, ",p[i]);
	fprintf(fp,"%.3f",p[pfield]);
	fprintf(fp,")\n");

	fclose(fp);
}

void instrument::reset()
{
	cmd_list.clear();
	loop_list.clear();
	num_ints = num_sfloats = num_float_ptrs = 0;
	pfield = 2;
	p[0] = 0;  	// start time
	p[1] = 3;  	// dur
	p[2] = 1; 	// amp
/*
	if(datpanel)
		reset_dat(datpanel,resetdat_button);
*/
	for(int i = 0; i < 30; i++)
	   if(ugen[i].made) {
		ugen[i].type = NONE;
		ugen[i].made = 0;
		ugen[i].connected = 0;
		switch(ugen[i].type) {
			case NONE:
				break;
			case OSCIL:
				free(ugen[i].oscil->amp);		
				free(ugen[i].oscil->freq);		
				free(ugen[i].oscil->phs);		
				free(ugen[i].oscil->out);		
				free(ugen[i].oscil);		
				break;
			case EVP:
				free(ugen[i].evp->amp);		
				free(ugen[i].evp->rise);		
				free(ugen[i].evp->decay);		
				free(ugen[i].evp->out);		
				free(ugen[i].evp);		
				break;
			case BUZZ:
				free(ugen[i].buzz->amp);		
				free(ugen[i].buzz->freq);		
				free(ugen[i].buzz->nhar);		
				free(ugen[i].buzz->out);		
				free(ugen[i].buzz);		
				break;
			case ADD:
			case SUBTRACT:
			case MULTIPLY:
			case DIVIDE:
				free(ugen[i].side->in0);	
				free(ugen[i].side->in1);	
				free(ugen[i].side->out);		
				free(ugen[i].side);		
				break;
			case RAND:
				free(ugen[i].rand->out);		
				free(ugen[i].rand);		
				break;
			case OUT:
				free(ugen[i].out->in);
				free(ugen[i].out);
			default:	
				break;
		}
	   }
}
