/*
 * State machine abstraction.
 * The idea is to keep as much of the hookup dialog
 * as possible in a text file, so that hookup protocols
 * are easy to change here or in other apps.
 * It has grown a few warts.
 *
 * M. J. Hawley
 * mike@media-lab.mit.edu
 * Copyright (c) November 1991, MIT Media Laboratory.
 */

extern strlen(), Command(), message(), Put(), Flush(), logout();
extern strcpy(), pgets(), strcmp();
extern char *index(), *rindex();

void setState();

typedef enum { Plain, Pattern, Status, Goto, FlushS, Pause, CmdS } SType;

typedef struct String {
    char *s;
    struct String *next;
} String;

typedef struct {
    int type;
    char *name;
    char *first, *last, *label;
    String *l;
} State;

#include "util.h"

SType lastType = Plain;

char *getStr(s,t) char *s, *t; {
    char end = ' ';

    *t = '\0';
    s = skipsp(s);
    lastType = Plain;
    switch (*s){
    Case '/': lastType = Pattern;
    Case '+': lastType = Status; s++;
    Case '!': lastType = CmdS; s++;
    Case ',': lastType = FlushS; s++;
    Case '=': lastType = Goto; s = skipsp(s+2);
    }

    if (*s=='/' || *s == '"') end = *s++;
    while (*s && !(*s == end || (end==' ' && *s == '\t'))){
        *t = *s++;
        if (*t == '\\') switch (*s){
            Case 'n': *t = '\n'; s++;
            Case 't': *t = '\t'; s++;
            Case 'b': *t = '\b'; s++;
            Default : *t = *s; s++;
        }
        t++;
    }
    *t = '\0';
    if (*s && *s != ' ' && *s != '\t') ++s;
    if (*s) s = skipsp(s);
    return s;
}

State *
newState(s) char *s; {
    State *S = Alloc(State);
    S->name = save(s);
    return S;
}

String *
addStr(s,t) String *s; char *t; {
    String *start = s, *n = Alloc(String);
    char *p;
    
    p = n->s = save(t);
    if (*p=='/' && (p = index(p+1,'/'))){
        while (p[-1]=='\\') p = index(p+1,'/');
        if (p) *p = '\0';
    }
    
    if (!s) return n;
    while (s->next) s = s->next;
    s->next = n;
    return start;
}

void
addString(S,s) State *S; char *s; {
    stripnl(s=skipsp(s));
    S->l = addStr(S->l,s);
}

#define MaxS 256
static State *ST[MaxS];
static int NS = 0;

void
readState(f) FILE *f; {
    char s[1024], n[1024];
    State *S = (State *)0;
    while (fgets(s,sizeof s,f)){
        stripcomment(s);
        if (blank(s)) continue;
        if (match(s,"[a-zA-Z]*:")){
            sscanf(s,"%[^:]",n);
            S = newState(n);
            ST[NS++] = S;
        } else
        if (S){
            addString(S,s);
        }
    }
}

State*
state(s) char *s; {
    int i;
    s = skipsp(s);
    for (i=0;i<NS;i++)
        if (strcmp(s,ST[i]->name)==0) return ST[i];
    return (State *)0;
}

char *str(s) char *s; { return s? s : ""; }

void
printState(s) State *s; {
    if (!s) printf("huh?\n"); else
    printf("%s: %s %s %s\n",s->name,str(s->first),str(s->last),str(s->label));
}

void
ReadState(s) char *s; {
    FILE *f;
    if (!NS){
        f = fopen(s,"r");
        if (f) readState(f), fclose(f);
    }
    setState("Attach");
}

State *curState = (State *)0;

void
setState(s) char *s; {
    curState = state(s);
    if (strcmp(s,"Detach")==0) logout();
    // if (strcmp(s,"Ready")==0) fetchReports();
}

void
execute(s) char *s; {
    char t[1024];

    if (s[0]=='/') s += strlen(s)+1;
    while ((s=getStr(s,t)) && *t) switch (lastType){
        Case CmdS:    Command(t);
        Case Status : message(t);
        Case Plain  : if (!state(t)) Put("%s",t);
        Case FlushS : Flush(t);
        Case Goto   : setState(t);
    }
}

void
execState(s,t) State *s; char *t; {
    String *l;
    if (!s) s = curState;
    if (!s || !s->l) return;
    for (l=s->l; l; l=l->next){
        if (l->s[0]!='/') execute(l->s);
        else
        if (l->s[0]=='/' && *t && match(t,l->s+1)) execute(l->s);
    }
}

void
runState(s) char *s; {
    execState(curState,s);
}

#define RAW 0
#define FIELD 1
#define PRETTY 2

int Format = 2;

void
setFormat(s) char *s; {
    if (!s) return;
    if (strcmp(s,"pretty")==0) Format = PRETTY; else
    if (strcmp(s,"fields")==0) Format = FIELD; else
    Format = RAW;
}

#define NS 512
static char
    s_name[NS],
    s_fips[NS],
    s_county[NS],
    s_st[NS], s_state[NS],
    s_na[NS], s_nation[NS],
    s_area[NS],
    s_elevation[NS], s_population[NS],
    s_feature_code[NS], s_feature[NS],
    s_location[NS],
    s_remark[NS],
    s_zone[NS],
    s_zip[NS], s_zipmore[NS*10];

#define P(f,c)  if (*c){\
                    sprintf(p,"%s:\t%s\n",f,c);\
                    p += strlen(p);\
                }
#define PP(a,b) sprintf(p,a,b); p += strlen(p);

static
reset(p) char *p; {
    if (*s_name) switch (Format){
    Case FIELD:
        P("city",s_name);
        P("county",s_county);
        P("county-code",s_fips);
        P("st",s_st);
        P("state",s_state);
        P("na",s_na);
        P("nation",s_nation);
        P("area-code",s_area);
        P("elevation",s_elevation);
        P("type",s_feature);
        P("type-code",s_feature_code);
        P("location",s_location);
        P("population",s_population);
        P("timezone",s_zone);
        P("zip",s_zip);
        P("zip-more",s_zipmore);
        P("remark",s_remark);
        PP("\n","");
    Case PRETTY:
        PP("%s",s_name);
        if (*s_state){ PP(", %s",s_state); }
        if (*s_nation && strcmp(s_na,"US")){PP(", %s",s_nation); }
        if (*s_zip)  { PP(" %s",s_zip); }
        PP("\n","");
        if (*s_feature && atoi(s_feature_code) != 45) { PP("%s, ",s_feature); }
        if (*s_population) { PP("pop. %s, ",s_population); }
        if (*s_elevation) { PP("elev. %s feet, ",s_elevation); }
        if (*s_zone) { PP("timezone %s, ",s_zone); }
        if (*s_county){ PP("%s county", s_county); }
        if (p[-1] != '\n'){ PP("\n",""); }
        if (*s_area){ PP("area code: %s\n",s_area); }
        if (*s_zipmore){ PP("other zip codes: %s\n",s_zipmore); }
        if (*s_remark){ PP("(%s)\n",s_remark); }
        PP("\n","");
    }
    s_name[0]= s_fips[0]= s_county[0]=
    s_st[0]= s_state[0]= s_na[0]= s_nation[0]= s_population[0] =
    s_area[0]= s_elevation[0]= s_feature_code[0] = s_feature[0]=
    s_location[0]= s_remark[0]= s_zone[0]= s_zip[0] = s_zipmore[0] = '\0';
}

#define nl stripnl
#define sp skipsp
#define S2(a,b) sscanf(s,"%*s %s %[^\n]",a,b); nl(a); nl(b); *s = '\0'
#define S1(a) if (*a) strcat(a,", ");\
              sscanf(s,"%*s %[^\n]",a+strlen(a));\
              strcpy(a,sp(a));\
              nl(a);\
              *s = '\0'

rewrite(s) char *s; {
    switch(Format){
    Case PRETTY:
    case FIELD:    switch (*s){
         Case '0': S1(s_name);
         Case '1': S2(s_fips,s_county);
         Case '2': S2(s_st,s_state);
         Case '3': S2(s_na,s_nation);
         Case 'A': S1(s_area);
         Case 'E': S1(s_elevation);
         Case 'F': S2(s_feature_code, s_feature);
         Case 'L': S1(s_location);
         Case 'P': S1(s_population);
         Case 'R': S1(s_remark);
         Case 'T': S1(s_zone);
         Case 'Z': if (!*s_zip){ S1(s_zip); } else { S1(s_zipmore); }
         Default : if (blank(s)) *s = '\0';
         }
    }
}

char *
getReport(buf) char *buf; {
    char *p = buf+1;
    char t[1024];
    int n = 1;
    strcpy(buf,"\n");
    message("found 0");
    reset(p);
    while (pgets(t,1024) && *t != '.'){
        if (Verbose) printf("+%s",t);
        if (*t == '0'){
            message("found %d",n++);
            reset(p);
            p += strlen(p);
        }
        rewrite(t);
        if (*t) strcpy(p,t);
        p += strlen(p);
    }
    reset(p);
    p += strlen(p);
    *p = '\0';
    return buf;
}
