//----------------------------------------------------------------------------------------------------
//
//	MailSpeaker
//
//	Inherits From:		Speaker
//
//	Declared In:		MailSpeaker.h
//
//	Class Description
//
//		MailSpeaker is a custom Speaker subclass that supports
//		remote messages implemented in Mail.app's custom Listener
//		subclass MailListener.  After setting its send port (via the method
//		"setSendPort: (port_t) aPort" ) to named port "MailSendDemo", 
//		you may message an instance of this class with one of the 
//		methods declared in this interface. It will in turn send Mail.app's 
//		MailListener instance the corresponding message (via a Mach 
//		message), which will be interpreted, and subsequently executed 
//		in Mail.app.
//
//		MailListener (and this class) support compose methods that both 
//		do and do not take a window number as an argument.  You can 
//		manage multiple compose windows by using those that do.  A
//		unique window number is returned by reference in the method
//		'openSend:'.  This is then used as the argument for methods that
//		take a window number.
//
//		Port information...  You should first set an instance's send port to
//		the named port "Mail".  This will launch Mail.app if not already.
//		Then, to perform the methods declared in this interface, set the 
//		send port to "MailSendDemo".
//		 
//		This class is generated by the Unix 'msgwrap' utility.  It has been 
//		reformatted for readability.  Mail.msg was the original file passed
//		to msgwrap.
//
//	Disclaimer
//
//		You may freely copy, distribute and reuse this software and its
//		associated documentation. I disclaim any warranty of any kind, 
//		expressed or implied, as to its fitness for any particular use.
//
//----------------------------------------------------------------------------------------------------
#import <appkit/Speaker.h>


@interface MailSpeaker : Speaker
{
}

//----------------------------------------------------------------------------------------------------
//  Compose Methods (window not specified)
//----------------------------------------------------------------------------------------------------
- (int) deliver;
- (int) openSend;
- (int) setBody : (char *) aString;
- (int) setCc : (char *) aString;
- (int) setSubject : (char *) aString;
- (int) setTo : (char *) aString;


//----------------------------------------------------------------------------------------------------
//  Compose Methods (window specified) 
//----------------------------------------------------------------------------------------------------
- (int) deliver : (int) aWindow;
- (int) openSend : (int *) aWindow;
- (int) setBody : (char *) aString inWindow : (int) aWindow;
- (int) setCc : (char *) aString inWindow : (int) aWindow;
- (int) setSubject : (char *) aString inWindow : (int) aWindow;
- (int) setTo : (char *) aString inWindow : (int) aWindow;

@end
