/***********************************************************************\
Font Entry class for Convert RTF which converts between Mac and NeXT rtf formats.
Copyright (C) 1993 David John Burrowes

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

The author, David John Burrowes, can be reached at:
	davidjohn@kira.net.netcom.com
	David John Burrowes
	1926 Ivy #10
	San Mateo, CA 94403-1367
\***********************************************************************/

/*
	Class: FontEntry
	Purpose:	When parsing an RTF file, it turns out that it would be nice to have a table of all the fonts listed in the RTF fonttbl block.  If one imagines a full table of font information to be in the form of a little 2D array, then this class corresponds to describing one row of that array.  Multiple instances of this class make up a table.
	In short, this class is merely a glorified struct for tracking information about a 'font'.
	It tracks the following information:
		The font number (inited to 0)
		An RTF token describing the font type (null object at first)
		A CString containing the name of the font (empy string at first)
		A count (intended for the number of times this font is used)  (initialized to 0)
		A flag indicated whether characters in this font should be converted (default is no conversion)
	In classic Mac misuse of terminology, then, this is really a type family entry.  oh well. 
*/

#import "ResultObject.h"
//
//	Values indicating different ways one can convert the font (more should be avail)
//
typedef enum
{
	NoConversion,
	FullConversion,
}
ConvertTypes;


@interface  FontEntry:ResultObject
{
	Integer			FontNumber;
	Instance			FontType;	// The RTF spec calls this the font family !!
	CString			FontName;
	Integer			Count;
	ConvertTypes	Convert;
}

- init;	
- free;
- IncrementCount;
- DecrementCount;
- (Integer) GetCount;
- SetCount: (Integer) NewCount;
- SetNumber: (Integer) NewNumber;
- (Integer) GetNumber;
- SetFontType: theType;
- (Instance) GetFontType;
- SetConversion: (ConvertTypes) newConvert;
- (ConvertTypes) GetConversion;
- SetName: (CString) TheName;
- (CString) GetName;
- AppendToName: (CString) TheName;
- PrepName;
@end
