/***********************************************************************\
Converter class for Convert RTF which converts between Mac and NeXT rtf formats.
Copyright (C) 1993 David John Burrowes

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

The author, David John Burrowes, can be reached at:
	davidjohn@kira.net.netcom.com
	David John Burrowes
	1926 Ivy #10
	San Mateo, CA 94403-1367
\***********************************************************************/


#import "AbstractConverter.h"
#import "FontEntry.h"
//
//	Types of possible source file kinds
//
typedef enum {
	MacRTF,
	NeXTRTF
} sources;
//
//	Which mode we are in the midst of in terms of writing things out
//
typedef enum {
	normal,
	inColorTable,
	inFontTable
} WritingState;

//
//	93.02.15	djb	added enumerated type for enhancement of types of font conversion.
//
typedef enum
{
	ConvertAll,
	ConvertStandard,
	ConvertNone
}
GuiConvertChoices;


@interface rtfConverter:AbstractConverter
{
	Boolean	colorsUsed;
	Boolean	addedSymbol;
	Instance	fontTable;
	sources	ConvertSource;
	Integer	SymbolNumber;
	Integer	numPicts;
	Integer	pictNum;
	Instance	MacTextConverter;
	Instance	NeXTTextConverter;
	Instance	CharConverter;
	
	Boolean	RemoveUnderline;
	Boolean	AlterSingleQuotes;
	GuiConvertChoices	ConvertAllText;
	Boolean	ExtractPicts;
	Boolean	ConvertPictures;
	Boolean	KeepTokensForMe;

	Boolean	PictConverterIsOpen;
	port_t	PictPort;
	
	//
	//	93.04.03	Added for bug-fix
	//
	ConvertTypes*	CharConvertStack;
	Integer			StackLocation;
	Integer			StackTop;
}

- init;
- free;
- (Boolean) isThisAGoodFile: Instance;

- SetConversionDirection: (Boolean) convertAsMacRTF;
- RemoveFirstNeXTUnderline: (Boolean) StripFirstUL0;
- ConvertTheSingleQuotes: (Boolean) doAsMac;
- SetTextConversion: (GuiConvertChoices) convertText;
- SetPictConversion: (Boolean) ConvertPictData AndDataRemoval:  (Boolean) DeletePictData;

- examineFile: sourceFile;
- ConvertFrom: sourceFile To: destinationFile;

- PrepareFontTable;
- WriteFontTableEntriesTo: destinationFile;
- ConvertWordIn: thetoken  To: destinationFile WithConversion: (ConvertTypes) ConvertChars;
- ConvertThisChar: (Character) aCharacter Into: destinationFile;
- ConvertMacPictImageIn: sourceFile  To: destinationfile;
- (ByteString) Convert: (Integer) sourceSize BytesToBinFrom: (ByteString) source;
//
//	93.04.03	djb	Added for char conversion bug fix
//
- ResetStack;
- StartNewGroup;
- FinishGroup;
- SetCurrentConverstionTo: (ConvertTypes) newValue;
- (ConvertTypes) GetCurrentConversion;

@end

