
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus

	File: Activator.h

	Description:

	The Activator provides a single interface for using the four differert ActivatorBars -- one for each edge of the screen.  This class defines methods for controlling the various attributes of the ActivatorBars, which together act as a single Activator object.  This includes setting which bars are active (visible), the color of the bars, and notifying the delegate when one of the bars is clicked in.
	
	Original Author: Jeremy Slade

	Revision History:
		Created
			V.101	JGS	Tue Mar  9 18:44:28 GMT-0700 1993

*/

#ifndef Actvator_h
#define Activator_h

#define Activator_VERSION		(101)


#import <objc/Object.h>
#import <appkit/color.h>


// Masks for showing which bars
#define ACT_NONE		0
#define ACT_TOP			(1 << 0)
#define ACT_LEFT		(1 << 1)
#define ACT_RIGHT		(1 << 2)
#define ACT_BOTTOM		(1 << 3)


@interface Activator : Object
{
	int	barMask;
	id	topBar;
	id	leftBar;
	id	bottomBar;
	id	rightBar;
	
	id	delegate;
	
	NXColor	fillColor;
	NXColor	borderColor;
}

// Creating, Initializing
+ initialize;
- init;
- free;

// Setting the delegate
- setDelegate:anObj;
- delegate;

// Controlling the bars
- setBar:(int)mask on:(BOOL)flag;
- (int)barMask;
- (BOOL)isBarOn:(int)mask;
- show:sender;
- hide:sender;
- setColor:(NXColor)color;
- (NXColor)color;
- setBorderColor:(NXColor)color;
- (NXColor)borderColor;
- setFloating:(BOOL)flag;
- (BOOL)isFloating;

// Activation
- barHit:sender;

// Dragging into activator bars
- setDraggingEnabled:(BOOL)flag;

@end


@interface Object ( ActivatorDelegate )
- activatorHit:sender;
@end


#endif // Activator_h

