
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: ActivatorBar
		
	Description:

	Provides functionality required for Acitvation bars used by the Activator. ActivatorBar has two basic duties:
	1) Size and position itself to a particular edge of the screen
	2) Notify the delegate (the Activator) when the bar is clicked on
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Tue Feb  2 18:42:19 GMT-0700 1993

*/


#ifndef ActivatorBar_h
#define ActivatorBar_h

#define ActivatorBar_VERSION 	(101)


#import <appkit/Panel.h>


// Position settings -- which edge of the screen is the bar on?
#define ABAR_NONE		0
#define ABAR_TOP		1
#define ABAR_RIGHT		2
#define ABAR_BOTTOM		3
#define ABAR_LEFT		4


@interface ActivatorBar : Panel
{
	int	pos;
}

// Creating, initializing
+ initialize;
- initAt:(int)position;
- free;

// Setting position
- setPosition:(int)position;
- setColor:(NXColor)color;
- setBorderColor:(NXColor)color;
- show:sender;
- hide:sender;

// Trapping mouseDown events
- barHit:sender;

// NXDraggingDestination Protocol
- (NXDragOperation)draggingEntered:(id <NXDraggingInfo>)sender;
- (NXDragOperation)draggingUpdated:(id <NXDraggingInfo>)sender;
- draggingExited:(id <NXDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NXDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NXDraggingInfo>)sender;
- concludeDragOperation:(id <NXDraggingInfo>)sender;

// Misc
- (BOOL)canBecomeKeyWindow;
- (BOOL)canBecomeMainWindow;

@end


@interface Object ( ActivatorBarDelegate )
- barHit:sender;
@end

#endif // ActivatorBar_h

