
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: AppIconView
	
	Description:

	The AppIconView is a custom view used to display customs icons, etc, in an Application's AppIcon window.  This class was originally developed to work as a general View class, but it has been modified to work especially for Locus.  The AppIconView automatically installs itself as a subview within a specified icon window view the attachToIcon: method.  You can then assign different icons/images to be displayed in the icon window.
	This view also conforms to the NXDraggingDestination Protocol to receive dragged files from other Apps.  For some reason, this is currently non-functional when used with the actual AppIcon window under NS 3.0.  I don't know if this is a bug in the NXDraggingDestination protocol, or if it is something else...
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Tue Feb  2 18:50:26 GMT-0700 1993

*/

#ifndef AppIconView_h
#define AppIconView_h

#define AppIconView_VERSION 	(101)


#import <appkit/View.h>
#import "Dragging.h"


@interface AppIconView : View 
{
	char *icon;
	id image;
	BOOL isAttached;
}

// Creating, Initializing instances
+ initialize;
- init;
- initFrame:(const NXRect *)frameRect;
- free;

// Attaching to an Icon window
- attachToIcon:window;
- detachFromIcon;

// Setting the Icon/Image to display
- setImage:anImage;
- setIcon:(const char *)name;
- (const char *)icon;
- image;
- drawSelf:(const NXRect *)rects :(int)count;

// NXDraggingDestination Protocol
- setDraggingEnabled:(BOOL)flag;
- (NXDragOperation)draggingEntered:(id <NXDraggingInfo>)sender;
- (NXDragOperation)draggingUpdated:(id <NXDraggingInfo>)sender;
- draggingExited:(id <NXDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NXDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NXDraggingInfo>)sender;
- concludeDragOperation:(id <NXDraggingInfo>)sender;

@end

#endif // AppIconView_h
