
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: DynamicItemSpec
	
	Description:
	
	DynamicItemSpec is a class used by Group to keep track of the specifications for the DynamicItems to be used by a Group.  The DynamicItemSpec consists of a shell-style file matching expression, such as "*.h", and a regular expression that is generated by translating the shell-style expression -- for the previous example, ".*\.h".  This is done because most users will be more familiar with shell-style matching expressions, but there is no direct support for them, while there is for regular expressions.  When a Group updates its dynamic items, it tells each DynamicItemSpec to find those items matching its expression and add them to the Group.
		
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Tue Feb  2 19:11:44 GMT-0700 1993
			
*/


#ifndef DynamicItemSpec_h
#define DynamicItemSpec_h

#define DynamicItemSpec_VERSION		(101)


#import <objc/Object.h>


@interface DynamicItemSpec : Object
{
	char *expression;	// Shell-style matching expression
	char *regexpr;		// Regular expression
	BOOL enabled;
}

// Creating, initializing
+ initialize;
- initExpr:(const char *)aString;
- free;

// Expression
- setExpression:(const char *)aString;
- (const char *)expression;
- (const char *)regexpression;
- setEnabled:(BOOL)flag;
- (BOOL)enabled;

// Scanning for matching items
- addMatchingItemsToGroup:group;

// Archiving
- awake;
- read:(NXTypedStream *)stream;
- write:(NXTypedStream *)stream;

@end

#endif // DynamicItemSpec_h
