
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	File: Globals.h

	Description:
	 
	This file contains global variables, and other standard stuff that are used by all classes ( such as importing <appkit/appkit.h>).  The global variables are declard as externs here; the actual storage for them is in Globals.m
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Mon Feb  1 22:18:39 GMT-0700 1993

*/


#ifndef Globals_h
#define Globals_h

#define Globals_VERSION	(101)


#import "ErrorReporting.h"
#import "VersionInfo.h"

#import <appkit/appkit.h> // Do this automatically for all classes


// Name of the directory under user's home where Folders are kept by default
#define LIBRARY_DIR		"Library/Locus"

// Name of the directory (w/in LIB_DIR) where Custom Icons are located
#define ICONS_DIR		"Icons"

// Name of the Folder automatically created when Locus is started
#define DEFAULT_FOLDER	"Dock"

// Name of the file w/in the Folder file package containing general info
#define FOLDER_INFO		".folderinfo"

// Extension for Folders (the file package)
#define FOLDER_EXT		"locus"

// Extnation for inidividual Group files within the Folder file package
#define GROUP_EXT		"locgrp"


// Is Locus AutoLaunching?
extern BOOL isAutoLaunch;


// debugging support -- see Locus_main.m
extern BOOL DEBUGGING;


/*
 *  Preferences ----------------------------------------------------------
 */

// Hide on autolaunch
#define PREF_AUTOLAUNCH_HIDE	"AutolaunchHide"
extern BOOL autolaunchHide;

// Mask defining which Activator bars are visible
#define PREF_ACTIVATOR_MASK	"ActivatorMask"
extern int activatorMask;

// Make activation bar float
#define PREF_ACTIVATOR_FLOAT		"ActivatorFloat"
extern BOOL floatActivator;

// Color of activator interior
#define PREF_ACTIVATOR_COLOR	"ActivatorFillColor"
extern NXColor activatorFillColor;

// Color of activator border
#define PREF_ACTIVATOR_BORDER	"ActivatorBorderColor"
extern NXColor activatorBorderColor;

// Location & Size of Main Panel
#define PREF_VIEWER_FRAME		"ViewerFrame"
extern NXRect defaultViewerFrame;

// Verify deletions, etc
#define PREF_VERIFY		"VerifyActions"
extern BOOL verifyActions;

// Autosave
#define PREF_AUTOSAVE	"AutoSave"
extern BOOL autoSave;

// Add through Activator
#define PREF_ADD_ACTIVATOR	"AddThroughActivator"
extern BOOL activatorAdd;

// Add through AppIcon
#define PREF_ADD_APPICON		"AddThroughAppIcon"
extern BOOL appIconAdd;

// InspectorMode
#define PREF_INSPECTORMODE	"InspectorMode"
extern int inspectorMode;

// InspectorLocation
#define PREF_INSPECTORLOCATION	"InspectorLocation"
extern NXPoint inspectorLocation;

// Hide on deactivate
#define PREF_HIDEDEACTIVE		"HideOnDeactivate"
extern BOOL hideDeactive;

// Startup Foldes
#define PREF_STARTUPFOLDERS		"StartupFolders"
extern id startupFolders;


// Defines the default values for preferences:
extern NXDefaultsVector Defaults;


// Valid drag in types
extern const char *dragInTypes[];
#define DRAGINTYPES	1



/*
 * Other Globals -------------------------------------------------
 */

// The MainController
extern id mainController;

// The FolderController
extern id folderController;

// The Inspector
extern id inspector;

// The Activator
extern id activator;

// The AppIcon View
extern id appIconView;
extern unsigned int appIconNum;


#define DIR_CREATE_MASK	(0777) // Mask used in creating directories


// Function used to set up global vars
void	initGlobals ( void );


#endif // Globals_h
