
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: GroupBrowserMatrix
	
	Description:

	This class is the matrix (single column) in which the Group is displayed.  This currently works for the browsing draw modes -- icon modes will need a completely different method of display.  The group to be displayed is installed directly as the matrix's cellList instead of being copied, for speed.
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Wed Feb  3 23:47:02 GMT-0700 1993

*/

#ifndef GroupBrowserMatrix_h
#define GroupBrowserMatrix_h

#define GroupBrowserMatrix_VERSION	(101)


#import <appkit/Matrix.h>
#import <appkit/drag.h>


@interface GroupBrowserMatrix : Matrix
{
	int selectionCount;
	id	activeCell;
}

// Creating, initializing instances
+ initialize;

// Loading the group
- setGroup:group;

// Responding to mouseDown events
- mouseDown:(NXEvent *)event;
- controlDragMouse:(NXEvent *)event;
- alternateDragMouse:(NXEvent *)event;

// Making Selection
- selectAll:sender;
- selectCellAt:(int)row :(int)col;
- selectCell:aCell;
- selectedCell;
- determineSelectionCount;
- selection;
- selectionList;
- (int)selectionCount;

// Drawing
- drawSelf:(NXRect *)rects :(int)count;
- scrollCellToVisible:(int)row :(int)col;
- sizeToCells;

// Cache windows used for faster drawing
- setupCacheWindows;
- sizeCacheWindow:cacheWindow to:(NXSize *)windowSize;

// NXDraggingSource Protocol
- (NXDragOperation)draggingSourceOperationMaskForLocal:(BOOL)isLocal;

@end

#endif // GroupBrowserMatrix_h
