
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: GroupInsPane
	
	Description:

	This is the ispector pane for Groups.  It is divided into two parts: the top half is general settings for the group, while the bottom half is divided into three separate categories of settings for the group, each displayed in their own sub-pane.
		
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Mon Feb  8 22:07:12 GMT-0700 1993
			
*/


#ifndef GroupInsPane_h
#define GroupInsPane_h

#define GroupInsPane_VERSION (101)


#import "InspectorPane.h"


@interface GroupInsPane : InspectorPane
{
	// General Group stuff
	id	nameField;
	id	keyField;
	id	pathField;
	id	sortSwitch;
	id	choosePathButton;
	id	drawModeMenu;	// PopUpList for chossing draw mode
	id	drawModeCover;	// Cover button for drawModeMenu
	id	subModeMenu;	// PopUpList for choosing sub-mode
	id	subModeCover;	// Cover button for subModeMenu
	id	swapView;		// SwapView for sub-mode panes
	int	subMode;
	
	// Display Options sub-mode
	id	displayOptsPane;
	id	dispOptFormatCover;		// Cover button for path format menu
	id	dispOptFormatMenu;		// Path format menu
	id	dispOptShowPath;
	id	dispOptActualImage;
	id	dispOptTriDots;
	id	dispOptShowInfo1;
	id	dispOptSmallIcons;

	// Allowed Types sub-mode
	id	allowedTypesPane;
	id	allowedBrowser;
	id	allowedRestrictSwitch;
	id	allowedField;
	id	allowedAddButton;
	id	allowedRemoveButton;
	
	// Dynamic Items sub-mode
	id	dynamicItemsPane;
	id	dynamicIntervalField;
	id	dynamicIntervalSlider;
	id	dynamicBrowser;
	id	dynamicField;
	id	dynamicDisableButton;
	id	dynamicEnableButton;
	id	dynamicRemoveButton;
	id	dynamicAddButton;
	
}

// Creating, initializing
+ initialize;
- initContent:(const NXRect *)contentRect
	style:(int)aStyle
	backing:(int)bufferingType
	buttonMask:(int)mask
	defer:(BOOL)flag;
- awakeFromNib;
- free;

// General Group stuff
- (BOOL)canInspect:anObject;
- showCurrent:sender;
- selectNameField:sender;
- changeGroupName:sender;
- changeGroupKey:sender;
- changeGroupPath:sender;
- chooseGroupPath:sender;
- changeGroupSort:sender;
- changeDrawToLargeBrowse:sender;
- changeDrawToSmallBrowser:sender;
- changeDrawToLargeIcon:sender;
- changeDrawToSmallIcon:sender;

// Setting the sub-mode
- setDisplayOptsMode:sender;
- setAllowedTypesMode:sender;
- setDynamicItemsMode:sender;
- swapPaneFor:sender;

// Display Options sub-mode
- setFormatFullPath:sender;
- setFormatFileAndPath:sender;
- setFormatFileOnly:sender;
- setFormatRelative:sender;
- setFormatNoPath:sender;
- setShowPath:sender;
- setShowActualImage:sender;
- setShowTriDots:sender;
- setShowInfoLine:sender;
- setShowSmallIcon:sender;

// Allowed Types sub-mode
- loadAllowedTypes:sender;
- allowedTypeSelected:sender;
- changeUseRestrictions:sender;
- newAllowedType:sender;
- removeAllowedTypes:sender;

// Dynamic Items sub-mode
- loadDynamicItemSpecs:sender;
- dynamicItemSpecSelected:sender;
- newDynamicItemSpec:sender;
- modifyDynamicItemSpec:sender;
- disableDynamicItemSpecs:sender;
- enableDynamicItemSpecs:sender;
- removeDynamicItemSpecs:sender;
- setDynamicUpdateInterval:sender;

// Text delegate methods
- textDidChange:sender;
- textDidEnd:sender endChar:(unsigned short)endChar;

// Browser delegate methods
- (int)browser:sender fillMatrix:matrix inColumn:(int)col;

@end

#endif // GroupInsPane_h
