
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: MainController
	
	Description:

	This class is the delegate of NXApp, and its main function is to direct the program flow.  A lot of the functionality that should be in this class is implemented by the FolderController, and it probably would have been cleaner and more intuitive to have a single Controller class, but that is not what happened.  This setup works, though it is not completely clear which controller is responsible for what, so I'm not going to worry about it at this point...
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Tue Feb  2 18:05:03 GMT-0700 1993

*/

#ifndef MainController_h
#define MainController_h

#define MainController_VERSION	(101)


#import <objc/Object.h>
#import <appkit/graphics.h> // For NXRect
#import <appkit/Window.h>


@interface MainController : Object
{
	id	infoPanel;
	id	preferences;
	
	id	theFolderController;	// Just points to global instance
								// Used for connecting in nib file
}

// Creating, initializing
+ initialize;
- init;
- free;

// Application delegate methods
- appWillInit:sender;
- appDidInit:sender;
- appWillTerminate:sender;
- appDidResignActive:sender;
- appDidBecomeActive:sender;
- appDidHide:sender;
- (int)app:sender openFile:(const char *)filename type:(const char *)aType;
- (BOOL)appAcceptsAnotherFile:sender;
- powerOff:(NXEvent *)theEvent;
- app:sender powerOffIn:(int)ms andSave:(BOOL)aFlag;

// Info
- showInfoPanel:sender;

// Prefs panel
- showPrefsPanel:sender;
- readPrefs:sender;
- writePrefs:sender;

// Activation
- showActivator:sender;
- hideActivator:sender;
- activatorHit:sender;
- doneActivating:sender;
- activator;

@end


#endif // MainController_h
