
/*
	Copyright 1993  Jeremy Slade.

	You are free to use all or any parts of the Locus project
	however you wish, just give credit where credit is due.
	The author (Jeremy Slade) shall not be held responsible
	for any damages that result out of use or misuse of any
	part of this project.

*/

/*
	Project: Locus
	
	Class: Preferences
	
	Description:
	
	Enables the user to set the global preferences, and then stores them in the defaults database.  Changes made are active at run-time, but will be lost when the app is terminated unless that are explicitly saved.
	
	Original Author: Jeremy Slade
	
	Revision History:
		Created
			V.101	JGS Mon Feb  8 22:49:15 GMT-0700 1993

*/


#ifndef Preferences_h
#define Preferences_h

#define Preferences_VERSION		(101)


#import <objc/Object.h>
#import <objc/zone.h>


// --------------------------------------------------------
//   Preferences Levels (different views)
// --------------------------------------------------------

#define PREFS_GENERAL_LEVEL		0
#define PREFS_VIEWER_LEVEL		1
#define PREFS_FOLDERS_LEVEL		2
extern char *prefsTitles[];


@interface Preferences : Object
{
	id	panel;
	
	id	swapView;
	id	modeCover; // Cover button for modeMenu -- need to set title
	
	int prefsLevel;
	
	// General options
	id	generalPrefsPanel;
	id	toggles;
	id	topSwitch;
	id	leftSwitch;
	id	rightSwitch;
	id	bottomSwitch;
	id	floatSwitch;
	id	interiorWell;
	id	borderWell;

	// Viewer frame options	
	id	viewerPrefsPanel;
	id	currentFrameButton;
	id	xField;
	id	yField;
	id	wField;
	id	hField;
	
	// Default folders
	id	folderPrefsPanel;
	id	browser;
	id	addButton;
	id	removeButton;
	
}

// Creating, initializing
+ initialize;
- init;
- awakeFromNib;
- free;

// Setting the preferences level
- setGeneralLevel:sender;
- setViewerLevel:sender;
- setFoldersLevel:sender;
- swapPaneFor:sender;

// Displaying
- panel;
- displayPrefs;
- displayActivator;
- displayToggles;
- displayViewerFrame;
- displayStartupFolders;

// Toggles
- setAutolaunchHide:sender;
- setHideDeactive:sender;
- setVerifyActions:sender;
- setAutoSave:sender;
- setActivatorAdd:sender;
- setAppIconAdd:sender;

// Activation Bar
- setActivatorTop:sender;
- setActivatorLeft:sender;
- setActivatorRight:sender;
- setActivatorBottom:sender;
- setActivatorFloat:sender;
- setActivatorFillColor:sender;
- setActivatorBorderColor:sender;

// Default Viewer frame
- setViewerFrameToCurrent:sender;
- setViewerX:sender;
- setViewerY:sender;
- setViewerW:sender;
- setViewerH:sender;

// Startup Folders
- (int)browser:sender fillMatrix:matrix inColumn:(int)col;
- addFolders:sender;
- removeFolders:sender;

// Saving settings
- setPrefs:sender;
- restorePrefs:sender;

@end

#endif // Preferences_h

