/*
 * Copyright (c) 1982 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)mille.h	5.3 (Berkeley) 6/18/88
 */

/*
 * @(#)mille.h	1.1 (Berkeley) 4/1/82
 */

/*
 * Miscellaneous constants
 */
	
#define	HAND_SIZE		7						// Number of cards in hand.
#define	HAND_DISTANCE_LIMIT				700		// The distance goal for each hand.
#define	HAND_DISTANCE_EXTENSION_LIMIT	1000	// If hand is extended the new distance goal.
#define	GAME_DISTANCE_LIMIT				5000	// Distance goal for a game.

												// These constants determine the number of each card
												//	in the deck.
#define	N_OUT_OF_GAS		3
#define	N_FLAT_TIRE			3
#define N_ACCIDENT			3
#define	N_SPEED_LIMIT		4
#define	N_STOP				5
#define	N_GASOLINE			6
#define	N_SPARE_TIRE		6
#define	N_REPAIRS			6
#define	N_END_OF_LIMIT		6
#define	N_ROLL				14
#define	N_RIGHT_OF_WAY		1
#define	N_PUNCTURE_PROOF	1
#define	N_DRIVING_ACE		1
#define	N_EXTRA_TANK		1
#define	N_200				4
#define	N_100				12
#define	N_75				10
#define	N_50				10
#define	N_25				10

#define	NUMBER_OF_CARDS_IN_DECK	(	N_OUT_OF_GAS + \
									N_FLAT_TIRE + \
									N_ACCIDENT + \
									N_SPEED_LIMIT + \
									N_STOP + \
									N_GASOLINE + \
									N_SPARE_TIRE + \
									N_REPAIRS + \
									N_END_OF_LIMIT + \
									N_ROLL + \
									N_RIGHT_OF_WAY + \
									N_PUNCTURE_PROOF + \
									N_DRIVING_ACE + \
									N_EXTRA_TANK + \
									N_200 +  N_100 + N_75 + N_50 + N_25 )
#define	NUMBER_OF_SAFETY_CARDS_IN_DECK ( N_RIGHT_OF_WAY + N_PUNCTURE_PROOF + N_DRIVING_ACE + N_EXTRA_TANK )


												// Enumerated constants assigned to the tag of each pile 
												//	according to its type.  Makes decision logic
												//	easier to switch() off a tag.
	typedef enum {	P_DISCARD = 1,
					P_HAND,
					P_SPEED,
					P_BATTLE,
					P_DISTANCE,
					P_SAFETY,
					P_DRAW
	} PILE_TYPE;


/*
 * Scores
 */

# define	SC_SAFETY		100
# define	SC_ALL_SAFE		300
# define	SC_COUP			300
# define	SC_TRIP			400
# define	SC_SAFE			300
# define	SC_DELAY		300
# define	SC_EXTENSION	200
# define	SC_SHUT_OUT		500


												// Enumerated type assigned to the tag of each card
												//	according to its type. 
	typedef enum {	C_25 = 0,
					C_50,					
					C_75,					
					C_100,					
					C_200,					
					C_OUT_OF_GAS_HAZARD,		
					C_FLAT_TIRE_HAZARD,			
					C_ACCIDENT_HAZARD,		
					C_STOP_HAZARD,			
					C_SPEED_LIMIT_HAZARD,	
					C_GASOLINE_REMEDY,		
					C_SPARE_TIRE_REMEDY,		
					C_REPAIRS_REMEDY,		
					C_ROLL_REMEDY,			
					C_END_OF_LIMIT_REMEDY,	
					C_EXTRA_TANK_SAFETY,		
					C_PUNCTURE_PROOF_SAFETY,	
					C_DRIVING_ACE_SAFETY,	
					C_RIGHT_OF_WAY_SAFETY	
	} CARD_TYPE;
	

/*
 * Score tag values.
 */
												// These are tag values used to index into a Matrix
												//	of TextFieldCells for the main scores displayed in
												//	the game window.
	typedef enum {	MAIN_MILESTONES_TAG = 0,
					MAIN_SAFETIES_TAG,
					MAIN_COUP_FOUREE_TAG,
					MAIN_HAND_TOTAL_TAG,
					MAIN_OVERALL_TOTAL_TAG,
					MAIN_GAMES_TAG
	} MAIN_SCORES_TAG_TYPE;
												// Ditto.  Except the matrix is in the "Other Scores"
												//	window.
	typedef enum {	OTHER_MILESTONES_TAG = 0,
					OTHER_SAFETIES_TAG,
					OTHER_ALL_FOUR_SAFETIES_TAG,
					OTHER_COUP_FOUREE_TAG,
					OTHER_TRIP_COMPLETE_TAG,
					OTHER_SAFE_TRIP_TAG,
					OTHER_DELAYED_ACTION_TAG,
					OTHER_EXTENSION_TAG,
					OTHER_SHUT_OUT_TAG,
					OTHER_HAND_TOTAL_TAG,
					OTHER_OVERALL_TOTAL_TAG,
					OTHER_GAMES_TAG
	} OTHER_SCORES_TAG_TYPE;


/*
 * Macros used to get score value objects for player's.
 */
#define	MAIN_MILESTONES_SCORE( _a )			[[ _a mainScoreMatrix ] findCellWithTag:MAIN_MILESTONES_TAG ]
#define	MAIN_SAFETIES_SCORE( _a )			[[ _a mainScoreMatrix ] findCellWithTag:MAIN_SAFETIES_TAG ]
#define	MAIN_COUP_FOUREE_SCORE( _a )		[[ _a mainScoreMatrix ] findCellWithTag:MAIN_COUP_FOUREE_TAG ]
#define	MAIN_HAND_TOTAL_SCORE( _a )			[[ _a mainScoreMatrix ] findCellWithTag:MAIN_HAND_TOTAL_TAG ]
#define	MAIN_OVERALL_TOTAL_SCORE( _a )		[[ _a mainScoreMatrix ] findCellWithTag:MAIN_OVERALL_TOTAL_TAG ]
#define	MAIN_GAMES_SCORE( _a )				[[ _a mainScoreMatrix ] findCellWithTag:MAIN_GAMES_TAG ]
#define	OTHER_MILESTONES_SCORE( _a )		[[ _a otherScoreMatrix ] findCellWithTag:OTHER_MILESTONES_TAG ]
#define	OTHER_SAFETIES_SCORE( _a )			[[ _a otherScoreMatrix ] findCellWithTag:OTHER_SAFETIES_TAG ]
#define	OTHER_ALL_FOUR_SAFETIES_SCORE( _a )	[[ _a otherScoreMatrix ] findCellWithTag:OTHER_ALL_FOUR_SAFETIES_TAG ]
#define	OTHER_COUP_FOUREE_SCORE( _a )		[[ _a otherScoreMatrix ] findCellWithTag:OTHER_COUP_FOUREE_TAG ]
#define	OTHER_TRIP_COMPLETE_SCORE( _a )		[[ _a otherScoreMatrix ] findCellWithTag:OTHER_TRIP_COMPLETE_TAG ]
#define	OTHER_SAFE_TRIP_SCORE( _a )			[[ _a otherScoreMatrix ] findCellWithTag:OTHER_SAFE_TRIP_TAG ]
#define	OTHER_DELAYED_ACTION_SCORE( _a )	[[ _a otherScoreMatrix ] findCellWithTag:OTHER_DELAYED_ACTION_TAG ]
#define	OTHER_EXTENSION_SCORE( _a )			[[ _a otherScoreMatrix ] findCellWithTag:OTHER_EXTENSION_TAG ]
#define	OTHER_SHUT_OUT_SCORE( _a )			[[ _a otherScoreMatrix ] findCellWithTag:OTHER_SHUT_OUT_TAG ]
#define	OTHER_HAND_TOTAL_SCORE( _a )		[[ _a otherScoreMatrix ] findCellWithTag:OTHER_HAND_TOTAL_TAG ]
#define	OTHER_OVERALL_TOTAL_SCORE( _a )		[[ _a otherScoreMatrix ] findCellWithTag:OTHER_OVERALL_TOTAL_TAG ]
#define	OTHER_GAMES_SCORE( _a )				[[ _a otherScoreMatrix ] findCellWithTag:OTHER_GAMES_TAG ]

