/* indent:4  tabsize:8  font:fixed-width */

/*---------------------------------------------------------------------------
|----------------------------------------------------------------------------
|
|    GameModule:Object
|
|    Abstract superclass.  Game modules (.solitaire) must have a controller
|    which is a subclass of GameModule.
|
|----------------------------------------------------------------------------
\---------------------------------------------------------------------------*/


#import <appkit/appkit.h>
#import "GameModule.h"
#import "Solitaire.h"

@interface GameModule:Object
{
    id		bundle;			// bundle for this module
    id		inspector;		// inspector view
    id		gameWindow;		// active game window
    id		prefs;			// game's preference object
    id		rulesWindow;		// window containing rules
    
    char        *gameName;              // name of game
    char        frameName[NX_MAXFRAMESTRINGLENGTH];
                                        // frame save string for gameWindow
    
    CardSize	cardSize;		// current size of cards
    BOOL	ignoreSizePref;		// TRUE if only one size of cards
                                        // available in this game
    CardBack	cardBack;		// pattern on card backs
    NXColor	desktopColor;		// color of desktop
}


- initFromBundle:aBundle withName:(const char *)name;

- inspector:sender;
- inspectorWillBeRemoved;
- inspectorInstalled;

- showRules:sender;

- startGame:sender;
- restartGame:sender;
- endGame:sender;

- win;
- lose;

- checkForWin;

- windowWillClose:sender;
- windowDidMove:sender;
- (BOOL) findPath:(char *)path forCardSize:(CardSize)cardSize
                                  realSize:(CardSize *)realSize;
- loadGameWindow:(const char *)path ofSize:(CardSize)size;

- commonGameSetup;


@end
