/* indent:4  tabsize:8  font:fixed-width */

/*----------------------------------------------------------------------------
|-----------------------------------------------------------------------------
|
|    Solitaire : ChangeManager
|
|    A single Solitaire object acts as the central controller for the
|    Solitaire game engine.  It is the application delegate, and is 
|    responsible for finding and loading .solitaire modules.
|
|    As a subclass of "ChangeManager", it can manage undo/redo.
|
|    Implementation of this class is split into Solitaire.m and 
|    SolitaireMore.m.
|
|    Thanks to Sam Streeper for BackSpace, which was a good source of 
|    sample code for this project.
|
|
|    CREDITS
|
|    Gary Ritchie<gary@uaneuro.uah.ualberta.ca>
|        -- dynamic loading, game engine, localization, documentation
|
|    Michael McCulloch<michael@hsv.tybrin.com>
|        -- artwork, CardSet preferences
|
|    Blake Stone<bstone@acs.ucalgary.ca>
|        -- original concept and Solitaire.app
|        -- CardSet classes, Klondike game
|
|-----------------------------------------------------------------------------
\----------------------------------------------------------------------------*/

#import <appkit/appkit.h>
#import "CardSet.subproj/cardset.h"
#import "undo.subproj/change.h"

@interface Solitaire:ChangeManager
{
    id		infoPanel;
    id		winPanel;		// default win panel
    id		settingsPanel;
    
    id		preferencesObject;
    id		gameSelectionBrowser;
    
    id		commonInspector;        // do-nothing inspector
    id		currentInspector;       // active inspector
    id		nullInspector;		// for games that wouldn't load
    NXRect	inspectorFrame;         // size of inspector
    id		oldInspectorOwner;      // game controller of previous game
    id		invisibleInspectorBox;  // view where inspectors are shown
    
    id		moduleList;		// list of available games

    int		realGameIndex;		// index of current game in browser

    BOOL	browserValid;           // are browser contents valid?
    BOOL	windowHasBeenDisplayed; // has the settings panel been shown?
    
}

- appDidInit:sender;
- appDidUnhide:sender;
- (BOOL)appAcceptsAnotherFile:sender;
- (int)app:sender openFile:(const char *)filename type:(const char *)aType;
- appWillTerminate:sender;
- app:sender powerOffIn:(int)ms andSave:(int)aFlag;

- selectGameIndex:sender;
- setGameIndex;
- installInspector:newInspector ownedBy:myController;
- gameController;

- (int)browser:sender fillMatrix:matrix inColumn:(int)column;
- (BOOL)browser:sender columnIsValid:(int)column;
- addCellWithString:(const char *)str at:(int)row toMatrix:matrix;


- newGame:sender;
- restartGame:sender;
- showRules:sender;

- showInfoPanel:sender;
- windowWillClose:sender;
- windowDidMove:sender;

- (CardSize)returnCardSize;
- (CardBack)returnCardBack;
- (NXColor)returnBackgroundColor;
- returnImageForSize:(CardSize)aSize;

- (BOOL)menuItemUpdate:(MenuCell *)menuCell;

- win;

@end


// Stuff for building the list of games

@interface Solitaire(More)

- getGameType;
- setGameType:(const char *)filename dir:(const char *)dirname;

- loadGamesFrom:(const char *)dirname;
- addGameToList:(const char *)filename dir:(const char *)dirname;

- (const char *)appDirectory;

@end

extern id SolEngine(void);
extern id SolGameController(void);
