/* Test application for ContourView.m class */
/* Izumi Ohzawa 92-5-22 */

#import <appkit/appkit.h>
#import "ContourPlotApp.h"
#import "ContourView.h"
#import "version.h"
#import <stdio.h>
#import <stdlib.h>
#import <math.h>

#define PI	3.141592654


@implementation ContourPlotApp

-appDidInit:sender
{
    zdata = NULL;

    // #### Some default settings which may be overridden later. ####
    nxInterp = 60;
    nyInterp = 60;
    xgstep = ygstep = 1.0;

    openPanel = [OpenPanel new];	// just caching
    [brightCWell setColor:NX_COLORRED];
    [darkCWell setColor:NX_COLORBLUE];
    [bgCWell setColor:NX_COLORWHITE];
    [lineCWell setColor:NX_COLORBLACK];
    [self colorWellsAction:self];	// reflect color well settings

    [versionField setStringValue: VERSION];


//  [contourView setDebugEnable:YES];		// only for debugging
    [contourView setEnabled: NO];		// start up disabled
    [contourView setFillEnable:YES];		// if color fill should be used
    [contourView setMinNumberOfPointsPerContour: 7];	// (optional) eliminates tiny noise
    [[contourView window] makeKeyAndOrderFront:self];
    return self;
}

// Loads the sample data file from app wrapper
- openSample:sender
{
char        buf[MAXPATHLEN + 1];
NXBundle   *bundle;

    bundle = [NXBundle mainBundle];
    if( [bundle getPath:buf forResource:"SpaceTimeRF" ofType:"2d"] ) {
	[self openFile: buf];
    }
    return self;
}

- viewSampleFile:sender
{
char       buf[MAXPATHLEN + 1];
NXBundle   *bundle;
NXStream   *stream;

    bundle = [NXBundle mainBundle];
    if( [bundle getPath:buf forResource:"SpaceTimeRF" ofType:"2d"] ) {
	if ((stream = NXMapFile(buf, NX_READONLY)) == NULL) {
	    NXRunAlertPanel("File Error",
			"Cannot open file %s","OK",NULL,NULL, buf);
	    return nil;
	}
	[commentText readText: stream];
	NXClose(stream);
	[[[commentText window] setTitleAsFilename: buf] orderFront:self];
    }
    return self;
}


// connect to Menu's File -> Open button.
//
- open:sender
{
static int FirstFile = 1;
const char *fileName;
const char *const types[2] = {NULL, NULL};
  if(FirstFile) {
	if ([openPanel runModalForDirectory: NXHomeDirectory() file: NULL types: types] 
		&& (fileName =[openPanel filename])) {
	    [self openFile:fileName];
	    FirstFile = 0;
	}
  }
  else {
	/* Use last directory again */
	if ([openPanel runModalForTypes:types] && (fileName =[openPanel filename])) {
	    [self openFile:fileName];
	}
  }
    return self;
}



static char *keyword[] =
	{
	"NUMPOINTS",	"XRANGE",	"YRANGE",	"ZDATA",	"GRIDSTEP",
	"NINTERPOL",
	""
	};
#define NKEYWORD	  6		/* # of names in *keyword[] */


- openFile:(const char *)fileName
{
FILE *fpin;
char linebuf[256], wdbuf[256];
int i, n2read;
int Done = 0;		// raised when ZDATA keyword is detected
int BadData = 0;	// raised if bad data detected
int prmnumber = 0;
int nzRead = 0;		// # of Z values read from file
NXStream *scStream;
char *scBuffer, *mbuf;
int scLen, dummy;

	if((fpin = fopen(fileName, "r")) == NULL) {
	    NXRunAlertPanel("File Error",
			"Cannot open file %s","OK",NULL,NULL, fileName);
	    return nil;
	}

	[[contourView window] setTitleAsFilename: fileName];
	// Invalidate data until new file loaded successfully.
	[contourView setEnabled:NO];	// disable plot until OK
	[contourView setComment:"% None" :4];
	nx = ny = -1;
        scStream = NXOpenMemory(NULL, 0, NX_WRITEONLY);	// create memory stream
        NXPrintf(scStream, "%% Contour Plot by: %s\n", VERSION);

	// Read line by line until we see ZDATA keyword, upon which
	// we switch to reading nx*ny float numbers regardless of line breaks.
	//
	while(!Done && !BadData)
	{
	    if(fgets(linebuf, 250, fpin) == NULL)
		break;
	    linebuf[250] = '\0';		/* for safety */
	    sscanf(linebuf, "%s", wdbuf);	/* first word */
	    if(wdbuf[0] == '%' || wdbuf[0] == '#' || wdbuf[0] == ';' || strlen(wdbuf) == 0) {
		if(wdbuf[0] == '%')
                    NXPrintf(scStream, "%s", linebuf);
		continue;	/* skip comment or blank lines */
	    }
	    prmnumber = -1;
	    for(i=0; i<NKEYWORD; i++) {
		   if( strcasecmp(wdbuf, keyword[i]) == 0 ) {
			prmnumber = i;
			break;
		   }
	    }

	    /* --- lookup in *parname[] done ----- */
	    if( prmnumber == -1 ) {
		fprintf(stderr, "...Unknown parameter: %s\n", wdbuf);
	    }
	    else {    /* valid prmnumber .. param name found */
		switch(prmnumber) {
		   case 0:	/* NUMPOINTS */
			sscanf(linebuf, "%*s %d %d", &nx, &ny);
			if(nx >= 5 && ny >= 5) {
			    if(zdata) free(zdata);
			    zdata = (float *)malloc((nx*ny)*sizeof(float));
			}
			else {
			    Done = BadData = 1;
	    		    NXRunAlertPanel("File Error",
			       "Bad number of points: (%d %d)","OK",NULL,NULL, nx, ny);
			}
			break;
		   case 1:	/* XRANGE */
			sscanf(linebuf, "%*s %f %f", &xmin, &xmax);
			break;
		   case 2:	/* YRANGE */
			sscanf(linebuf, "%*s %f %f", &ymin, &ymax);
			break;
		   case 3:	/* ZDATA */
			Done = 1;
			break;
		   case 4:	/* GRIDSTEP -- Optional */
			sscanf(linebuf, "%*s %f %f", &xgstep, &ygstep);
			break;
		   case 5:	/* NINTERPOL -- Optional */
			sscanf(linebuf, "%*s %d %d", &nxInterp, &nyInterp);
			break;

		} /* end switch(prmnumber) */
	    } /* end if( prmnumber == ..){} else {} */
	} /* end while( ... ) */

	if(BadData) {
	    fclose(fpin);
	    return nil;
	}
	// Now, start reading zdata[] values one by one
	nzRead = 0;
	n2read = nx*ny;
	for(i=0; i<n2read; i++) {
	    if(fscanf(fpin, "%f", &zdata[i]) == EOF) {
		BadData = 1;
	    	NXRunAlertPanel("File Error",
			"Premature end of file (%d read, %d expected)",
			"OK",NULL,NULL, nzRead, n2read);
		break;
	    }
	    nzRead++;
	} /* end of for(i=0...) */

	fclose(fpin);
	if(BadData) return nil;

	// Data successfully read
	// Copy comments in the data file to ContourView's comment to
	// keep track of data source.
        NXFlush(scStream);
        NXGetMemoryBuffer(scStream, &scBuffer, &scLen, &dummy);
	// This may not be needed, but I don't know if scBuffer above is null terminated.
	// If it is, malloc() stuff is not needed.
	mbuf = (char *)malloc(scLen+1);
	strncpy(mbuf, scBuffer, scLen);
	mbuf[scLen] = '\0';
	[contourView setComment: mbuf : scLen];
	[[commentText window] setTitle: "Comments from Data File"];
	[commentText setText: mbuf];
	free(mbuf);
        NXCloseMemory(scStream, NX_FREEBUFFER);

	// Set info fields from file
	[nxField setIntValue: nx];
	[nyField setIntValue: ny];
	[xminField setFloatValue: xmin];
	[xmaxField setFloatValue: xmax];
	[yminField setFloatValue: ymin];
	[ymaxField setFloatValue: ymax];
	// These may be optionally specified in file.
	[xgStepField setFloatValue: xgstep];
	[ygStepField setFloatValue: ygstep];
	[nxInterpField setIntValue: nxInterp];
	[nyInterpField setIntValue: nyInterp];

	// Set real data portion here.
	[self setDataToContourView];
	return self;
}


- setDataToContourView
{
    /* - setCartesianGridData:(float *)f :(float)xmin :(float)xmax
				  :(float)ymin :(float)ymax
				  ofSize:(int)nx :(int)ny
				  withInterpolationTo:(int)n1x :(int)n1y;
    */
    [contourView setCartesianGridData: zdata : xmin : xmax : ymin : ymax
				ofSize: nx : ny
				withInterpolationTo: nxInterp : nyInterp];
    [contourView setGridLineWidth:0.2 color:NX_COLORBLACK
			spacing: xgstep : ygstep];	// if grid is used
    [contourView setEnabled: YES];
    [contourView display];
    return self;
}




- setNcontourLevels:sender
{
static int new, old=16;
    new = [sender intValue]/2 *2;	// force to even number
    if(new == old) return self;
    old = new;
    [nlevelField setIntValue:new];
    [contourView setNumberOfContourLevels:new];
    [contourView display];
    return self;
}

- setLineWidth:sender
{
float newwidth;
    newwidth = [sender floatValue];
    if(newwidth < 0.0) newwidth = 0.0;
    if(newwidth > 20.0) newwidth = 20.0;
    [contourView setContourLineWidthMinor:newwidth andMajor: newwidth];
    [contourView display];
    return self;
}

- setGridSpacing:sender
{
float gx, gy;
    gx = [xgStepField floatValue];
    xgstep = gx;

    gy = [ygStepField floatValue];
    ygstep = gy;

    [contourView setGridLineWidth:0.2 color:NX_COLORBLACK
			spacing: xgstep : ygstep];	// if grid is used
    if([tickSwitch state] || [gridSwitch state])
        [contourView display];
    return self;
}

- setNInterpolation:sender
{
int ix, iy;
     ix = [nxInterpField intValue];
     if(ix < 5) [nxInterpField setIntValue: nxInterp];
     else nxInterp = ix;

     iy = [nyInterpField intValue];
     if(iy < 5) [nyInterpField setIntValue: nyInterp];
     else nyInterp = iy;

     [contourView setCartesianGridData: zdata : xmin : xmax : ymin : ymax
				ofSize: nx : ny
				withInterpolationTo: nxInterp : nyInterp];
     [contourView display];
     return self;
}


- clearAction:sender
{
    [contourView clear:self];
    return self;
}

- drawAction:sender
{
    [contourView display];
    return self;
}

- colorWellsAction:sender
{
NXColor cB, cD, cBG, cLine;
    cB =[brightCWell color];
    cD =[darkCWell color];
    cBG = [bgCWell color];
    cLine = [lineCWell color];
    [contourView setFillColors:cB :cD];
    [contourView setBackgroundColor:cBG];
    [contourView setContourLineColor:cLine];
    [contourView display];
    return self;
}

- useColorAction:sender
{
int state;
	state = [sender state];
	[contourView setFillEnable:(BOOL)state];
	[contourView display];
	return self;
}

- setGridMesh:sender
{
    if([sender state])
	[contourView setGridEnable:YES];
    else
	[contourView setGridEnable:NO];
    [contourView display];
    return self;
}

- setTickMark:sender
{
    if([sender state])
	[contourView setTicksEnable:YES];
    else
	[contourView setTicksEnable:NO];
    [contourView display];
    return self;
}


- setAxis:sender
{
    if([sender state])
	[contourView setAxisEnable:YES];
    else
	[contourView setAxisEnable:NO];
    [contourView display];
    return self;
}

- flipYAction:sender
{
int state;
	state = [sender state];
	[contourView setInvertY:(BOOL)state];
	[contourView display];
	return self;
}

// This fixes the problem of messed up text when you resize the
// document window.
- windowDidResize:sender
{
	[commentText calcLine];
	[commentText sizeToFit];
	return self;
}


@end
