/*	matrix.h - matrix algebra definitions
 *	Copyright (C) 1993 Corona Design, Inc. All rights reserved.
 *
 *	Abstract
 *		Common algebraic manipulations of 4X4 matrices and vectors.
 *		Reference: Foley & van Dam, Computer Graphics
 *	Requires C++ compiler.
 *
 *	RCS path: 
 *		$Source: /Users/pkron/Projects/voxel/Pyramid/matrix.subproj/RCS/matrix.cc,v $
 *	Modified: $Date: 93/09/15 12:33:50 $ by $Author: pkron $
 *	Current State: $State: Exp $ locked by $Locker:  $
 */

#include	"standard.h"
#include	"matrix.h"
#include	"perspect.h"

extern "C" {
#include	<math.h>
			}


VECTOR	unitVector = {1, 1, 1, 0};
struct _matrix	unitMatrix = 
		{{
			{1, 0, 0, 0},
			{0, 1, 0, 0},
			{0, 0, 1, 0},
			{0, 0, 0, 1}
		}};
			

float 	
_vector:: vectorLength()
	{
	return( sqrt( x*x + y*y + z*z));
	}
	
	
void	
_vector:: crossProduct( VECTOR *w)
	{
	float	x0 = x, y0 = y;
	
	x = y  * w->z - z  * w->y;
	y = z  * w->x - x0 * w->z;
	z = x0 * w->y - y0 * w->x;
	}
	
								// perform v = M * v	
	
void	
_vector:: matrixMultiply( MATRIX aMatrix)
	{
	float	x0 = x, y0 = y, z0 = z;
	float	(*M)[4][4] = &aMatrix->matrix;
	
	x = x0 * (*M)[0][0] + y0 * (*M)[0][1] + z0 * (*M)[0][2] 
			+ (*M)[0][3];
	y = x0 * (*M)[1][0] + y0 * (*M)[1][1] + z0 * (*M)[1][2] 
			+ (*M)[1][3];
	z = x0 * (*M)[2][0] + y0 * (*M)[2][1] + z0 * (*M)[2][2] 
			+ (*M)[2][3];
	w = x0 * (*M)[3][0] + y0 * (*M)[3][1] + z0 * (*M)[3][2] 
			+ (*M)[3][3];
	}


void
_vector:: scalarMultiply( float k)
	{
	x *= k;
	y *= k;
	z *= k;
	w *= k;
	}	
	


					// rotation vectors
VECTOR *
_matrix:: getRx()
	{
	return( (VECTOR *)&matrix[0][0]);
	}
	
	
VECTOR *
_matrix:: getRy()
	{
	return( (VECTOR *)&matrix[1][0]);
	}
	
	
VECTOR *
_matrix:: getRz()
	{
	return( (VECTOR *)&matrix[2][0]);
	}
	

								// perform A = B * A
void	
_matrix:: matrixMultiply( MATRIX bMatrix)
	{
	int		i;
	float	M[4][4];
	float	(*B)[4][4] = &bMatrix->matrix;
	
	M = matrix;
	for ( i=0; i LE 3; ++i )
		{
		matrix[0][i]	= (*B)[0][0] * M[0][i] + (*B)[0][1] * M[1][i] 
						+ (*B)[0][2] * M[2][i] + (*B)[0][3] * M[3][i];
		matrix[1][i] 	= (*B)[1][0] * M[0][i] + (*B)[1][1] * M[1][i] 
						+ (*B)[1][2] * M[2][i] + (*B)[1][3] * M[3][i];
		matrix[2][i] 	= (*B)[2][0] * M[0][i] + (*B)[2][1] * M[1][i] 
						+ (*B)[2][2] * M[2][i] + (*B)[2][3] * M[3][i];
		matrix[3][i] 	= (*B)[3][0] * M[0][i] + (*B)[3][1] * M[1][i] 
						+ (*B)[3][2] * M[2][i] + (*B)[3][3] * M[3][i];
		}
	}
	
	
#ifdef	_LOG
/*
 *	$Log:	matrix.cc,v $
Revision 1.1  93/09/15  12:33:50  pkron
Created

 */
#endif		
	
