
/* App Delegate object for addressing envelopes */

#import <ctype.h>
#import <appkit/appkit.h>
#import "Controller.h"

/* if you change these, fix DefaultDefaults below also */
NXDefaultsVector envelopeDefaults = {
	{"address", "D. Beatty &  C. Dragel\n\
6317-III Darlington Rd\n\
Pittsburgh, PA   15217"},
	{"size", "small"},
	{"caps", "1"},
	{"command", "lpr -PDeskJet300 -w80"},
	{0, 0}
};

@implementation Controller

- DefaultDefaults:sender
{
    [ReturnAddr setStringValue:"D. Beatty &  C. Dragel\n\
6317-III Darlington Rd\n\
Pittsburgh, PA   15217"];
    [PrintText setStringValue:"lpr -PDeskJet300 -w80"];
    [CapsSwitch setState:1];
    [SizeSel selectCellAt:1:1];
    return self;
}

- SaveDefaults:sender
{
    NXWriteDefault("Envelope", "size", 
    			[SizeSel selectedRow] == 1 ? "small" : "business");
    NXWriteDefault("Envelope", "caps", [CapsSwitch state] ? "1" : "0");
    NXWriteDefault("Envelope", "command", [PrintText stringValue]);
    NXWriteDefault("Envelope", "address", [ReturnAddr stringValue]);
    [PrefPanel performClose:self];
    return self;
}

- RevertDefaults:sender
{
    int useCaps, business;
    
    NXRegisterDefaults("Envelope", envelopeDefaults);
    useCaps= '1'==*(char*)NXGetDefaultValue("Envelope", "caps");
    business= !strcmp((char*)NXGetDefaultValue("Envelope", "size"),
    			 "business");
    [PrintText setStringValue:(char*)NXGetDefaultValue("Envelope", "command")];
    [ReturnAddr setStringValue:(char*)NXGetDefaultValue("Envelope", "address")];
    [CapsSwitch setState:useCaps];
    [SizeSel selectCellAt: (business ? 0 : 1 ) : 0];
    return self;
}

- ShowBig:sender
{
    [BigText setStringValue:[SmallText stringValue]];
    [SmallWindow orderOut:self];
    [BigWindow makeKeyAndOrderFront:self];
    return self;
}

- PrintBig:sender
{
    FILE *pp;
    pp= popen([PrintText stringValue], "w");
    fprintf(pp,"%s\n", [BigText stringValue]);
    pclose(pp);
    [BigWindow orderOut:self];
    [NXApp hide:self];
    return self;
}

- CloseBig:sender
{
    [BigWindow orderOut:self];
    [NXApp hide:self];
    return self;
}

- ShowSmall:sender
{
    [SmallText setStringValue:[BigText stringValue]];
    [BigWindow orderOut:self];
    [SmallWindow makeKeyAndOrderFront:self];
    return self;
}

- PrintSmall:sender
{
    FILE *pp;
    pp= popen([PrintText stringValue], "w");
    fprintf(pp,"%s\n", [SmallText stringValue]);
    pclose(pp);
    [SmallWindow orderOut:self];
    [NXApp hide:self];
    return self;
}

- CloseSmall:sender
{
    [SmallWindow orderOut:self];
    [NXApp hide:self];
    return self;
}

- appDidInit:sender
{
    MyListener = [NXApp appListener];
    [MyListener setServicesDelegate: self];
    [self RevertDefaults:self];
    return self;
}

- addressEnvelope:(id)pbid userData:(const char *)udata error:(char **)errmsg
{
char *data; 
int   length;
const char *const *types;
int   hasAscii, i;
int useCaps, business;
NXStream *contents;
char *conbuf; int conlen; int conmaxlen;
NXStream *pboard;
int c;

    useCaps= '1'==*(char*)NXGetDefaultValue("Envelope", "caps");
    business= !strcmp((char*)NXGetDefaultValue("Envelope", "size"),
    			 "business");
    types = [pbid types];           /* get a list of pasteboard types */
    hasAscii = 0;
    for(i=0; !hasAscii && types[i]; i++)
	if(!strcmp(types[i], NXAsciiPboardType)) hasAscii = 1;
    if(hasAscii) {
	[pbid readType:NXAsciiPboardType data:&data length:&length];
	if(data && length) {

	    pboard= NXOpenMemory(data, length, NX_READONLY);
	    contents= NXOpenMemory(NULL, 0, NX_WRITEONLY);

	    NXPrintf(contents, "%s", [ReturnAddr stringValue]);
	    NXPrintf(contents, business ? "\n\n\n\n\n\n\n\n\n" : "\n\n\n\n\n\n\n\n");

	    do {
		while ( (c= NXGetc(pboard)) && c != EOF && isspace(c) )
		    continue;
		if ( c != EOF ) {
		    NXPrintf(contents, business ? "\t\t\t\t\t\t" : "\t\t\t\t");
		    do {
			if (useCaps && isalpha(c) && islower(c) )
			    NXPutc(contents, toupper(c));
			else
			    NXPutc(contents, c);
		    }
		    while ( (c= NXGetc(pboard)) && c != EOF && c != '\n');
		    NXPutc(contents, '\n');
		}
	    } while ( c != EOF );

	    NXCloseMemory(pboard, NX_SAVEBUFFER); /* ? */
	    NXGetMemoryBuffer(contents, &conbuf, &conlen, &conmaxlen);
	    conbuf[conlen] = '\0';

	    if (business) {
		[BigText setStringValue:conbuf];
		[BigWindow makeKeyAndOrderFront:self];
	    } else {
		[SmallText setStringValue:conbuf];
		[SmallWindow makeKeyAndOrderFront:self];
	    }
	    NXCloseMemory(contents, NX_FREEBUFFER); 
	
	} /* end if(data && length) */
	else
	    *errmsg = "Selection is empty.";
	vm_deallocate((vm_task_t)task_self(), 
			(vm_address_t)data, (vm_size_t)length);
    }  /* end if(hasAscii) */
    else
	*errmsg = "No ASCII text found in your selection.";
    return self;
}


@end
