// -------------------------------------------------------------------------------------
// PaletteMatrix.h
// -------------------------------------------------------------------------------------

#import <libc.h>
#import <mach/cthreads.h>
#import <sys/time.h>
#import <objc/zone.h>
#import <objc/error.h>
#import <appkit/nextstd.h>
#import <appkit/graphics.h>
#import <appkit/Font.h>
#import <appkit/color.h>
#import <appkit/Matrix.h>
#import <appkit/Panel.h>
#import "ImagePortfolio.h"

// -------------------------------------------------------------------------------------
#define	myPastboardTYPE			"imageCellPath"

// -------------------------------------------------------------------------------------

@interface PaletteMatrix : Matrix
{

	BOOL			pbCopyTiff;					// copy tiff image to pasteboard
	BOOL			pbCopyPath;					// copy path to pasteboard
  
	id				imageList;					// main image cell container
	id				deleteList;					// delete list save for undelete
	int				maxCols;
	id				delegate;					// cell delegate
	mutex_t			loadMutex;					// cell image load mutex
	BOOL			loadingCells;				// cell loading flag
  
}

// -------------------------------------------------------------------------------------

- initFrame:(const NXRect*)frameRect;
- free;
- freeCells;
- freeSelectedCells;

- delete:sender;
- undelete:sender;
- (BOOL)undeletable;
- saveToFile:(char*)fileName;

- (BOOL)setPreferences:(char*)prefBuff returnRows:(int*)rows cols:(int*)cols;
- setDelegate:anObject;
- setCellSize:(const NXSize*)cellSize;
- (NXSize*)cellSize;
- (NXSize*)intercell;
- resizeAndDisplay;

- setFont:fontObj;
- addImage:(const char*)filePath;
- addImageCell:iCell;
- (int)imageListCount;
- imageAt:(int)index;

- (BOOL)anySelectableCells;
- findCellWithImageFilePath:(char*)fullPath;
- selectedCell;
- selectedCellList;
- (char*)selectedCellPaths;
- (int)selectedCellCount;

- sortByCellTitle:sender;
- shellSort:(SEL)sortCompare;

- (char*)getPreferenceString:(char*)buff;

@end
