//
// OldEvent.m
// Copyright (c) 1988, 1989, 1990 by Jiro Nakamura 
// All rights reserved
//
// Implementation definition of class OldEvent. OldEvent handles the database management
// of Cassandra.
// This is for convert. Using Event.m v2.3   modified 90/02/03
//
// RCS Information
// Revision Number->	$Revision: 2.3 $
// Last Revised->	$Date: 90/02/03 13:01:44 $
//
static char rcsid[] = "$Header: /user/jiro/Programming/Cassandra/src/RCS/Event.m,v 2.3 90/02/03 13:01:44 jiro Exp $";

#import "OldEvent.h"
#import <objc/objc.h>
#import <strings.h>
#import <appkit/Panel.h>       /* for NXRunAlertPanel */
#import "cass.h"	
#import "calendar.h"
#import "Global.h"
#import "misc.h"
#ifdef DEBUG
	#undef DEBUG
#endif

// Redefine NXRunAlertPanel since this is not a interactive GUI program
#define NXRunAlertPanel(head, mesg, but1, but3, but4) fprintf(stderr, mesg)

@implementation OldEvent

+ newAt:(const char*) eFile
{
	self = [super new];
	strcpy(eventFile,eFile);

	priority = 50;
	destroy = anniversary = sleepNo = sleepInt =0;
	present = next = previous = 0;

	strcpy(alarmSound, ALARMSOUND);
	strcpy(msg,"empty");
	return self;
}

+ new
{
	fprintf(stderr,"This should never be called!");
	self = [super new];
	return self;
}

- free
{
#ifdef DEBUG
	fprintf(stderr,"Event.m:Free ->Closing event!\n");
#endif
	[super free];
	return nil;
}

//
//
// Messages that affect how the file is stored on disk
//
//

- default: sender
{
	present =  0;
	previous = next = -1;
	destroy =  1;
	anniversary =sleepInt = sleepNo = priority = 0;
	ts.tm_sec =  ts.tm_min =  ts.tm_hour = 0;
	ts.tm_mday = ts.tm_mon =  ts.tm_year = 0;
	ts.tm_wday =  ts.tm_yday =  ts.tm_isdst = 0;
	
	strcpy(msg, "");
	strcpy(alarmSound,[global alarmSound]);
	return self;
}


- readEvent  : (EFileLink) here		/* read itself from the event file */
{
	FILE * source;
	static char errormsg[128];
		
	#ifdef DEBUG
		fprintf(stderr,	"------------------------\n"
					"[OldEvent readEvent: %d]\n",here);
	#endif
	

	if( here < 0)
		{
		NXRunAlertPanel("File Seek Error", "Old: Cannot read a negative "
			"number. ","OK",NULL,NULL);
		strcpy(msg, "Illegal event/link value....");
		next = previous = -1;
		present = here;
		return nil;
		}

	sprintf(errormsg, "Fatal error occured while trying to read event file %s", eventFile);
	source = fileOpen(eventFile, "r+",errormsg);

	sprintf(errormsg, "An fatal error occured when we were trying to seek to event number %d for reading event file %s.\n This program has crashed.", here, eventFile);
	if( fileSeek(source, here , errormsg) == NULL)
		{
		next = previous = -1;
		present = here;
		return nil;
		}
	
	present = fgetnum(source);
	if( present != here)
		{
		fprintf(stderr, "Uh oh.... File header gives %d instead of %d.\nWe've most probably run out of file. I'll try to compensate...\n",present, here);
		next = previous = -1;
		present = here;
		return nil;
		}
	else
		{
		previous = fgetnum(source);
		next = fgetnum(source);
		}
	
	if(fscanf( source, 	"\n\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n"
					"\n\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n%d\n"
					"%s\n\n"
					"\n\n\n",
					&priority, &destroy, &anniversary,
					&sleepNo, &sleepInt,
					&ts.tm_sec, &ts.tm_min, &ts.tm_hour,
					&ts.tm_mday, &ts.tm_mon, &ts.tm_year,
					&ts.tm_wday, &ts.tm_yday, &ts.tm_isdst,
					alarmSound
					) != 15)
		{
		fprintf(stderr, "Error in scanning.... uhoh..\n\n");
		if( here == 0)
			{
				strcpy(msg, CASSANDRAVERSION);
				[self writeEvent: here];
			}
		strcpy(msg, "Looks like we've run out of event file, bud.\nBut no problem. It's cool.");
		fclose(source);
		return nil;
		}
	else
		fgets(msg,300,source);
			
	ts.tm_wday = wday(ts.tm_mday, ts.tm_mon+1, 1900+ts.tm_year);
	ts.tm_yday = yday(ts.tm_mday, ts.tm_mon+1, ts.tm_year+1900);
	
	fclose(source);
	return self;
}
				
- firstEvent
{
	#ifdef DEBUG
		fprintf(stderr,"Reading in the first event.\n");
	#endif

	[self readEvent : 0];
	if( [self next] == 0 || [self next] == -1)
		return self;
	[self readEvent : [self next]];
	return self;
}	
	
- writeEvent : (EFileLink) here	
/* write itself into the event file using present as its index */
{
	FILE *source;
	char errormsg[256];
		
	if( here < 0)
		{
		NXRunAlertPanel( "Write Error", "We can't write to a negative event number.","OK",NULL,NULL);
		return nil;
		}

	sprintf(errormsg, "Fatal error when trying to write to event file %s", 
		eventFile);
	source = fileOpen(eventFile, "r+",errormsg);
	
	sprintf(errormsg, "An fatal error occured when we were trying to seek to event number %d for writing in event file %s.\n This program has crashed.", here, eventFile);
	fileSeek(source, here, errormsg);
	
	present = here;
	
	if( ts.tm_year > 1900)
		ts.tm_year -= 1900;

	if(fprintf( source, 	"%d\n%d\n%d\n"
					"\n\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n"
					"\n\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n%d\n"
					"%d\n%d\n%d\n"
					"%s\n\n"
					"\n\n\n"
					"%s\n",
					present, previous, next,
					priority,destroy, anniversary,
					sleepNo, sleepInt,
					ts.tm_sec, ts.tm_min, ts.tm_hour,
					ts.tm_mday, ts.tm_mon, ts.tm_year,
					ts.tm_wday, ts.tm_yday, ts.tm_isdst,
					alarmSound,msg) == -1)
		{
		fprintf(stderr, "Event.m:   Error occured writing file.");
		strcpy(msg, "Error occured in writing out Elink number.\n\n");
		fclose(source);
		return nil;
		}
	
		#ifdef DEBUG
			fprintf(stderr, "Wrote out Efilelink.\n  Here = %d, Previous = %d, Next=%d\n", here, previous, next);
			fprintf(stderr, "Date:   time = %d:%d, day= %d, month = %d, year = %d.\n\n",ts.tm_hour, ts.tm_min, ts.tm_mday, ts.tm_mon, ts.tm_year);
		#endif
		
	fclose(source);
	return self;
}

- (EFileLink) insertEvent
{
	return [self insertEventFrom : 1];
}



/* insert itself into the file with current day as index */
/* and starting from <here> in searching for a open deleted space */
- (EFileLink) insertEventFrom : (EFileLink) here
{
	id event, step;
	int loop;

	// First things first, we must check the format of our time structure since 
	// we have given other functions full freedom to scew it up

	fixTmStructure( &ts);
			
	event = [OldEvent newAt:eventFile];
	step = [OldEvent newAt:eventFile];
	[event readEvent:0];
	for( [event readEvent: [event next]]; 
	      [event present] != 0 && timeCompare([event time], &ts) < 1;
	      [event readEvent:[event next]])
	      	{
	      	#ifdef DEBUG
			fprintf(stderr, "From %d looping onto %d\n",[event present],[event next]);
		#endif
		}
	#ifdef DEBUG	
		fprintf(stderr, "inserting between %d <- event -> %d.\n",[event previous], [event present]);
	#endif
	
	previous = [event previous];
	next = [event present];
	
	[step setNext: 0];
	for( loop = here; !([step next] == -1 && [step previous] == -1); [step readEvent: loop++])
		{
		#ifdef DEBUG
			fprintf(stderr, "reading at %d.\n",loop);
		#endif
		}
		
	present = [step present];
	
	[self writeEvent: present];
	
	[step readEvent: previous];
	[step setNext : present];
	[step writeEvent:previous];
	
	[step readEvent: next];
	[step setPrevious : present];
	[step writeEvent:next];
	
	[step free];
	[event free];
	#ifdef DEBUG
		fprintf(stderr,"successfully inserted %d <- %d -> %d\n\n",[self previous],[self present], [self next]);
	#endif

	return( [self present] );
}

- deleteEvent  : (EFileLink) here		/* delete itself from the file with present as index */
{
	id previousEvent, nextEvent;
	
	#ifdef DEBUG
		fprintf(stderr, "Deleting <%d>\n\n",here);
	#endif
	
	if( here <= 0)
		{
			NXRunAlertPanel(NULL,"You cannot delete link 0 or below. OK?", "Shucks...",NULL,NULL);
			strcpy(msg,"We can't delete Link 0 because it is the header, ok?");
			fprintf(stderr,"Can't delete the flipping header, ok!  Here = %d.\n\n",here);
			return nil;
		}
	
	previousEvent= [OldEvent newAt: eventFile];
	nextEvent = [OldEvent newAt:eventFile];
	
	[previousEvent readEvent  :here];
	if( ( [previousEvent previous] == -1  )   ||  (  [previousEvent next]  == -1  ))
		{
		/*   link was already deleted, so we can't delete it..... */
		strcpy(msg, "Can't delete link because it's already been deleted!?!\n\n");
		return nil;
		}
	
	[nextEvent readEvent :[previousEvent next]];				/* Set the previous_link of the next link */
	[nextEvent setPrevious  : [previousEvent previous]];			/* to the one of the previous one */
	[nextEvent writeEvent  :[previousEvent next]];				/* confusing, eh? */
	
	[nextEvent readEvent  :[previousEvent previous] ];	
	/* do the same confusing spiel for previous */
	[nextEvent setNext  : [previousEvent next]];
	[nextEvent writeEvent  :[previousEvent previous] ];
	
	[previousEvent setNext  : -1];	/* cancel out the header to finally kill it */
	[previousEvent setPrevious  :-1];	
	[previousEvent writeEvent  :here];	/* and write it out */
	
	#ifdef DEBUG
		fprintf(stderr, "Record %d successfully deleted, O Leader.\n\n",here);
	#endif
	
	[previousEvent free];
	[nextEvent free];
	return self;
}


- (int) murderEvent : (EFileLink) here
/* delete if it is not an anniversary event, update otherwise */
{
	OldEvent *event;
	int annv;
	
	#ifdef DEBUG
		fprintf(stderr,"Arrggh! Help! I'm getting murdered! (says OldEvent <%d>)\n\n",here);
	#endif
	
	event = [OldEvent newAt :eventFile];
	[event readEvent : here];
	
	[event deleteEvent : here];

	switch( [event destroy])
		{
		case 0:
			break;
		case 1:
			return -1;
		default:
			[event setDestroy : ([event destroy] -1 )];
		}
		
	
	annv = [event anniversary];
	switch( annv / 100)      
		{
		/* Don't worry about overflows as we add intervals to our dates */
		/* insertEvent is smart enough to handle it */
		case 0:
			return -1;		/* No anniversary */
		case 1:				/* If it is a day anniversary */
			[event setMday : ([event mday] + (annv - 100))];
			break;
		case 2:				/* Weekly anniversary */
			[event setMday : ([event mday] + (annv - 200) * 7 )];
			break;
		case 3:				/* Monthly anniversary */
			[event setMon : ([event mon] + (annv - 300))];
			break;
		case 4:				/* Yearly anniversary */
			[event setYear : ([event year] + (annv - 400))];
			break;
		default:
			 fprintf(stderr,"I do not recognize anniversary event: %d.\n",annv);
			 return -1;
		}
	
	/* Use the same EFilelink space that we deleted, this saves us time and */
	/* maybe some confusion */
	return( [event insertEventFrom:here]  );
}


- (EFileLink) present
{
	return present;
}

- (EFileLink) previous
{
	return previous;
}

- (EFileLink) next
{
	return next;
}

- setPresent : (EFileLink) apresent
{
	present = apresent;
	return self;
}
- setPrevious  : (EFileLink) aprevious
{
	previous =aprevious;
	return self;
}

- setNext  : (EFileLink) anext
{
	next = anext;
	return self;
}

- (struct tm *) time
{
	return &ts;
}

- setTime : (struct tm *) time
{	
	ts = *time;
	return self;
}

- setMday : (int) x
{	
	ts.tm_mday = x;
	return self;
}

- (int) mday;
{
	return( ts.tm_mday);
}

- setMon: (int) x
{
	ts.tm_mon = x;
	return self;
}

- (int) mon
{
	return (ts.tm_mon);
}

- setYear: (int) x
{
	ts.tm_year = x;
	return self;
}

- (int) year
{
	return (ts.tm_year);
}

- setHour: (int) x
{
	ts.tm_hour = x;
	return self;
}

- (int) hour
{
	return (ts.tm_hour);
}

- setMin: (int) x
{
	ts.tm_min = x;
	return self;
}

- (int) min
{
	return(ts.tm_min);
}

- setSec: (int) x
{
	ts.tm_sec = x;
	return self;
}

- (int) sec
{
	return(ts.tm_sec);
}

- (int) destroy
{
	return destroy;
}

- setDestroy : (int) dst
{
	destroy = dst;
	return self;
}


- (int) anniversary
{
	return anniversary;
}

- setAnniversary : (int) anv
{
	anniversary = anv;
	return self;
}


- (int) sleepNo
{
	return sleepNo;
}

- setSleepNo : (int) sn
{
	sleepNo = sn;
	return self;
}


- (int) sleepInt
{
	return sleepInt;
}

- setSleepInt : (int) si
{
	sleepInt = si;
	return self;
}

- (int) priority
{
	return priority;
}

- setPriority : (int) si
{
	priority = si;
	return self;
}

- (char *) alarmSound
{
	return alarmSound;
}


- setAlarmSound : (char *) aS
{	
	strcpy( alarmSound, aS);
	return self;
}



- (char *) message
{
	return msg;
}


- setMessage : (char *) message
{	
	strcpy( msg, message);
	return self;
}

- setWday : (int) wd
{
	ts.tm_wday = wd;
	return self;
}

- setYday : (int) yd
{
	ts.tm_yday = yd;
	return self;
}

- (int) wday
{
	return ts.tm_wday;
}

- (int) yday
{
	return ts.tm_yday;
}
@end		


