/* File: MagnifyView.h - View class for 'Magnify'
 *
 * By: Christopher Lane
 * Symbolic Systems Resources Group
 * Knowledge Systems Laboratory
 * Stanford University
 *
 * Date:  6 November 1992
 *
 * Copyright: 1990, 1991 & 1992 by The Leland Stanford Junior University.
 * This program may be distributed without restriction for non-commercial use.
 */
 
#import <mach/cthreads.h>
#import <appkit/appkit.h>

#define NX_NOBUTTONS (0)
#define NX_MAXPLANES (5)

@interface MagnifyView : View
{  
	int flags;
	mutex_t lock;
	NXPoint mouse;
	NXSize offset;
	float scale, gridSize;
	NXRect virtualBounds;
	NXColorSpace colorSpace;
	unsigned char *data[NX_MAXPLANES];
	BOOL frozen, showCursor, showGrid;
	id invisibleWindow, invisibleView;
	
	int pixelsWide;
	int pixelsHigh;
	int bitsPerSample;
	int samplesPerPixel;
	int bitsPerPixel;
	int bytesPerRow;
	int planarConfig;
	int photoInt;
}

- initFrame:(const NXRect *) frameRect;
- (BOOL) acceptsFirstResponder;
- copy:sender;
- drawSelf:(const NXRect *) rects :(int) rectCount;
- free;
- sizeTo:(NXCoord) newWidth :(NXCoord) newHeight;
- mouseMoved:(NXEvent *) theEvent;

- drawCursor;
- drawGrid;

- updateBitmap:(NXPoint *) point;

- (int) flags;
- (NXPoint *) mouse;

- (float) scale;
- setScale:(float) value;
- (float) gridSize;
- setGridSize:(float) value;

- (BOOL) isFrozen;
- setFrozen:(BOOL) state;
- setShowGrid:(BOOL) state;
- setShowCursor:(BOOL) state;

- createWindows;

@end
