#import <objc/Object.h>
#import "Global.h"

/* Version 0.9b			Apr-19-92		First Public Release	*/
/* Version 0.95b		Jun-10-92		Minor File path fixes	*/
/* Version 1.0b			Aug-10-92		Minor Bug Fixes			*/

/* The various shared objects and panels are declared as instance variables for	*/
/* several reasons:  first, you can draw connections in IB that way; second, 	*/
/* it is possible that some other object might want to manipulate these objects	*/
/* for their own purposes.														*/

@interface MultApp: Object
{
	BOOL	haveOpenedDocument;			/* whether we have opened a document yet */
    id 		plpAccessory;				/* accessory view in the page layout panel */
	id 		infoPanel;					/* the Info Panel */
	id		prefPanel;					/* the Preferences panel */
	id		helpPanel;					/* a simple Help Panel (until 3.0!) */
	id		pageMargin;					/* augmented PageLayout object */
	id		stringSet;					/* StringSet for localization */
	char	launchDir[MAXPATHLEN+1];	/* the app wrapper directory */
	char	defaultDir[MAXPATHLEN+1];	/* the default directory */
	/* Defaults for the application */
	BOOL	saveAll;					/* save all upon closing */
	BOOL	dumpAll;					/* ignore dirty bits upon closing */
}

/* Factory Methods */

+ initialize;
+ setDocClass:newDoc;
+ docClass;

/* Public Methods */

- currentDocument;
- setDefaultDir:(const char *)dir;
- (const char *)currentDirectory;
- (const char *)launchDirectory;

- setupPageLayout:(float)lm :(float)rm :(float)tm :(float)bm; 

- (BOOL)saveAll;
- setSaveAll:(BOOL)value;
- (BOOL)dumpAll;
- setDumpAll:(BOOL)value;

/* Shared Panels Target/Action */

- info:sender;
- help:sender;
- pref:sender;
- saveAsPanel:sender;
- pageLayout:sender;
- stringSet:sender;

/* Target/Action Methods */

- new:sender;
- open:sender;
- saveAll:sender;
- print:sender;
- mailToMe:sender;

/* Menu updating method */

- (BOOL)menuItemUpdate:menuCell;
- (BOOL)validateCommand:menuCell;

/* Application Delegate Methods */

- appWillInit:sender;
- appWillTerminate:sender;
- appDidInit:sender;
- (int)appOpenFile:(const char *)path type:(const char *)type;
- (BOOL) appAcceptsAnotherFile:sender;
- (int)app:sender unmounting:(const char *)fullPath;

/* Listener Methods */

// Drag & Drop
- (int)iconEntered:(int)windowNum at:(double)x :(double)y iconWindow:(int)iconWindowNum iconX:(double)iconX iconY:(double)iconY iconWidth:(double)iconWidth iconHeight:(double)iconHeight pathList:(const char *)pathList;
- (int)iconExitedAt:(double)x :(double)y;
- (int)iconReleasedAt:(double)x :(double)y ok:(int *)flag;

//File Manipulation
- (int)msgDirectory:(char *const *)fullPath ok:(int *)flag;
- (int)msgFile:(char *const *)fullPath ok:(int *)flag;

// Misc
- (int)msgPrint:(const char *)fullPath ok:(int *)flag;
- (int)msgVersion:(char *const *)aString ok:(int *)flag;
- (int)msgQuit:(int *)flag;

@end
