# Begin TickleServices Version 1.02mab Data
"Divider" = "170.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # Splits multi-file Workspace-style lists into TCL-style lists.
proc filenames {} {
    global NXFilenamePboardType
    
	# If there is only one filename, make certain it's a list.
	# Otherwise, split on \\t.
    set filenames [pasteboard read $NXFilenamePboardType]
    if {[string first \"\\t\" $filenames]!=-1} {
	set filenames [split $filenames \"\\t\"]
    } else {
	set filenames [list $filenames]
    }
    return $filenames
}

    # Pass \"arguments\" as arguments to \"command\".  The arguments
    # are reformed into a list not containing newlines, since
    # otherwise newlines would separate commands.  Thus, passargs
    # can be used as a convenience to make scripts easier to read.
proc passargs {command arguments} {
    return [uplevel $command [join [split $arguments]]]
}

    # Determine the NeXTSTEP version.
if {![regexp \"NeXT Mach (\\[^:\\n\\]*): .*\" [exec hostinfo] ignore \
NeXTVersion]} {
    set NeXTVersion Unknown
}
if {[string match \"2.?\" $NeXTVersion]} {
    proc WSNotify {} {
    }
} else {
    proc WSNotify {} {
	catch {exec [findcmd WSNotify]}
    }
}

    # Attempt to find the cmdName command and return the path to
    # it.  It is cached in the global Command$cmdName variable for
    # future uses and in the default TickleServices Command$cmdName
    # for future runs.
proc findcmd {cmdName} {
    set varname Command$cmdName
    global $varname env
    
	# I have to say [set $varname] to get the value of $varname,
	# because $$varname won't do it (that attempts to get the
	# value of the variable \"$varname\" instead).  This works
	# because the $varname substitution happens _before_ set
	# is called.
    if {[info exists $varname] && [file executable [set $varname]]} {
	return [set $varname]
    }

	# Get location from defaults.
    set cmd [defaults read TickleServices $varname \"\"]
    
	# If it's not set or not there, try to find it.
    if {$cmd==\"\" || ![file executable $cmd]} {
	set cmd \"\"
	
	    # Attempt to find the user's path.  This is so bizarre
	    # because you want to get the PATH from the user's
	    # command-line shell, note TickleServer's PATH, which
	    # was inherited from Workspace and is not sufficient
	    # to our needs.
	if {[info exists env(SHELL)]} {
	    set shell $env(SHELL)
	} else {
	    set shell /bin/csh
	}
	    # This has only been verified to work with these six
	    # shells.  If the user isn't using one of them, try
	    # csh as a last resort under the assumption that their
	    # shell won't work, or might even do something discourteous
	    # like never exit.  Let me know if other shells work.
	if {[lsearch [file tail $shell] {bash csh rc sh tcsh zsh}]==-1} {
	    set shell /bin/csh
	}
	    # The @/: crud is to differentiate the echo'ed output
	    # from anything the shell puts out in the course of
	    # starting up.  In other words, it is a hack.
	if {![catch {exec $shell -c \"echo \\\"@@:@@:@@\\$PATH@:@@@@:@\\\"\"\
} path]} {
	    if {[regexp \"@@:@@:@@(\\[^#\\]*)@:@@@@:@\" $path barf path]} {
		set path [split $path \":\"]
		foreach dir $path {
		    if {[file executable $dir/$cmdName]} {
			set cmd $dir/$cmdName
			break
		    }
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices.app
	    # wrapper.
	if {$cmd==\"\"} {
	    set apppaths [defaults read Workspace ApplicationPaths {}]
	    if {$apppaths==\"\"} {
		set apppaths {
		    ~/Apps
		    /LocalApps
		    /NextApps
		    /NextDeveloper/Apps
		    /NextAdmin
		    /NextDeveloper/Demos
		}
	    } else {
		set apppaths [split $apppaths \":\"]
	    }
	    foreach path $apppaths {
		if {[file executable $path/TickleServices.app/$cmdName]} {
		    set cmd $path/TickleServices.app/$cmdName
		    break
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices
	    # ServicesPaths directories.
	if {$cmd==\"\"} {
	    set servicespaths [defaults read TickleServices ServicesPaths {}\
]
	    if {$servicespaths==\"\"} {
		set servicespaths {
		    ~/Library/TickleServices
		    /LocalLibrary/TickleServices
		}
	    } else {
		set servicespaths [split $servicespaths \":\"]
	    }
	    foreach path $servicespaths {
		if {[file executable $path/$cmdName]} {
		    set cmd $path/$cmdName
		    break
		}
	    }
	}
	
	    # If we've found something, ask the user for verification.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    case [alertpanel \"Find $cmdName executable\" \"Use `$cmd' for $\
cmdName?\" \"OK\" \"Choose\" \"Cancel\"] {
		0	{set cmd \"\"}
		-1	return
	    }
	}
	
	    # If we haven't found it yet, ask the user about it.
	if {$cmd==\"\" || ![file executable $cmd]} {
	    set cmd [passargs openpanel {
		-title \"Select $cmdName executable\"
		-prompt \"Name:\"
	    }]
	}
	
	    # If we've got it, write it to the defaults database.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    defaults write TickleServices $varname $cmd
	} else {
	    return
	}
    }
    set $varname $cmd
    return $cmd
}


    # These lists give the archive extensions and commands to
    # operate on those extensions.  Commands are passed to TCL
    # \"exec\" for execution in the working directory the selected
    # files are in.  Within the commands, %f is replaced with the
    # archive name, while %F is replaced with the filename with
    # the archive name removed.  %{cmd} is replaced with the full
    # path to the Unix command cmd as returned by findcmd.
    # 
    # To remove elements, prepend a '#' between the '{' and '.',
    # otherwise TCL won't parse the sublists correctly.  Of course,
    # you _could_ just remove them from the list entirely!
    
    # These commands are for _creating_ archives.
set archivers {
    {.compressed	{%{tar} -cf - %F | %{compress} > %f}}
    {.tar.Z	{%{tar} -cf - %F | %{compress} > %f}}
    {.tar.F	{%{tar} -cf - %F | %{freeze} -c > %f}}
    {.tar.Y	{%{tar} -cf - %F | %{yabba} > %f}}
    {.tar.gz	{%{tar} -cf - %F | %{gzip} -9c > %f}}
    {.tar.zip	{%{tar} -cf - %F | %{zip} -9q > %f}}
    {.tar	{%{tar} -cf %f %F}}
    {.afio.Z	{find %F -print | %{afio} -o - | %{compress} -c > %f}}
    {.afio.F	{find %F -print | %{afio} -o - | %{freeze} -c > %f}}
    {.afio.Y	{find %F -print | %{afio} -o - | %{yabba} > %f}}
    {.afio.gz	{find %F -print | %{afio} -o - | %{gzip} -9c > %f}}
    {.afio.zip	{find %F -print | %{afio} -o - | %{zip} -9q > %f}}
    {.afio	{find %F -print | %{afio} -o %f}}
    {.Z		{%{compress} %F}}
    {.F		{%{freeze} %F}}
    {.Y		{%{yabba} < %F > %f}}
    {.gz	{%{gzip} -9 %F}}
    {.zip	{%{zip} -9qr %f %F}}
    {.lzh	{%{lharc} a %f %F}}
    {.arc	{%{arc} a %f %F}}
    {.zoo	{%{zoo} a %f %F}}
}

    # These commands are for _extracting_ archives.  The unarchiver
    # will be run in the %F file's directory, and %F will be replaced
    # with the basename.
set unarchivers {
    {.compressed	{%{compress} -dc < %f | %{tar} -xf -}}
    {.tar.Z	{%{compress} -dc %f | %{tar} -xf -}}
    {.tar.F	{%{freeze} -dc %f | %{tar} -xf -}}
    {.tar.Y	{%{unyabba} < %f | %{tar} -xf -}}
    {.tar.z	{%{gzip} -dc %f | %{tar} -xf -}}
    {.tar-z	{%{gzip} -dc %f | %{tar} -xf -}}
    {.tar.gz	{%{gzip} -dc %f | %{tar} -xf -}}
    {.tar.zip	{%{gzip} -dc %f | %{tar} -xf -}}
    {.tar.lzh	{%{lharc} pq %f | %{tar} -xf -}}
    {.tar	{%{tar} -xf %f}}
    {.afio.Z	{%{compress} -dc %f | %{afio} -i -}}
    {.afio.F	{%{freeze} -dc %f | %{afio} -i -}}
    {.afio.Y	{%{unyabba} < %f | %{afio} -i -}}
    {.afio.z	{%{gzip} -dc %f | %{afio} -i -}}
    {.afio.gz	{%{gzip} -dc %f | %{afio} -i -}}
    {.afio.zip	{%{gzip} -dc %f | %{afio} -i -}}
    {.afio.lzh	{%{lharc} pq %f | %{afio} -i -}}
    {.afio	{%{afio} -i %f}}
    {.Z		{%{compress} -d %f}}
    {.F		{%{freeze} -d %f}}
    {.Y		{%{unyabba} < %f > %F}}
    {.z		{%{gzip} -d %f}}
    {.gz	{%{gzip} -d %f}}
    {.zip	{%{unzip} %f}}
    {.lzh	{%{lharc} xf %f}}
    {.arc	{%{arc} ox %f}}
    {.zoo	{%{zoo} x %f}}
}

    # These commands are for _listing_ archives.
set listers {
    {.compressed	{%{compress} -dc < %f | %{tar} -tvf -}}
    {.tar.Z	{%{compress} -dc %f | %{tar} -tvf -}}
    {.tar.F	{%{freeze} -dc %f | %{tar} -tvf -}}
    {.tar.Y	{%{unyabba} < %f | %{tar} -tvf -}}
    {.tar.z	{%{gzip} -dqc %f | %{tar} -tvf -}}
    {.tar-z	{%{gzip} -dqc %f | %{tar} -tvf -}}
    {.tar.gz	{%{gzip} -dqc %f | %{tar} -tvf -}}
    {.tar.zip	{%{gzip} -dc %f | %{tar} -tvf -}}
    {.tar.lzh	{%{lharc} pq %f | %{tar} -tvf -}}
    {.tar	{%{tar} -tvf %f}}
    {.afio.Z	{%{compress} -dc %f | %{afio} -t -v -}}
    {.afio.F	{%{freeze} -dc %f | %{afio} -t -v -}}
    {.afio.Y	{%{unyabba} < %f | %{afio} -t -v -}}
    {.afio.z	{%{gzip} -dc %f | %{afio} -t -v -}}
    {.afio.gz	{%{gzip} -dc %f | %{afio} -t -v -}}
    {.afio.zip	{%{gzip} -dc %f | %{afio} -t -v -}}
    {.afio.lzh	{%{lharc} pq %f | %{afio} -t -v -}}
    {.afio	{%{afio} -t -v %f}}
    {.Z		{echo %F}}
    {.F		{echo %F}}
    {.Y		{echo %F}}
    {.z		{echo %F}}
    {.gz	{echo %F}}
    {.zip	{%{unzip} -l %f}}
    {.lzh	{%{lharc} v %f}}
    {.arc	{%{arc} ol %f}}
    {.zoo	{%{zoo} l %f}}
}


    # Choose an extension type, caching the value in the cacheName
    # variable and default.
proc chooseext {cacheName} {
    global $cacheName archivers

	# Build a list of extension possibilities.
    set exts {}
    foreach archiver $archivers {
	if {![string match \"#*\" $archiver]} {
	    lappend exts [lindex $archiver 0]
	}
    }
    
	# Figure out the default extension type.
    if {![info exists $cacheName]} {
	set $cacheName [defaults read TickleServices $cacheName \"\"]
	if {[set $cacheName]==\"\"} {
	    set $cacheName \".tar.Z\"
	}
    }
    
	# Ask the user to choose an extension type.
    set ext [listpanel -sort -select [set $cacheName] {Select Archive\
 Type} $exts]
    
	# If it's changed, cache it for next time.
    if {$ext!=[set $cacheName]} {
	set $cacheName $ext
	defaults write TickleServices $cacheName $ext
    }
    return $ext
}

    # Replace the %-expansions in archival commands.
proc substitutions {cmd fullname filename} {
    regsub -all \"%f\" $cmd \\\"$fullname\\\" cmd
    regsub -all \"%F\" $cmd \\\"$filename\\\" cmd
    
	# Find each %{} sequence and expand it.
    while {[regexp \"%{(\\[^{}\\]*)}\" $cmd barf cmdName]} {
	set cmdPath [findcmd $cmdName]
	if {$cmdPath==\"\"} {
	    error $cmdName
	} else {
	    regsub -all \"%{$cmdName}\" $cmd \\\"$cmdPath\\\" cmd
	}
    }
    return $cmd
}
    # Archive the filenames into ext-style archives.  Alert panels
    # are titled with alertTitle.
proc doarchive {ext alertTitle} {
    global archivers
	# Find the commands for the extension.
    set which [lsearch $archivers \"$ext\\[ \\t\\n\\]*\"]
    if {$which==-1} {
	    # Let the user know if there weren't any commands!
	alertpanel $alertTitle \"Couldn't find create commands for `$ext'!\"
	return
    }
    set cmd [lindex [lindex $archivers $which] 1]
    
	# Hit the filenames and archive them.
    foreach filename [filenames] {
	    # Build the archive form of the filename.
	set fullname $filename$ext
    
	    # Back it up if it exists.
	if {[file exists $fullname]} {
	    catch {exec mv $fullname $fullname~}
	}
	
	    # Replace the %-expansions.
	if {[catch {substitutions $cmd [file tail $fullname] [file tail $fil\
ename]} acmd]} {
	    if {[alertpanel $alertTitle \"Unable to find command `$acmd', sk\
ipping `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
		return
	    }
	    continue
	}
	set pwd [pwd]
	cd [file dirname $filename]
	set ret [catch [concat exec $acmd] res]
	cd $pwd
	if {$ret} {
	    if {[alertpanel $alertTitle \"Error while creating `[file tail $\
filename]$ext':  $res\" \"Continue\" \"Cancel\"]<1} {
		return
	    }
	}
    }
}
";
"Window Size" = "{ 718.000, 374.000}";
|
"Menu Item" = "Archive/Archive";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Create an archive of a fixed type, specified by the ArchiveDefaultE\
xt
# extension.  This is the type you \"normally\" want to use.

if {![info exists ArchiveDefaultExt]} {
    set ext [defaults read TickleServices ArchiveDefaultExt \"\"]
    if {$ext==\"\"} {
	if {[alertpanel {Create Archive} \"No default archive extension.  Ch\
oose one?\" \"Choose\" \"Cancel\"]<1} {
	    return
	}
	set ext [chooseext ArchiveDefaultExt]
    }
} else {
    set ext $ArchiveDefaultExt
}
if {$ext!=\"\"} {
    doarchive $ext {Create Archive}
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Archive As...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Create archives out of the selected files.

set ext [chooseext ArchiveCreateAsExt]
if {$ext!=\"\"} {
    doarchive $ext {Create Archive As}
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Archive To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Create archives out of the selected files.

set ext [chooseext ArchiveCreateAsExt]
if {$ext==\"\"} {
    return
}
set target [dirpanel -title {Archive To} -prompt Directory: -return]

    # Find the commands for the extension.
set which [lsearch $archivers \"$ext\\[ \\t\\n\\]*\"]
if {$which==-1} {
	# Let the user know if there weren't any commands!
    alertpanel {Archive To} \"Couldn't find create commands for `$ext\
'!\"
    return
}
set cmd [lindex [lindex $archivers $which] 1]

    # Hit the filenames and archive them.
foreach filename [filenames] {
	# Build the archive form of the filename.
    set fullname $target/[file tail $filename]$ext

	# Back it up if it exists.
    if {[file exists $fullname]} {
	catch {exec mv $fullname $fullname~}
    }
    
	# Replace the %-expansions.
    if {[catch {substitutions $cmd $fullname [file tail $filename]} a\
cmd]} {
	if {[alertpanel {Archive To} \"Unable to find command `$acmd', skipp\
ing `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
	    return
	}
	continue
    }
    set pwd [pwd]
    cd [file dirname $filename]
    set ret [catch [concat exec $acmd] res]
    cd $pwd
    if {$ret} {
	if {[alertpanel {Archive To} \"Error while creating `[file tail $ful\
lname]':  $res\" \"Continue\" \"Cancel\"]<1} {
	    return
	}
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Extract";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Unbundle the selected archive files into original files and
# directories.

foreach filename [filenames] {
    set flag 0
    foreach unarchiver $unarchivers {
	set ext [lindex $unarchiver 0]
	if {[string match \"*$ext\" $filename]} {
	    set flag 1
	    set cmd [lindex $unarchiver 1]
		# Parse the non-ext part from the filename.
	    set basename [string range $filename 0 [expr [string last $ext $\
filename]-1]]
	    
		# Replace the %-expansions.
	    if {[catch {substitutions $cmd [file tail $filename] [file tail \
$basename]} acmd]} {
		if {[alertpanel {Extract Archive} \"Unable to find command `$acmd',\
 skipping `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
		    return
		}
		break
	    }

	    set pwd [pwd]
	    cd [file dirname $filename]
	    set ret [catch [concat exec $acmd] res]
	    cd $pwd
	    if {$ret} {
		alertpanel {Extract Archive} \"Error while extracting `[file tail $\
filename]':  $res\"
	    } elseif {[file isdirectory $basename]} {
		exec open $basename
	    } elseif {[file exists $basename]} {
		if {[alertpanel {Extract Archive} \"Open `[file tail $basename]'?\"\
 \"Open\" \"Continue\"]==1} {
		    exec open $basename
		}
	    } else {
		alertpanel {Extract Archive} \"Unable to find `[file tail $basename\
]' to open.\"
	    }
	    break
	}
    }
    if {!$flag} {
	alertpanel {Extract Archive} \"Unable to find extract commands for `\
[file tail $filename]'.\"
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Extract As...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Unbundle the selected archive files according to a user-specified
# extension.

set ext [chooseext ArchiveExtractAsExt]
if {$ext==\"\"} {
    return
}
set which [lsearch $unarchivers \"$ext\\[ \\t\\n\\]*\"]
if {$which==-1} {
	# Let the user know if there weren't any commands!
    alertpanel {Extract As} \"Couldn't find extract commands for `$ex\
t'!\"
    return
}
set cmd [lindex [lindex $unarchivers $which] 1]

foreach filename [filenames] {
    if {![string match *$ext $filename]} {
	case [alertpanel {Extract As} \"`[file tail $filename]' does not app\
ear to be a $ext archive.\" \"Skip\" \"Extract Anyhow\" \"Cancel\"] {
	    1	{continue}
	    -1	{return}
	}
	set basename [file rootname $filename]
    } else {
	    # Parse the non-ext part from the filename.
	set basename [string range $filename 0 [expr [string last $ext $file\
name]-1]]
    }
    
	# Replace the %-expansions.
    if {[catch {substitutions $cmd [file tail $filename] [file tail $\
basename]} acmd]} {
	alertpanel {Extract Archive} \"Unable to find command `$acmd', skipp\
ing `[file tail $filename]'\"
	break
    }

    set pwd [pwd]
    cd [file dirname $filename]
    set ret [catch [concat exec $acmd] res]
    cd $pwd
    if {$ret} {
	alertpanel {Extract Archive} \"Error while extracting `[file tail $f\
ilename]':  $res\"
    } elseif {[file isdirectory $basename]} {
	exec open $basename
    } elseif {[file exists $basename]} {
	if {[alertpanel {Extract Archive} \"Open `[file tail $basename]'?\" \
\"Open\" \"Continue\"]==1} {
	    exec open $basename
	}
    } else {
	alertpanel {Extract Archive} \"Unable to find `[file tail $basename]\
' to open.\"
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Extract In Directory";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Unbundle the selected archive files into original files and
# directories.  This creates a new directory for the extracted files.

foreach filename [filenames] {
    set flag 0
    foreach unarchiver $unarchivers {
	set ext [lindex $unarchiver 0]
	if {[string match \"*$ext\" $filename]} {
	    set flag 1
	    set cmd [lindex $unarchiver 1]
		# Parse the non-ext part from the filename.
	    set basename [string range $filename 0 [expr [string last $ext $\
filename]-1]]
	    
	    if {[catch {substitutions $cmd ../[file tail $filename] [file ta\
il $basename]} acmd]} {
		if {[alertpanel {Extract Archive} \"Unable to find command `$acmd',\
 skipping `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
		    return
		}
		break
	    }

	    set pwd [pwd]
	    exec mkdirs $basename
	    cd $basename
	    set ret [catch [concat exec $acmd] res]
	    cd $pwd
	    if {$ret} {
		alertpanel {Extract Archive} \"Error while extracting `[file tail $\
filename]':  $res\"
	    } else {
		exec open $basename
	    }
	    break
	}
    }
    case $flag {
	0	{
	    alertpanel {Extract Archive} \"Unable to find extract commands f\
or `[file tail $filename]'.\"
	}
	2	{}
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Extract To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Unbundle the selected archive files into original files and
# directories.  The user is prompted for a directory to extract
# them into.

set target [dirpanel -title {Extract To} -prompt Directory: -return]

foreach filename [filenames] {
    set flag 0
    foreach unarchiver $unarchivers {
	set ext [lindex $unarchiver 0]
	if {[string match \"*$ext\" $filename]} {
	    set flag 1
	    set cmd [lindex $unarchiver 1]
		# Parse the non-ext part from the filename.
	    set basename [string range $filename 0 [expr [string last $ext $\
filename]-1]]
	    
	    if {[catch {substitutions $cmd $filename [file tail $basename]} \
acmd]} {
		if {[alertpanel {Extract Archive} \"Unable to find command `$acmd',\
 skipping `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
		    return
		}
		break
	    }

	    set pwd [pwd]
	    cd $target
	    set ret [catch [concat exec $acmd] res]
	    cd $pwd
	    if {$ret} {
		alertpanel {Extract Archive} \"Error while extracting `[file tail $\
filename]':  $res\"
	    } else {
		exec open $target/[file tail $basename]
	    }
	    break
	}
    }
    case $flag {
	0	{
	    alertpanel {Extract Archive} \"Unable to find extract commands f\
or `[file tail $filename]'.\"
	}
	2	{}
    }
}
";
"Timeout" = "60000";
|
"Key Equivalent" = "Z";
"Menu Item" = "Archive/Gzip";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Generic gzip service
# (original from David Meyer <dmm0t@rincewind.mech.virginia.edu>)
# (Modifications from Scott Hess <scott@nic.gac.edu>)
#  If a directory is selected, tar and gzip it
#  If a .tar.[zZ] or .tar.gz is selected, gunzip and untar it.
#  If a .compressed is selected, gunzip and untar it, and delete .com\
pressed file
#      (same behavior as Workspace)
#  if a .[zZ] or .gz is selected, gunzip it
#  otherwise, gzip whatever is selected.

    # Make certain the target directory is writable.
set filenames [filenames]
set dirname [file dirname [lindex $filenames 0]]
if {![file writable $dirname]} {
    alertpanel \"Gzip\" \"`$dirname' is not writable.\"
    return
}

    # Locate critical executables
foreach prog \"gzip tar\" {
    if {[findcmd $prog]==\"\"} {
	return
    }
}

foreach file $filenames {
    set fname [file tail $file]
    case $file {
	{*.tar.Z *.tar.z *.tar.gz *.compressed} {
	    set pwd [pwd]
	    cd $dirname
	    if {[catch {exec [findcmd gzip] -dc $fname | [findcmd tar] -xf -\
} msg]!=0} {
		set savedInfo $errorInfo
		cd $pwd
		if {$msg==\"\\n gzip: stdout: Broken pipe\"} {
		    error $msg $savedInfo
		}
	    }
	    cd $pwd
	    WSNotify
	    if {[string match *.compressed $file]} {
		exec [findcmd rm] -f $file
	    }
	    WSNotify
	}
	{*.Z *.z *.gz} {
	    exec [findcmd gzip] -d $file
	    WSNotify
	}
	default {
	    if {[file isdirectory $file]} {
		set pwd [pwd]
		cd $dirname
		if {[catch {exec [findcmd tar] -cf - $fname | [findcmd gzip] -c > $\
fname.tar.gz} msg]!=0} {
		    set savedInfo $errorInfo
		    catch {exec [findcmd rm] -rf $fname.tar.gz}
		    cd $pwd
		}
		cd $pwd
		WSNotify
		if {[alertpanel \"Gzip\" \"Created `$fname.tar.gz', remove `$file'?\
\" \"Remove\" \"Don't Remove\"]==1} {
		    exec [findcmd rm] -rf $file
		}
		WSNotify
	    } else {
		exec [findcmd gzip] $file
		WSNotify
	    }
	}
    }
}

WSNotify
";
|
"Menu Item" = "Archive/List";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# List the files in the selected archive.

foreach filename [filenames] {
    set flag 0
    foreach lister $listers {
	set ext [lindex $lister 0]
	if {[string match \"*$ext\" $filename]} {
	    set flag 1
	    set cmd [lindex $lister 1]
		# Parse the non-ext part from the filename.
	    set basename [string range $filename 0 [expr [string last $ext $\
filename]-1]]
	    
	    if {[catch {substitutions $cmd [file tail $filename] [file tail \
$basename]} acmd]} {
		if {[alertpanel {List Archive} \"Unable to find command `$acmd', sk\
ipping `[file tail $filename]'\" \"Continue\" \"Cancel\"]<1} {
		    return
		}
		break
	    }

	    set pwd [pwd]
	    cd [file dirname $filename]
	    catch [concat exec $acmd] res
	    cd $pwd
	    exec open << $res &
	    break
	}
    }
    case $flag {
	0	{
	    alertpanel {List Archive} \"Unable to find list commands for `[f\
ile tail $filename]'.\"
	    # This is a nice addition, but can be very painful if
	    # a large directory was selected.
	    #set pwd [pwd]
	    #cd [file dirname $filename]
	    #catch \"exec find [file tail $filename] -ls\" res
	    #exec open << $res &
	    #cd $pwd
	}
	2	{}
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Archive/Set Default...";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXFontPboardType";
"Send Type" = "NXPostScriptPboardType";
"Send Type" = "NXRTFPboardType";
"Send Type" = "NXRulerPboardType";
"Send Type" = "NXTIFFPboardType";
"Send Type" = "NXTabularTextPboardType";
"Tcl" = "\
# Create an archive of a fixed type, specified by extension.

chooseext ArchiveDefaultExt
";
"Timeout" = "60000";
# End TickleServices Data
