# Begin TickleServices Version 1.02mab Data
"Divider" = "173.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # This version of exec doesn't strip the trailing newline from th\
e
    # output.
proc nexec {args} {
    set fileId [uplevel open [list \"|$args\"]]
    set ret [read $fileId]
    close $fileId
    return $ret
}

    # Reads the specified file and returns the text.
proc readfile {filename {hexcode 0}} {
    set fileId [open $filename r]
    if {$hexcode} {
        set ret [read -hexcode $fileId]
    } else {
        set ret [read $fileId]
    }
    close $fileId
    return $ret
}

    # Writes the text to the specified file.
proc writefile {filename text {hexcode 0}} {
    set fileId [open $filename w]
    if {$hexcode} {
        set ret [puts -nonewline -hexcode $fileId $text]
    } else {
        set ret [puts -nonewline $fileId $text]
    }
    close $fileId
    return $ret
}

    # Splits multi-file Workspace-style lists into TCL-style lists.
proc filenames {} {
    global NXFilenamePboardType
    
	# If there is only one filename, make certain it's a list.
	# Otherwise, split on \\t.
    set filenames [pasteboard read $NXFilenamePboardType]
    if {[string first \"\\t\" $filenames]!=-1} {
	set filenames [split $filenames \"\\t\"]
    } else {
	set filenames [list $filenames]
    }
    return $filenames
}

    # Attempt to find the cmdName command and return the path to
    # it.  It is cached in the global Command$cmdName variable for
    # future uses and in the default TickleServices Command$cmdName
    # for future runs.
proc findcmd {cmdName} {
    set varname Command$cmdName
    global $varname env
    
	# I have to say [set $varname] to get the value of $varname,
	# because $$varname won't do it (that attempts to get the
	# value of the variable \"$varname\" instead).  This works
	# because the $varname substitution happens _before_ set
	# is called.
    if {[info exists $varname] && [file executable [set $varname]]} {
	return [set $varname]
    }

	# Get location from defaults.
    set cmd [defaults read TickleServices $varname \"\"]
    
	# If it's not set or not there, try to find it.
    if {$cmd==\"\" || ![file executable $cmd]} {
	set cmd \"\"
	
	    # Attempt to find the user's path.  This is so bizarre
	    # because you want to get the PATH from the user's
	    # command-line shell, note TickleServer's PATH, which
	    # was inherited from Workspace and is not sufficient
	    # to our needs.
	if {[info exists env(SHELL)]} {
	    set shell $env(SHELL)
	} else {
	    set shell /bin/csh
	}
	    # This has only been verified to work with these six
	    # shells.  If the user isn't using one of them, try
	    # csh as a last resort under the assumption that their
	    # shell won't work, or might even do something discourteous
	    # like never exit.  Let me know if other shells work.
	if {[lsearch [file tail $shell] {bash csh rc sh tcsh zsh}]==-1} {
	    set shell /bin/csh
	}
	    # The @/: crud is to differentiate the echo'ed output
	    # from anything the shell puts out in the course of
	    # starting up.  In other words, it is a hack.
	if {![catch {exec $shell -c \"echo \\\"@@:@@:@@\\$PATH@:@@@@:@\\\"\"\
} path]} {
	    if {[regexp \"@@:@@:@@(\\[^#\\]*)@:@@@@:@\" $path barf path]} {
		set path [split $path \":\"]
		foreach dir $path {
		    if {[file executable $dir/$cmdName]} {
			set cmd $dir/$cmdName
			break
		    }
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices.app
	    # wrapper.
	if {$cmd==\"\"} {
	    set apppaths [defaults read Workspace ApplicationPaths {}]
	    if {$apppaths==\"\"} {
		set apppaths {
		    ~/Apps
		    /LocalApps
		    /NextApps
		    /NextDeveloper/Apps
		    /NextAdmin
		    /NextDeveloper/Demos
		}
	    } else {
		set apppaths [split $apppaths \":\"]
	    }
	    foreach path $apppaths {
		if {[file executable $path/TickleServices.app/$cmdName]} {
		    set cmd $path/TickleServices.app/$cmdName
		    break
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices
	    # ServicesPaths directories.
	if {$cmd==\"\"} {
	    set servicespaths [defaults read TickleServices ServicesPaths {}\
]
	    if {$servicespaths==\"\"} {
		set servicespaths {
		    ~/Library/TickleServices
		    /LocalLibrary/TickleServices
		}
	    } else {
		set servicespaths [split $servicespaths \":\"]
	    }
	    foreach path $servicespaths {
		if {[file executable $path/$cmdName]} {
		    set cmd $path/$cmdName
		    break
		}
	    }
	}
	
	    # If we've found something, ask the user for verification.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    case [alertpanel \"Find $cmdName executable\" \"Use `$cmd' for $\
cmdName?\" \"OK\" \"Choose\" \"Cancel\"] {
		0	{set cmd \"\"}
		-1	return
	    }
	}
	
	    # If we haven't found it yet, ask the user about it.
	if {$cmd==\"\" || ![file executable $cmd]} {
	    set cmd [passargs openpanel {
		-title \"Select $cmdName executable\"
		-prompt \"Name:\"
	    }]
	}
	
	    # If we've got it, write it to the defaults database.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    defaults write TickleServices $varname $cmd
	} else {
	    return
	}
    }
    set $varname $cmd
    return $cmd
}

# Note to those who like RCSINIT - you can do something like:
#     set env(RCSINIT) \"-q\"
# and that will be in the environment of commands exec'ed in the futu\
re.

    # Check out an RCS file.  Return \"NoRCSFile\" if there is no RCS
    # file to checkout.  Return \"Writable\" if there is already a wr\
itable
    # working file.  Return the error output if there is an error.
    # Return \"\" if everything goes well.
proc RCSco {filename {force 0}} {
	# If the file ends with \",v\" it's an RCS file.
    if {[regexp \"(.*),v$\" $filename foo base]} {
	set filename $base

	    # Remove RCS/ element from path, if present.
	if {[regsub \"(.*/)RCS/(\\[^/\\]*)\" $filename \"\\\\1\\\\2\" base]}\
 {
	    set filename $base
	}
    } else {
	    # Otherwise, it's a working file, so check to see if it's
	    # got an RCS file.
	set rcsd \"[file dirname $filename]/RCS/[file tail $filename],v\"
        if {![file exists $filename,v] && ![file exists $rcsd]} {
	    return [list NoRCSFile $filename]
        }
    }
    
	# We can catch writable working files here, rather than letting
	# RCS do it, and then force things if need be.
    if {!$force && [file writable $filename]} {
	return [list Writable $filename]
    } elseif {$force} {
        set result [catch {exec [findcmd co] -l -f $filename} output]
    } else {
        set result [catch {exec [findcmd co] -l $filename} output]
    }
    
	# If there was an error while checking out, let the user know.
    if {$result} {
	return [list $output $filename]
    }

    return [list {} $filename]
}

    # Pass \"arguments\" as arguments to \"command\".  The arguments
    # are reformed into a list not containing newlines, since
    # otherwise newlines would separate commands.  Thus, passargs
    # can be used as a convenience to make scripts easier to read.
proc passargs {command arguments} {
    return [uplevel $command [join [split $arguments]]]
}

    # This is the core of the \"Copy To\", \"Link To\", and \"Move To\
\"
    # services.  It's just a convenience because I found that they
    # all were looking pretty much identical.
proc __doto {title cmd ing} {
	# Get the list of filenames.
    set filenames [filenames]
    
	# If it's a multiple selection, get a directory to bulk
	# transfer to.
    if {[llength $filenames]>1} {
	set ret 0
	
	    # Let the user choose a directory to operate from.
	set target [passargs dirpanel {
	    -directory [file dirname [lindex $filenames 0]]
	    -title \"$title Target Directory\"
	    -prompt {Name:}
	    -return
	}]
	if {$target==\"\"} {
	    return
	}
	if {![file isdirectory $target]} {
	    alertpanel \"$title To\" \"$target is not a directory.\"
	    return
	}
    } else {
	    # Get a target to copy to.
	set target [passargs savepanel {
	    -directory [file dirname $filenames]
	    -title \"$title Target\"
	    -prompt {Name:}
	    -file [file tail $filenames]
	    -type {}
	    -return
	}]
	if {$target==\"\"} {
	    return
	}
	
	    # Note that if savepanel worked, then the user verified that
	    # they wished to remove any previous file.
	if {[file exists $target]} {
	    if {[catch {exec rm -f $target} output]} {
		alertpanel \"$title To\" \"Unable to remove $target: $output\"
		return
	    }
	}
	
	    # Put it in a list so that the concat doesn't pull it
	    # apart (consider filenames that contain spaces).
	set filenames [list $filenames]
    }
    if {[catch [concat exec $cmd $filenames [list $target]] output]} \
{
	alertpanel \"$title To\" \"Error $ing to $target: $output\"
    }
}
";
"Window Size" = "{ 690.000, 404.000}";
|
"Menu Item" = "File/Check Out";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Check out files using RCS co.

    # Get the line of files.
set filenames [filenames]

foreach filename $filenames {
    set res [RCSco $filename]
    set tail [file tail [lindex $res 1]]
    case [lindex $res 0] {
	NoRCSFile	{
	    alertpanel {Check Out} \"No revisions for `$tail'\"
	    continue
	}
	Writable	{
	    set msg \"Writable file `$tail' already exists.  Overwrite?\"
	    case [alertpanel {Check Out} $msg {Overwrite} {Skip} {Cancel}] {
		-1	{return}
		0	{continue}
		1	{
		    set res [RCSco $filename 1]
		    case [lindex $res 0] {
			{}	{continue}
			default	{}
		    }
		}
	    }
	}
	{}	{continue}
	default		{}
    }
    if {[alertpanel {Check Out} \"Error checking out `$tail': [lindex\
 $res 0]\" Continue Cancel]!=1} {
	return
    }
}
";
|
"Key Equivalent" = "^";
"Menu Item" = "File/Check Out and Open";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Check out files using RCS co and then open them.  This is useful in
# two cases.  One is when you check the file out from the Workspace,
# as a sort of shorthand.  The other is when you have the file in Edi\
t.
# I've noticed that if you have an unmodified file in Edit and open i\
t
# again, it reloads it.  This means that it won't complain about
# inconsistant modify times if you opened the file before checking it
# out.

    # Get the line of files.
set filenames [filenames]

foreach filename $filenames {
    set res [RCSco $filename]
    set tail [file tail [lindex $res 1]]
    case [lindex $res 0] {
	NoRCSFile	{
	    alertpanel {Check Out} \"No revisions for `$tail'\"
	    continue
	}
	Writable	{
	    set msg \"Writable file `$tail' already exists.  Overwrite?\"
	    case [alertpanel {Check Out} $msg {Overwrite} {Skip} {Cancel}] {
		-1	{return}
		0	{continue}
		1	{
		    set res [RCSco $filename 1]
		    case [lindex $res 0] {
			{}	{
			    if {[catch {exec /usr/bin/open [lindex $res 1]} output]} {
				alertpanel {Check Out and Open} \"Error opening `$tail': $output\"\
 OK
			    }
			    continue
			}
			default	{}
		    }
		}
	    }
	}
	{}	{
	    if {[catch {exec /usr/bin/open [lindex $res 1]} output]} {
		alertpanel {Check Out and Open} \"Error opening `$tail': $output\" \
OK
	    }
	    continue
	}
	default		{}
    }
    if {[alertpanel {Check Out} \"Error checking out `$tail': [lindex\
 $res 0]\" Continue Cancel]!=1} {
	return
    }
}
";
|
"Menu Item" = "File/Clean Directory";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Clean the *~ files from the selection.

# If there are multiple filenames, clean each directory.
# If there is only one and it is a file, clean its _directory_.

set filenames [filenames]

    # If there is only a single selection, use its directory if
    # the file is not already a directory.
if {[llength $filenames]==1} {
    if {![file isdirectory $filenames]} {
        set filenames [file dirname $filenames]
    }
}
foreach filename $filenames {
    if {[file isdirectory $filename]} {
	    # Use $filename/. to force soft links to be resolved,
	    # since find won't recurse into them.
	set files [exec find $filename/. -name *~ -print]
	    # Split into filenames along newlines.
	foreach file [split $files \"\\n\"] {
	    exec rm -r $file
	}
    } elseif {[string match \"*~\" $filename]} {
	exec rm -r $filename
    }
}
";
|
"Menu Item" = "File/Copy Filename";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Copy the selected filename to the selection pasteboard.

pasteboard -main write [pasteboard read $NXFilenamePboardType]
";
|
"Menu Item" = "File/Copy To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# If given a single filename, prompts user for a new name and copies
# to there.  If given multiple filenames, prompts user to choose a
# directory and bulk-copies all of the selected files to the director\
y.

__doto Copy {cp -r} copying
";
"Timeout" = "60000";
|
"Key Equivalent" = "g";
"Menu Item" = "File/Group In Directory...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Group the selected files into a subdirectory.

set filenames [filenames]

    # Get the name of the directory using a promptpanel.  I use a
    # promptpanel because dirpanel isn't as slick as it could be
    # when it comes to selecting non-existant directories.
set newdir [promptpanel {Name For New Directory}]

    # If it's empty, dump out (note that a Cancel on the panel will
    # return, too - this is just in case the user didn't enter
    # anything).
if {[string trim $newdir]==\"\"} {
    return
}

    # Get the directory the files are in.
set dirname [file dirname [lindex $filenames 0]]

    # Get the full path of the new directory.
set newdirname $dirname/$newdir

    # If the target directory name is already used, let the user
    # know.
if {[file exists $newdirname]} {
    alertpanel {Group In Directory} \"`$newdir' already exists.\"
    return
}

    # Create the target directory.
if {[catch {exec mkdirs $newdirname} res]} {
    alertpanel {Group In Directory} \"Error creating `$newdir', $res\"
    return
}

    # Move the files into the target.  Use -f to force it (assume
    # the user means it).
if {[catch [concat exec mv -f $filenames [list $newdirname]] res]} {
	# Try moving any partially-moved files back.
    if {![catch {glob $newdirname/*} movedfiles]} {
	set movedfiles [glob $newdirname/*]
	if {[llength $movedfiles]} {
	    catch [concat exec mv -f $movedfiles $dirname]
	}
    }
    
	# Try to remove the new directory.  This is complicated by
	# the fact that the new directory might actually be several
	# levels deep if the user entered slashes in the name.
    set dirnamelen [string length $dirname]
    while {[string length $newdirname]>$dirnamelen} {
	if {[catch {rmdir $newdirname}]} {
	    break;
	}
	set newdirname [file dirname $newdirname]
    }

    alertpanel {Group In Directory} \"Error moving files to `$newdir'\
, $res\"
    return
}
";
"Timeout" = "60000";
|
"Menu Item" = "File/Hard Link To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# If given a single filename, prompts user for a new name and hard
# links to there.  If given multiple filenames, prompts user to
# choose a directory and bulk-links all of the selected files to
# the directory.

__doto {Hard Link} ln {hard linking}
";
"Timeout" = "60000";
|
"Menu Item" = "File/Link To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# If given a single filename, prompts user for a new name and softlin\
ks
# to there.  If given multiple filenames, prompts user to choose a
# directory and bulk-links all of the selected files to the directory\
.

__doto Link {ln -s} linking
";
"Timeout" = "60000";
|
"Menu Item" = "File/Move To...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# If given a single filename, prompts user for a new name and moves i\
t
# to there.  If given multiple filenames, prompts user to choose a
# directory and bulk-moves all of the selected files to the directory\
.

__doto Move mv moving
";
"Timeout" = "60000";
|
"Menu Item" = "File/Open Directory";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Get the first filename in the selection, and open its directory.

set filename [lindex [filenames] 0]
if {![file isdirectory $filename]} {
    set filename [file dirname $filename]
}
exec /usr/bin/open $filename
";
|
"Key Equivalent" = "O";
"Menu Item" = "File/Open From Directory...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Get the first filename in the selection, bring up an openpanel
# from that directory, and allow the user to open files there.

set filename [lindex [filenames] 0]
if {![file isdirectory $filename]} {
    set filename [file dirname $filename]
}
set filenames [openpanel -multiple -directory $filename -return]
exec /usr/bin/open $filenames
";
"Timeout" = "60000";
|
"Menu Item" = "File/Open Glob...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Prompt the user for a shell-style regular expression, and open
# matching files from the selected directory.
#
# Scott Hess
# Monday June 7, 1993
# Use this code as you will

set filename [lindex [filenames] 0]
if {![file isdirectory $filename]} {
    set filename [file dirname $filename]
}
set files {}
foreach file [promptpanel \"Open from `[file tail $filename]'\" \"*\"\
] {
    set files [concat $files [glob $filename/$file]]
}
if {$files!=\"\"} {
    eval [concat exec /usr/bin/open $files]
}
global TSInhibitReactivate
set TSInhibitReactivate {}
";
"Timeout" = "60000";
# End TickleServices Data
