# Begin TickleServices Version 1.02mab Data
"Divider" = "154.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # Convert a path of the form /bla/bla/bla/.../tail to
    # \"tail  -  /bla/bla/bla/...\"
proc path2title {path} {
    return \"[file tail $path]    [file dirname $path]\"
}

    # Convert a path back from a title.
proc title2path {title} {
    if {[regexp \"(.*)    (.*)\" $title barf tail dir]} {
	return $dir/$tail
    } else {
	error \"`$title' invalid\"
    }
}

    # Splits multi-file Workspace-style lists into TCL-style lists.
proc filenames {} {
    global NXFilenamePboardType
    
	# If there is only one filename, make certain it's a list.
	# Otherwise, split on \\t.
    set filenames [pasteboard read $NXFilenamePboardType]
    if {[string first \"\\t\" $filenames]!=-1} {
	set filenames [split $filenames \"\\t\"]
    } else {
	set filenames [list $filenames]
    }
    return $filenames
}

    # Return the architecture info for executables, object files,
    # and library archives contained in filename.
proc archInfo {filename} {
    if {![file exists /bin/lipo]} {
	return {thin m68k}
    }
    set execs [exec find $filename -type f ( -name \"*.\\[oa\\]\" -o \
-perm -100 ) -exec lipo \\{\\} -info \\;]
    set ret {}
    foreach line [split $execs \\n] {
	if {[regexp \"Architectures in the fat file: (.*) are:(( (m68k|i386|\
i486))*)\" $line barf file arch]} {
	    lappend ret [list $file fat [string trim $arch]]
	} elseif {[regexp \"Non-fat file: (.*) is architecture:( (m68k|i386|\
i486))\" $line barf file arch]} {
	    lappend ret [list $file thin [string trim $arch]]
	} elseif {[regexp \"lipo: can't figure out the architecture type of:\
 (.*)\" $line barf file]} {
	    lappend ret [list $file script {}]
	} else {
	    lappend ret [list $line unknown {}]
	}
    }
    return [lsort $ret]
}

    # Thin executables, object files, and library archives in
    # filename to only contain architectures specified by arch.
    # Returns 0 if the user decided to cancel the operation in
    # response to an alert panel, 1 otherwise.
proc thinTo {filename arch} {
    set info [archInfo $filename]
    
	# Uses this to calculate relative paths for the alert
	# panels, rather than using the absolute path.
    set dirnameLen [string length [file dirname $filename]/]
    foreach i $info {
	if {[lindex $i 1]==\"fat\" && [lsearch [lindex $i 2] $arch]!=-1} {
	    exec lipo [lindex $i 0] -thin $arch -output [lindex $i 0]
	} elseif {[lsearch [lindex $i 2] $arch]==-1 && ([lindex $i 1]==\"fat\
\" || [lindex $i 1]==\"thin\")} {
	    set relative [string range [lindex $i 0] $dirnameLen end]
	    set msg \"No $arch architecture type in [lindex $i 1] `$relative\
'.\"
	    if {[alertpanel \"Thin To $arch\" $msg \"Skip File\" \"Cancel Th\
in\"]==0} {
		return 0
	    }
	}
    }
    return 1
}

    # Figure out the default architecture for the machine we are
    # running on.
if {[file executable /usr/bin/arch]} {
    set hostArch [exec /usr/bin/arch]
} else {
    set hostArch m68k
}
";
"Window Size" = "{ 679.000, 390.000}";
|
"Menu Item" = "Lipo/Edit Files...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Give the user a list of files and architectures in the selected
# pathnames, and allow them to thin to an architecture or remove
# an architecture.  This is another of those services that's
# exceedingly hokey in that it could use a much better interface,
# but I'm working with what TickleServices1.01 has ...

while {1} {
    set info {}
    foreach filename [filenames] {
	set info [concat $info [archInfo $filename]]
    }
    set display {}
    foreach i $info {
	case [lindex $i 1] {
	    {fat} {
		lappend display \"fat\\[[join [split [lindex $i 2]] ,]\\] [path2tit\
le [lindex $i 0]]\"
	    }
	    {thin} {
		lappend display \"thin\\[[lindex $i 2]\\] [path2title [lindex $i 0]\
]\"
	    }
	}
    }
    set ret [listpanel -multiple -empty {Lipo Executables} $display E\
xit {Thin To...} {Remove...}]
    set fatArchs {}
    foreach elt [lindex $ret 1] {
	regexp {(thin|fat)\\[([^]]*)\\] (.*)} $elt barf type archs title
	foreach arch [split $archs ,] {
	    if {[lsearch $fatArchs $arch]==-1} {
		lappend fatArchs $arch
	    }
	}
    }
    case [lindex $ret 0] {
	-1 {
	    set index [lsearch $fatArchs $hostArch]
	    set nonHostArch [lreplace $fatArchs $index $index]
	    if {![catch {set removeArchs [listpanel -multiple -select $nonHo\
stArch -sort {Remove} $fatArchs]}]} {
		foreach elt [lindex $ret 1] {
		    regexp {(thin|fat)\\[([^]]*)\\] (.*)} $elt barf type archs titl\
e
		    set filename [title2path $title]
		    if {$type==\"fat\"} {
			set archs [split $archs ,]
			set rArchs {}
			foreach arch $removeArchs {
			    if {[lsearch $archs $arch]!=-1} {
				lappend rArchs $arch
			    }
			}
			if {[lsort $archs]==[lsort $rArchs]} {
			    if {[alertpanel {Architecture Mismatch} \"Cannot remove all ar\
chitectures from `[file tail $filename]'.\" \"Continue\" \"Cancel Rem\
ove\"]==0} {
				break
			    }
			} elseif {[llength $rArchs]>0} {
			    set command [list exec lipo $filename]
			    foreach arch $rArchs {
				lappend command -remove $arch
			    }
			    lappend command -output $filename
			    eval $command
			}
		    } elseif {[lsearch $removeArchs $archs]!=-1} {
			if {[alertpanel {Architecture Mismatch} \"`[file tail $filename]' \
is already thin.\" \"Continue\" \"Cancel Remove\"]==0} {
			    break
			}
		    }
		}
	    }
	}
	0 {
	    if {![catch {set arch [listpanel -select $hostArch -sort {Thin T\
o} $fatArchs]}]} {
		foreach elt [lindex $ret 1] {
		    regexp {(thin|fat)\\[([^]]*)\\] (.*)} $elt barf type archs titl\
e
		    set filename [title2path $title]
		    if {$type==\"fat\"} {
			set archs [split $archs ,]
			if {[lsearch $archs $arch]==-1} {
			    if {[alertpanel {Architecture Mismatch} \"Fat `[file tail $fil\
ename]' doesn't contain architecture type $arch.\" \"Continue\" \"Can\
cel Thin\"]==0} {
				break
			    }
			} else {
			    exec lipo $filename -thin $arch -output $filename
			}
		    } elseif {$archs!=$arch} {
			if {[alertpanel {Architecture Mismatch} \"Thin `[file tail $filena\
me]' is architecture type $archs.\" \"Continue\" \"Cancel Thin\"]==0}\
 {
			    break
			}
		    }
		}
	    }
	}
	1 {return}
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Lipo/Thin For Current";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Find executables, *.o, and *.a files in the selected paths and
# thin them to executables for the architecture of the machine
# TickleServices is running on.

foreach filename [filenames] {
    if {![thinTo $filename $hostArch]} {
	break
    }
}
";
|
"Menu Item" = "Lipo/Thin For HP-PA";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Find executables, *.o, and *.a files in the selected paths and
# thin them to hppa executables.

foreach filename [filenames] {
    if {![thinTo $filename \"hppa\"]} {
	break
    }
}
";
|
"Menu Item" = "Lipo/Thin For Intel";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Find executables, *.o, and *.a files in the selected paths and
# thin them to i386 executables.

foreach filename [filenames] {
    if {![thinTo $filename \"i386\"]} {
	break
    }
}
";
|
"Menu Item" = "Lipo/Thin For Motorola";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Find executables, *.o, and *.a files in the selected paths and
# thin them to 680x0 executables.

foreach filename [filenames] {
    if {![thinTo $filename \"m68k\"]} {
	break
    }
}
";
|
"Menu Item" = "Lipo/Thin For YMP";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Find executables, *.o, and *.a files in the selected paths and
# thin them to Cray YMP executables.

foreach filename [filenames] {
    if {![thinTo $filename \"ymp\"]} {
	break
    }
}
alertpanel {Thin For Cray YMP} {You just had to press it, didn't you?\
} {Hang Head in Shame}
";
# End TickleServices Data
