# Begin TickleServices Version 1.02mab Data
"Divider" = "196.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # This version of exec doesn't strip the trailing newline from th\
e
    # output.
proc nexec {args} {
    set fileId [uplevel open [list [format \"|%s\" $args]]]
    set ret [read $fileId]
    close $fileId
    return $ret
}

    # Reads the specified file and returns the text.
proc readfile {filename {hexcode 0}} {
    set fileId [open $filename r]
    if {$hexcode} {
        set ret [read -hexcode $fileId]
    } else {
        set ret [read $fileId]
    }
    close $fileId
    return $ret
}

    # Writes the text to the specified file.
proc writefile {filename text {hexcode 0}} {
    set fileId [open $filename w]
    if {$hexcode} {
        set ret [puts -nonewline -hexcode $fileId $text]
    } else {
        set ret [puts -nonewline $fileId $text]
    }
    close $fileId
    return $ret
}

    # read the quoteString from the defaults database.  The quoteStri\
ng
    # defaults to \">\", which is a semi-standard quoting character.
global quoteString
set quoteString [defaults read TickleServices QuoteString \">\"]

    # Quote the passed string using quoteString and return it.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the stuff in the line and
    # the end-of-line (optional because the last line may not have
    # one), and then replace the front of the line with the quoteStri\
ng.
    # Escape \\ and & in the quote string just in case.
proc quote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $str \"[escape $qs {\\\\\
&}]\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Strip quotes from the passed string.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the quote string at the
    # front, then the stuff in the line, and then the end-of-line
    # (optional because the last line may not have one), and replace
    # that with just the in-the-line stuff plus the end-of-line.
proc unquote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^[escape $qs {|\\\\()[]^*$?}](\\[^\\n\\]*)(\\n\
?)\" $str \"\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Reformat the passed string.
proc fmt {str {goal_length 65} {max_length 75}} {
    return [nexec /usr/ucb/fmt $goal_length $max_length << $str]
}

    # expandRTF expands graphic attachments so that the filenames
    # are absolute.  This is necessary because Edit stores attachment
    # names as relative to the .rtfd file they are in.
    # 
    # expandMailRTF is similar to expandRTF, except that it also
    # converts the NeXTGraphic RTF directives into Mail-style
    # Attachment directives.  This is necessary for NS2.1.
    # 
    # packRTF is the inverse of expandRTF.  The absolute filenames
    # in the NeXTGraphic directives are reduced to their basenames,
    # and the list of actual filenames is returned in the variables
    # specified by filenamesRef.
    # 
    # packMailRTF is to packRTF as expandMailRTF is to expandRTF.
proc expandRTF {rtf filename} {
    while {[regexp \"{(\\[^{\\]*){\\\\\\\\NeXTGraphic(\\[0-9\\]*) (\\\
[^/\\]\\[^\\n\\]*) \\\\\\\\width(\\[0-9\\]*) \\\\\\\\height(\\[0-9\\]\
*)\\n}(\\[^}\\]*)}\" $rtf directive head position attachment width he\
ight tail]} {
	set esc [escape $directive \"|*+?.^\\$\\\\\\[\\]\"]
	set rep [escape \"{$head{\\\\NeXTGraphic$position $filename/$attachm\
ent \\\\width$width \\\\height$height\\n}$tail}\" \"\\\\&\"]
	if {![regsub -all $esc $rtf $rep rtf]} {
	    error \"Unable to process RTF directive\"
	}
    }
    return $rtf
}
proc expandMailRTF {rtf filename} {
    set addin 0
    while {[regexp \"{\\[^{\\]*{\\\\\\\\NeXTGraphic(\\[0-9\\]*) (\\[^\
\\n\\]*) \\\\\\\\width\\[0-9\\]* \\\\\\\\height\\[0-9\\]*\\n}\\[^}\\]\
*}\" $rtf directive position attachment]} {
	set esc [escape $directive \"|*+?.^\\$\\\\\\[\\]\"]
		incr position $addin
	incr addin -1
	set rep [escape \"{{\\\\attachment$position $filename/$attachment\\n\
}\\n}\" \"\\\\&\"]
	if {![regsub -all $esc $rtf $rep rtf]} {
	    error \"Unable to process RTF directive\"
	}
    }
    return $rtf
}
proc packRTF {rtf filenamesRef} {
    upvar $filenamesRef filenames
    set addin 0
    while {[regexp \"{(\\[^{\\]*){\\\\\\\\NeXTGraphic(\\[0-9\\]*) (/\\\
[^\\n\\]*) \\\\\\\\width(\\[0-9\\]*) \\\\\\\\height(\\[0-9\\]*)\\n}(\\\
[^}\\]*)}\" $rtf directive head position filename width height tail]}\
 {
	set esc [escape $directive \"|*+?.^\\$\\\\\\[\\]\"]
	incr position $addin
	incr addin -1
	set rep [escape \"{$head{\\\\NeXTGraphic$position [file tail $filena\
me] \\\\width$width \\\\height$height\\n}$tail}\" \"\\\\&\"]
	if {![regsub $esc $rtf $rep rtf]} {
	    error \"Unable to process RTF directive\"
	}
	lappend filenames $filename
    }
    return $rtf
}
proc packMailRTF {rtf filenamesRef} {
    upvar $filenamesRef filenames
    set addin 0
    while {[regexp \"{\\[^{\\]*{\\\\\\\\NeXTGraphic(\\[0-9\\]*) (/\\[\
^\\n\\]*) \\\\\\\\width\\[0-9\\]* \\\\\\\\height\\[0-9\\]*\\n}\\[^}\\\
]*}\" $rtf directive position filename]} {
	set esc [escape $directive \"|*+?.^\\$\\\\\\[\\]\"]
	incr position $addin
	incr addin -1
	set rep [escape \"{{\\\\attachment$position [file tail $filename]\\n\
}\\n}\" \"\\\\&\"]
	if {![regsub $esc $rtf $rep rtf]} {
	    error \"Unable to process RTF directive\"
	}
	lappend filenames $filename
    }
    return $rtf
}


    # Determine the NeXTSTEP version.
if {![regexp \"NeXT Mach (\\[^:\\n\\]*): .*\" [exec hostinfo] ignore \
NeXTVersion]} {
    set NeXTVersion Unknown
}
    # If operating under NS2.x, substitute the NS2.1 expandMailRTF
    # for expandRTF.  Comment this section out if you don't need
    # the ability to paste the RTF into Mail but you do need it in
    # a different program that works correctly.
if {[string match \"2.?\" $NeXTVersion]} {
    rename expandRTF \"_expandRTF\"
    rename expandMailRTF expandRTF
}

    # Create a NeXTMail message body based on the RTF data in rtf.
proc createNeXTMail {rtf} {
	# Start with the location of the index file.
    set filenames \"/tmp/.TSTmpDir/index.rtf\"

	# Convert Edit-style attachments to Mail-style attachments.
    set rtf [packMailRTF $rtf filenames]
    
	# Build a tar pipeline to create the message.
    set cmd {tar -cf -}
    foreach filename $filenames {
	append cmd \" -C [file dirname $filename] [file tail $filename]\"
    }
    append cmd \" | compress | uuencode .tar.fromTickleServices.attac\
h\"
    
	# Create the index file.
    exec mkdirs /tmp/.TSTmpDir
    writefile /tmp/.TSTmpDir/index.rtf $rtf

	# Generate the uuencoded portion.
    set output [eval [concat exec $cmd]]
    
	# Nuke the temporary directory.
    catch {exec rm -rf /tmp/.TSTmpDir}
    
    return $output
}


    # This proc is used by the \"Reformat Text\" service.  It's a
    # relatively complete TCL emulation of the fmt(1) command.
set tab(0) \"        \"
set tab(1) \"       \"
set tab(2) \"      \"
set tab(3) \"     \"
set tab(4) \"    \"
set tab(5) \"   \"
set tab(6) \"  \"
set tab(7) \" \"
proc refmt {input {goal_length 65} {max_length 75}} {
    global tab
    set acc {}
    set pref {}
    set glength [expr $goal_length-1]
    
    # The regexp version was _horribly_ slow when there was no \\n
    # and the text was large.
    #set newline [regexp \"(.*)\\n$\" $input all input]
    #set newline [string match \"*\\n\" $input]
    #puts newline==$newline
    foreach line [split $input \"\\n\"] {
	while {[regexp -indices \\t $line t]} {
	    regsub \\t $line $tab([expr [lindex $t 0]%8]) line
	}
	regexp \"^(\\[ >#/\\]*)(.*)$\" $line line prefix text
	set text [string trimright $text]
	if {$pref!=$prefix || $text==\"\"} {
	    set plen [string length $pref]
	    while {[regsub \"(\\t*)        (.*)\" $pref \"\\\\1\\t\\\\2\" pr\
ef]} {}
	    incr goal_length -$plen
	    incr glength -$plen
	    incr max_length -$plen
	    while {$acc!=\"\"} {
		set rindex [string last \" \" [string range $acc 0 $glength]]
		set index [string first \" \" [string range $acc $goal_length end]]
		if {$index==-1} {
		    set index [string length $acc]
		    incr index -$goal_length
		}
		if {$rindex==-1 || ($goal_length+$index<=$max_length && $index<$goa\
l_length-$rindex)} {
		    set extent [expr $goal_length+$index]
		} else {
		    set extent [incr rindex -1]
		}
		lappend ret $pref[string trim [string range $acc 0 $extent]]
		set acc [string trim [string range $acc [incr extent] end]]
	    }
	    incr goal_length $plen
	    incr glength $plen
	    incr max_length $plen
	}
	set pref $prefix
	case $text {
	    \"\"		{
		while {[regsub \"(\\t*)        (.*)\" $pref \"\\\\1\\t\\\\2\" pref]\
} {}
		lappend ret $pref
	    }
	    \"*\\[.:!?\\]\"	{append acc $text \"  \"}
	    default	{append acc $text \" \"}
	}
    }
    if {$acc!=\"\"} {
	set plen [string length $pref]
	while {[regsub \"(\\t*)        (.*)\" $pref \"\\\\1\\t\\\\2\" pref]}\
 {}
	incr goal_length -$plen
	incr glength -$plen
	incr max_length -$plen
	while {$acc!=\"\"} {
	    set rindex [string last \" \" [string range $acc 0 $glength]]
	    set index [string first \" \" [string range $acc $goal_length en\
d]]
	    if {$index==-1} {
		set index [string length $acc]
		incr index -$goal_length
	    }
	    if {$rindex==-1 || ($goal_length+$index<=$max_length && $index<=\
$goal_length-$rindex)} {
		set extent [expr $goal_length+$index]
	    } else {
		set extent [incr rindex -1]
	    }
	    lappend ret $pref[string trim [string range $acc 0 $extent]]
	    set acc [string trim [string range $acc [incr extent] end]]
	}
    }
    #if {$newline} {
	#lappend ret {}
    #}
    return [join $ret \"\\n\"]
}

    # Return a list of users in the specified NetInfo domain and
    # parent domains.
proc _users {domain} {
	# Check to see if the domain is valid.
    if {[catch {exec /usr/bin/niutil -read $domain /}]} {
	return
    }

	# Get user list for parent domain.
    set parentUsers [_users $domain/..]
    
	# Get userlist for domain.
    if {[catch {exec /usr/bin/nidump passwd $domain} users]} {
	return $parentUsers
    }
    
	# Traverse userlist and append to parent list.
    foreach user [split $users \\n] {
	regexp {([^:]*):} $user barf user
	if {[lsearch $parentUsers $user]==-1} {
	    lappend parentUsers $user
	}
    }
    return $parentUsers
}
    # Return a list of users valid from . on up.
proc users {} {
    return [_users .]
}
";
"Window Size" = "{ 702.000, 452.000}";
|
"Menu Item" = "Mail/Deliver RTF To...";
"Send Type" = "NXRTFPboardType";
"Tcl" = "\
# Generate a list of users and prompt for which to send the selection
# to.

    # Get the cached list of names selected last time.
if {![info exists DeliverTo]} {
    if {![info exists whoami]} {
	set whoami [exec /usr/ucb/whoami]
    }
    set DeliverTo [defaults read TickleServices DeliverTo $whoami]
}

    # Ask the user who to deliver to.
set DeliverTo [listpanel -multiple -select $DeliverTo -sort {Deliver \
To} [users]]
defaults write TickleServices DeliverTo $DeliverTo

    # Separate the recipients with \", \" instead of \" \".
set msg \"To: [join [split $DeliverTo] {, }]\\n\"

    # Get a subject for the email.
set subject [promptpanel Deliver {Enter message subject} OK Cancel {N\
o Subject}]
case [lindex $subject 0] {
    1	{append msg \"Subject: [lindex $subject 1]\\n\"}
    0	{return}
}

    # Generate a NeXTMail body.
set body [createNeXTMail [pasteboard read $NXRTFPboardType]]

    # Fill in the rest of the msg.
append msg \"Next-Attachment: .tar.fromTickleServices.attach, \"
append msg \"[string length $body], 1/1, \"
append msg \"[expr [string length $body]*4/3], 0\\n\\n\"

append msg $body

eval [concat exec /usr/lib/sendmail $DeliverTo << [list $msg]]
";
"Timeout" = "60000";
|
"Menu Item" = "Mail/Deliver To...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Generate a list of users and prompt for which to send the selection
# to.

    # Get the cached list of names selected last time.
if {![info exists DeliverTo]} {
    if {![info exists whoami]} {
	set whoami [exec /usr/ucb/whoami]
    }
    set DeliverTo [defaults read TickleServices DeliverTo $whoami]
}

    # Ask the user who to deliver to.
set DeliverTo [listpanel -multiple -select $DeliverTo -sort {Deliver \
To} [users]]
defaults write TickleServices DeliverTo $DeliverTo

    # Separate the recipients with \", \" instead of \" \".
set msg \"To: [join [split $DeliverTo] \", \"]\\n\"

    # Get a subject for the email.
set subject [promptpanel Deliver {Enter message subject} OK Cancel {N\
o Subject}]
case [lindex $subject 0] {
    1	{append msg \"Subject: [lindex $subject 1]\\n\"}
    0	{return}
}

append msg \\n[pasteboard read]
eval [concat exec /usr/lib/sendmail $DeliverTo << [list $msg]]
";
"Timeout" = "60000";
|
"Enabled" = "NO";
"Menu Item" = "Mail/Format Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Reformat the selection to around 65 columns.

pasteboard write [fmt [pasteboard read]]
";
|
"Key Equivalent" = ".";
"Menu Item" = "Mail/Insert ASCII Signature";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# Return signature file.

    # Get location for the signature file.
set asciiSig [defaults read TickleServices SignatureFile ~/.signature\
]

if {[file exists $asciiSig]} {
    pasteboard write \"---\\n[readfile $asciiSig]\"
} else {
    alertpanel {Insert Signature} \"Unable to find `$asciiSig'.\"
}
";
|
"Key Equivalent" = "'";
"Menu Item" = "Mail/Insert RTF Signature";
"Receive Type" = "NXRTFPboardType";
"Tcl" = "\
# Return text of RTF signature file.

    # Get location for the signature file.
set rtfSig [defaults read TickleServices RTFSignatureFile ~/.signatur\
e.rtf]

if {[file exists $rtfSig]} {
    if {[file isdirectory $rtfSig]} {
	if {[file exists $rtfSig/TXT.rtf]} {
		# glob the signature to remove leading ~, because
		# the attachment directive won't allow ~.
	    set rtfSig [glob $rtfSig]
	    set sig [expandRTF [readfile $rtfSig/TXT.rtf] $rtfSig]
	    pasteboard write $sig $NXRTFPboardType
	} else {
	    alertpanel {Insert RTF Signature} \"Unable to find TXT.rtf in `[\
file tail $rtfSig]' wrapper.\"
	}
    } else {
	pasteboard write [readfile $rtfSig] $NXRTFPboardType
    }
} else {
    alertpanel {Insert RTF Signature} \"Unable to find `$rtfSig'.\"
}
";
|
"Enabled" = "NO";
"Key Equivalent" = ".";
"Menu Item" = "Mail/Insert Signature";
"Receive Type" = "NXAsciiPboardType";
"Receive Type" = "NXRTFPboardType";
"Tcl" = "\
# Return signature file.

    # Get location for the signature file.
set asciiSig [defaults read TickleServices SignatureFile ~/.signature\
]

    # Under NS3.0, if you put both types in the pasteboard, Mail
    # is smart enough to take the correct type according to the
    # current Compose window's state.
if {![string match \"2.?\" $NeXTVersion]} {
    set rtfSig [defaults read TickleServices RTFSignatureFile ~/.sign\
ature.rtf]
    
    if {[file exists $asciiSig]} {
	if {[file exists $rtfSig]} {
	    pasteboard declare $NXRTFPboardType $NXAsciiPboardType
	    if {[file isdirectory $rtfSig]} {
		if {[file exists $rtfSig/TXT.rtf]} {
			# glob the signature to remove leading ~, because
			# the attachment directive won't allow ~.
		    set rtfSig [glob $rtfSig]
		    set sig [expandRTF [readfile $rtfSig/TXT.rtf] $rtfSig]
		    pasteboard write $sig $NXRTFPboardType
		} else {
		    alertpanel {Insert RTF Signature} \"Unable to find TXT.rtf in `\
[file tail $rtfSig]' wrapper.\"
		}
	    } else {
		pasteboard write [readfile $rtfSig] $NXRTFPboardType
	    }
	}
	pasteboard write \"---\\n[readfile $asciiSig]\"
    } else {
	if {[file exists $rtfSig]} {
	    pasteboard write [readfile [glob $rtfSig]] $NXRTFPboardType
	} else {
	    alertpanel {Insert Signature} \"Unable to find `$asciiSig' or `$\
rtfSig'.\"
	}
    }
} elseif {[file exists $asciiSig]} {
    pasteboard write \"---\\n[readfile $asciiSig]\"
} else {
    alertpanel {Insert Signature} \"Unable to find `$asciiSig'.\"
}
";
|
"Enabled" = "NO";
"Menu Item" = "Mail/Open RTF Signature File";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Open RTF signature file.
#
# I'm not certain how useful this service is since one could easily
# \"Insert RTF Signature\" to get the current signature, modify it, a\
nd
# then \"Set RTF Signature\" to replace it.  That is why the services\
 is
# disabled by default.

    # Get location for the signature file.
set rtfSig [defaults read TickleServices RTFSignatureFile ~/.signatur\
e.rtf]

if {[file exists $rtfSig]} {
    exec open [glob $rtfSig]
} else {
    alertpanel {Open RTF Signature File} \"Unable to find `$rtfSig'.\"
}
";
|
"Enabled" = "NO";
"Menu Item" = "Mail/Open Signature File";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Open signature file.
#
# I'm not certain how useful this service is since one could easily
# \"Insert Signature\" to get the current signature, modify it, and t\
hen
# \"Set Signature\" to replace it.  That is why the services is disab\
led
# by default.

    # Get location for the signature file.
set asciiSig [defaults read TickleServices SignatureFile ~/.signature\
]

if {[file exists $asciiSig]} {
    exec open [glob $asciiSig]
} else {
    alertpanel {Open Signature File} \"Unable to find `$asciiSig'.\"
}
";
|
"Key Equivalent" = ">";
"Menu Item" = "Mail/Quote Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Quote the selection with the default quoteString.  See !Initializat\
ion!
# for info on quoteString.

pasteboard write [quote [pasteboard read]]
";
|
"Enabled" = "NO";
"Key Equivalent" = "R";
"Menu Item" = "Mail/Reformat Quoted Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Unquote the selection with the default quoteString, reformat it, an\
d
# then requote it, again with quoteString.  See !Initialization!  for
# info on quoteString.

    # fmtWidth is used to make fmt take into account the amount of
    # indentation due to the quoteString.
set fmtWidth [expr {65-[string length $quoteString]}]
pasteboard write [quote [fmt [unquote [pasteboard read]] $fmtWidth]]
";
|
"Key Equivalent" = "F";
"Menu Item" = "Mail/Reformat Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Reformat the selection to less than 66 columns using refmt.

#pasteboard write [refmt [pasteboard read] 75 79]
pasteboard write [refmt [pasteboard read]]
";
|
"Menu Item" = "Mail/Set Default Quote String";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Set the default quoteString.

    # In general quote strings shouldn't contain newlines.
if {[string first \"\\n\" [pasteboard read]]!=-1} {
    if {[alertpanel \"Set Quote String\" \"Selected quote string cont\
ains one or more newlines.  Are you certain you wish to use this quot\
e string?\" \"Cancel\" \"OK\"]==1} {
        return
    }
}

    # Set the current value, and also the default for future use.
set quoteString [pasteboard read]
defaults write TickleServices QuoteString $quoteString
";
|
"Key Equivalent" = "Q";
"Menu Item" = "Mail/Set Quote String";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Set the quoteString, without modifying the default.

    # In general quote strings shouldn't contain newlines.
if {[string first \"\\n\" [pasteboard read]]!=-1} {
    if {[alertpanel \"Set Quote String\" \"Selected quote string cont\
ains one or more newlines.  Are you certain you wish to use this quot\
e string?\" \"Cancel\" \"OK\"]==1} {
        return
    }
}

    # Set the current value, but don't change the default.
set quoteString [pasteboard read]
";
|
"Menu Item" = "Mail/Set RTF Signature";
"Send Type" = "NXRTFPboardType";
"Tcl" = "\
# Set the RTF signature.

    # Get location for the signature file.
set rtfSig [defaults read TickleServices RTFSignatureFile ~/.signatur\
e.rtf]

set filenames {}
set rtf [packRTF [pasteboard read $NXRTFPboardType] filenames]
if {[llength $filenames]>0} {
    if {[file extension $rtfSig]==\".rtf\"} {
	set rtfSig ${rtfSig}d
    }
} else {
    if {[file extension $rtfSig]==\".rtfd\"} {
	set rtfSig [file rootname $rtfSig].rtf
    }
}

    # Give the user a last chance before overwriting the file.
if {[file exists $rtfSig]} {
    if {[alertpanel {Set RTF Signature} \"Overwrite current signature\
 in $rtfSig?\" {OK} {Cancel}]!=1} {
	return
    }
	# Still, what if they might want it back?  Back it up.
    catch {exec rm -rf [glob $rtfSig]~}
    catch {exec mv [glob $rtfSig] [glob $rtfSig]~}
}
defaults write TickleServices RTFSignatureFile $rtfSig

if {[llength $filenames]>0} {
    catch {exec mkdirs $rtfSig}
    writefile $rtfSig/TXT.rtf $rtf
    eval [concat exec cp $filenames [list $rtfSig]]
} else {
    writefile $rtfSig $rtf
}
";
|
"Enabled" = "NO";
"Menu Item" = "Mail/Set RTF Signature File";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Set the signature file.
#
# I'm not certain how useful this service is since one could easily
# \"Insert RTF Signature\" to get the current signature, modify it, a\
nd
# then \"Set RTF Signature\" to replace it.  That is why the service \
is
# disabled by default.

    # Fan out the possible file list we received.
set filename [split [pasteboard read $NXFilenamePboardType] \"\\t\"]

    # Since there cannot be more than one signature file at a time,
    # complain if it was a multiple selection.
if {[llength $filename]>1} {
    alertpanel {Set RTF Signature File} {Multi-file selection, but ca\
n only use one file for signature.}
    
    # If the file is a directory or something, complain.
} else if {![file isfile $filaname] && ![file exists $filenames/TXT.r\
tf]} {
    alertpanel {Set RTF Signature File} {Signature file must be a reg\
ular file.}
} else {
    defaults write TickleServices RTFSignatureFile $filename
}
";
|
"Menu Item" = "Mail/Set Signature";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Set the signature.

    # Get location for the signature file.
set asciiSig [defaults read TickleServices SignatureFile ~/.signature\
]

    # Give the user a last chance before overwriting the file.
if {[file exists $asciiSig]} {
    if {[alertpanel {Set Signature} \"Overwrite current signature in \
$asciiSig?\" OK Cancel]!=1} {
	return
    }
	# Still, what if they might want it back?  Back it up.
    catch {exec mv [glob $asciiSig] [glob $asciiSig]~}
}

    # Write the signature file from the pasteboard.
writefile $asciiSig [pasteboard read]
";
|
"Enabled" = "NO";
"Menu Item" = "Mail/Set Signature File";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Set the signature file.
#
# I'm not certain how useful this service is since one could easily
# \"Insert Signature\" to get the current signature, modify it, and t\
hen
# \"Set Signature\" to replace it.  That is why the service is disabl\
ed
# by default.

    # Fan out the possible file list we received.
set filename [split [pasteboard read $NXFilenamePboardType] \"\\t\"]

    # Since there cannot be more than one signature file at a time,
    # complain if it was a multiple selection.
if {[llength $filename]>1} {
    alertpanel {Set Signature File} {Multi-file selection, but can on\
ly use one file for signature.}
    
    # If the file is a directory or something, complain.
} else if {![file isfile $filaname]} {
    alertpanel {Set Signature File} {Signature file must be a regular\
 file.}
} else {
    defaults write TickleServices SignatureFile $filename
}
";
|
"Key Equivalent" = "<";
"Menu Item" = "Mail/Unquote Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Unquote the selection with the default quoteString.  See !Initializ\
ation!
# for info on quoteString.

pasteboard write [unquote [pasteboard read]]
";
# End TickleServices Data
