# Begin TickleServices Version 1.02mab Data
"Divider" = "225.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # This version of exec doesn't strip the trailing newline from th\
e
    # output.
proc nexec {args} {
    global errorInfo

    set fileId [uplevel open [list \"|$args\"]]
    set ret [read $fileId]
    close $fileId
    return $ret
}

    # Splits multi-file Workspace-style lists into TCL-style lists.
proc filenames {} {
    global NXFilenamePboardType
    
	# If there is only one filename, make certain it's a list.
	# Otherwise, split on \\t.
    set filenames [pasteboard read $NXFilenamePboardType]
    if {[string first \"\\t\" $filenames]!=-1} {
	set filenames [split $filenames \"\\t\"]
    } else {
	set filenames [list $filenames]
    }
    return $filenames
}

    # Reads the specified file and returns the text.
proc readfile {filename {hexcode 0}} {
    set fileId [open $filename r]
    if {$hexcode} {
        set ret [read -hexcode $fileId]
    } else {
        set ret [read $fileId]
    }
    close $fileId
    return $ret
}

    # Writes the text to the specified file.
proc writefile {filename text {hexcode 0}} {
    set fileId [open $filename w]
    if {$hexcode} {
        set ret [puts -nonewline -hexcode $fileId $text]
    } else {
        set ret [puts -nonewline $fileId $text]
    }
    close $fileId
    return $ret
}

    # read the quoteString from the defaults database.  The quoteStri\
ng
    # defaults to \">\", which is a semi-standard quoting character.
global quoteString
set quoteString [defaults read TickleServices QuoteString \">\"]

    # read the indentString from the defaults database.  The indentSt\
ring
    # defaults to \"    \".
global indentString
set indentString [defaults read TickleServices IndentString \"    \"]

    # Quote the passed string using quoteString and return it.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the stuff in the line and
    # the end-of-line (optional because the last line may not have
    # one), and then replace the front of the line with the quoteStri\
ng.
    # Escape \\ and & in the quote string just in case.
proc quote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $str \"[escape $qs {\\\\\
&}]\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Strip quotes from the passed string.
    # Don't modify the regsub unless you _really_ know what you're
    # doing.  It's sort of evil ... I match the quote string at the
    # front, then the stuff in the line, and then the end-of-line
    # (optional because the last line may not have one), and replace
    # that with just the in-the-line stuff plus the end-of-line.
proc unquote {str {qs {}}} {
    if {![string length $qs]} {
        global quoteString
        set qs $quoteString
    }
    if {[regsub -all \"^[escape $qs {\\\\()[]^*$?}](\\[^\\n\\]*)(\\n?\
)\" $str \"\\\\1\\\\2\" foo]} {
	return $foo
    } else {
	return $str
    }
}

    # Expand tabs to spaces.
proc expand {str} {
    return [nexec expand << $str]
}

    # Unexpand spaces to tabs.
proc unexpand {str} {
    return [nexec unexpand << $str]
}

    # Reformat the passed string.
proc fmt {str {width 66}} {
    return [nexec /usr/ucb/fmt $width [expr {$width+8}] << $str]
}
";
"Window Size" = "{ 795.000, 344.000}";
|
"Menu Item" = "Tickle Services/Append Text...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Append the selection to the end of an existing file.

set fileId [open [openpanel -title Append -return] a]
puts -nonewline $fileId [pasteboard read]
close $fileId
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Copy as Service";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Snarf the selection and put it on the main pasteboard as a
# TickleServices service.

pasteboard -main write [pasteboard read] \"Tickle Service Pasteboard \
Type\"
";
|
"Menu Item" = "Tickle Services/Count Words...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Run wc to count the words, and format a nice panel to display
# the results.  Could probably do the counting as easily directly
# in TCL, but it never seemed worthwhile.

set result [exec wc << [pasteboard read]]
set msg \"Found:\\n\\t[lindex $result 0]\\tline\"
if {[lindex $result 0]!=1} {
    append msg \"s\"
}
append msg \"\\n\\t[lindex $result 1]\\tword\"
if {[lindex $result 1]!=1} {
    append msg \"s\"
}
append msg \"\\n\\t[lindex $result 2]\\tcharacter\"
if {[lindex $result 2]!=1} {
    append msg \"s\"
}
alertpanel {Word Count} \"$msg\" {OK}
";
"Timeout" = "60000";
|
"Key Equivalent" = "e";
"Menu Item" = "Tickle Services/Enscript Text...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Feed the selection to enscript for printing.
# The original for this service was from
# Mike Dixon <mdixon@parc.xerox.com>

set msg \"Print selection to [defaults read System Printer]?\"
case [alertpanel Enscript $msg \"2 column\" \"1 column\" \"Parameters\
\"] {
    1 {exec enscript -2r << [pasteboard read]}
    0 {exec enscript << [pasteboard read]}
    -1 {
	set cmd [promptpanel -select 9-end Enscript \"enscript -2r\"]
	eval [concat exec $cmd << [list [pasteboard read]]]
    }
}
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Evaluate Arithmetic";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
pasteboard write [expr [pasteboard read]]
";
|
"Key Equivalent" = "#";
"Menu Item" = "Tickle Services/Format # Comment";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Reformat #-style comments.

    # Expand any tabs in the text.
set text [expand [pasteboard read]]

    # Figure out how far it's indented.
set trimdlen [string length [string trimleft $text]]
set indent [expr [string length $text]-$trimdlen-1]

    # Get a string of spaces for indentation.
set indentString [string range $text 0 $indent]

    # Strip leading '#', and ' '.
if {[regsub -all \"^\\[ #\\]*(\\[^\\n\\]*)(\\n?)\" $text \"\\\\1\\\\2\
\" foo]} {
    set text $foo
}

    # Reformat the remaining text.
set text [string trimright [fmt $text [expr 62-$indent]]]
append text \"\\n\"

    # Put \"# \" in front of each line.
if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $text \"# \\\\1\\\\2\" foo]\
} {
    set text $foo
}

    # If there was indentation, apply it.
if {$indentString!=\"\"} {
    set text [quote $text $indentString]
}

    # Unexpand leading spaces to tabs and put results on the pasteboa\
rd.
pasteboard write [unexpand $text]
";
|
"Key Equivalent" = "/";
"Menu Item" = "Tickle Services/Format // Comment";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Reformat //-style comments.

    # Expand any tabs in the text.
set text [expand [pasteboard read]]

    # Figure out how far it's indented.
set trimdlen [string length [string trimleft $text]]
set indent [expr [string length $text]-$trimdlen-1]

    # Get a string of spaces for indentation.
set indentString [string range $text 0 $indent]

    # Strip leading '/', '*', and ' '.
if {[regsub -all \"^\\[ /*\\]*(\\[^\\n\\]*)(\\n?)\" $text \"\\\\1\\\\\
2\" foo]} {
    set text $foo
}

    # Reformat the remaining text.
set text [string trimright [fmt $text [expr 62-$indent]]]
append text \"\\n\"

    # Put \"# \" in front of each line.
if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $text \"// \\\\1\\\\2\" foo\
]} {
    set text $foo
}

    # If there was indentation, apply it.
if {$indentString!=\"\"} {
    set text [quote $text $indentString]
}

    # Unexpand leading spaces to tabs and put results on the pasteboa\
rd.
pasteboard write [unexpand $text]
";
|
"Key Equivalent" = "C";
"Menu Item" = "Tickle Services/Format Comment";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# This is preliminary work to build a comment-formatter.
# I like to format my c comments like:
#	/* This is a comment, wrapped
#	 * pretty narrowly, just to show
#	 * what I mean.
#	 */

    # Expand any tabs in the text.
set text [expand [pasteboard read]]

    # Figure out how far it's indented.
set trimdlen [string length [string trimleft $text]]
set indent [expr [string length $text]-$trimdlen-1]

    # Get a string of spaces for indentation.
set indentString [string range $text 0 $indent]

    # Strip leading '/', '*', and ' '.
if {[regsub -all \"^\\[ /*\\]*(\\[^\\n\\]*)(\\n?)\" $text \"\\\\1\\\\\
2\" foo]} {
    set text $foo
}

    # Reformat the remaining text.
set text [string trimright [fmt $text [expr 62-$indent]]]

    # Put \" * \" in front of each line.
if {[regsub -all \"^(\\[^\\n\\]*)(\\n?)\" $text \" * \\\\1\\\\2\" foo\
]} {
    set text $foo
}

    # Bracket with \"/\" and \"*/\"
set text \"/[string range $text 1 end]\\n */\\n\"

    # If there was indentation, apply it.
if {$indentString!=\"\"} {
    set text [quote $text $indentString]
}

    # Unexpand leading spaces to tabs and put results on the pasteboa\
rd.
pasteboard write [unexpand $text]
";
|
"Menu Item" = "Tickle Services/Grep In...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
if {[llength [filenames]]==1} {
    set filename \"`[file tail [lindex [filenames] 0]]'\"
} else {
    set filename \"\"
}
set text [promptpanel \"Grep In $filename\" {Enter search text here}]
if {[catch [concat nexec grep -n [list $text] [filenames]] res]} {
    set savedInfo $errorInfo
    if {$res!=\"\"} {
	error $res $savedInfo
    }
}

if {$res==\"\"} {
    alertpanel \"Grep In\" \"`$text' not found.\"
} else {
	# Execute the open in the background because open doesn't
	# exit in a timely manner when opening from stdin.
    exec open << $res &
    set TSInhibitReactivate {}
}
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Import ASCII...";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# Pop up an Open Panel, choose a file and return text of it.

pasteboard write [readfile [openpanel -title \"Import ASCII\" -return\
]]
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Import RTF...";
"Receive Type" = "NXRTFPboardType";
"Tcl" = "\
# Pop up an Open Panel, choose a file and return text of it as RTF.

set ret [readfile [openpanel -title {Import RTF} -return]]
pasteboard write $ret $NXRTFPboardType
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Import TIFF...";
"Receive Type" = "NXTIFFPboardType";
"Tcl" = "\
# This _should_ load the specified .tiff file, and return it to the
# sender.  Unfortunately, Icon doesn't work with it, so I can't say
# for certain that it _does_ work!

set ret [readfile [openpanel -types tiff -title {Import TIFF} -return\
] 1]
pasteboard -hexcode write $ret $NXTIFFPboardType
";
"Timeout" = "60000";
|
"Key Equivalent" = "]";
"Menu Item" = "Tickle Services/Indent Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Indent the selection with the indentString.  indentString defaults
# to \"    \" (see !Initialization!).

pasteboard write [unexpand [quote [pasteboard read] $indentString]]
";
|
"Menu Item" = "Tickle Services/Launch Front-end";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXPostScriptPboardType";
"Send Type" = "NXRTFPboardType";
"Send Type" = "NXTIFFPboardType";
"Tcl" = "\
# Launch the TickleServices front-end without having to go find it
# manually.

    # Initially attempt to launch it using open.  If that fails,
    # try to find it.
if {[catch {exec open -a TickleServices}]} {

	# First, try to get it from the defaults database.
    set app [defaults read TickleServices FrontEnd {}]
    
	# If not there, try to find it on the Workspace LaunchPaths.
    if {$app==\"\" || ![file executable $app/TickleServices]} {
	    # Get the list of application paths for Workspace.
	set apppaths [defaults read Workspace ApplicationPaths {}]
	if {$apppaths==\"\"} {
	    set apppaths {
		~/Apps
		/LocalApps
		/NextApps
		/NextDeveloper/Apps
		/NextAdmin
		/NextDeveloper/Demos
	    }
	} else {
	    set apppaths [split $apppaths \":\"]
	}
	foreach path $apppaths {
	    if {[file executable $path/TickleServices.app/TickleServices]} {
		set app $path/TickleServices.app
	    }
	}
    }
    
	# If still not found, let the user find it.
    if {$app==\"\" || ![file executable $app/TickleServices]} {
	set app [openpanel -file TickleServices.app -title {TickleServices A\
pp} -types app -return]
	
	    # Assume we found something, and retain it for the future.
	defaults write TickleServices FrontEnd $app
    }
    if {![catch {glob $app} app]} {
	exec open $app
    }
}
";
|
"Key Equivalent" = "N";
"Menu Item" = "Tickle Services/Number Lines";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Number the selected lines.

pasteboard write [nexec /bin/cat -n << [pasteboard read]]
";
|
"Key Equivalent" = "[";
"Menu Item" = "Tickle Services/Outdent Text";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Outdent the selection according to indentString.  indentString defa\
ults
# to \"    \" (see !Initialization!).

set outdented [unquote [expand [pasteboard read]] $indentString]
pasteboard write [unexpand $outdented]
";
|
"Key Equivalent" = "L";
"Menu Item" = "Tickle Services/Pipe From...";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
# Prompt for a command to run, and replace the selection with the
# command's output.

set cmd [promptpanel {Pipe From Command}]
if {$cmd==\"\"} {
    return
}
    # Split any <file commands.  Under most shells, something
    # like </tmp/barf implies that < and /tmp/barf are separate
    # syntactic elements, while under exec they aren't.
regsub -all \"(<)(\\[^ \\])\" $cmd \"\\\\1 \\\\2\" cmd
pasteboard write [eval [concat nexec $cmd]]
";
"Timeout" = "60000";
|
"Key Equivalent" = "|";
"Menu Item" = "Tickle Services/Pipe Through...";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Prompt for a command to run, and filter the selection through
# that command.

set cmd [promptpanel {Pipe Through Command}]
if {$cmd==\"\"} {
    return
}
pasteboard write [eval [concat nexec $cmd << [list [pasteboard read]]\
]]
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Pipe To...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Prompt for a command and pipe the selection to the command.  >
# can be used to pipe the output to a file, >> can be used to append
# the output to a file.

set cmd [promptpanel {Pipe To Command}]
if {$cmd==\"\"} {
    return
}
set index [lsearch $cmd \">>*\"]
if {$index!=-1} {
    set filename [lindex $cmd $index]
    if {$filename==\">>\"} {
	set filename [lindex $cmd [expr $index+1]]
	set cmd [lreplace $cmd $index [expr $index+1]]
    } else {
	set filename [string range $filename 2 end]
	set cmd [lreplace $cmd $index $index]
    }
    if {[file exists $filename]} {
	set file [open $filename a]
    } else {
	set file [open $filename w]
    }
    puts -nonewline $file [eval [concat nexec $cmd << [list [pasteboa\
rd read]]]]
    close $file
} else {
	# Split any >file commands.  Under most shells, something
	# like >/tmp/barf implies that > and /tmp/barf are separate
	# syntactic elements, while under exec they aren't.
    regsub -all \"(>)(\\[^ \\])\" $cmd \"\\\\1 \\\\2\" cmd
    eval [concat exec $cmd << [list [pasteboard read]]]
}
";
"Timeout" = "60000";
|
"Key Equivalent" = "p";
"Menu Item" = "Tickle Services/Print Text";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Print the selection to the default printer.

exec lpr << [pasteboard read]
";
|
"Key Equivalent" = "S";
"Menu Item" = "Tickle Services/Save ASCII...";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Save the selection to a user-selected file.

writefile [savepanel -return] [pasteboard read]
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Save RTF...";
"Send Type" = "NXRTFPboardType";
"Tcl" = "\
# Save the RTF selection to a user-selected rtf file.

writefile [savepanel -return] [pasteboard read $NXRTFPboardType]
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Save TIFF...";
"Send Type" = "NXTIFFPboardType";
"Tcl" = "\
# Save the TIFF selection to a user-selected .tiff file.  Use -hexcod\
e
# to protect binary data.

set tiff [pasteboard -hexcode read $NXTIFFPboardType]
writefile [savepanel -type tiff -return] $tiff 1
";
"Timeout" = "60000";
|
"Menu Item" = "Tickle Services/Sort";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
pasteboard write [nexec sort << [pasteboard read]]
";
|
"Menu Item" = "Tickle Services/Time";
"Receive Type" = "NXAsciiPboardType";
"Tcl" = "\
pasteboard write [exec date]
";
|
"Key Equivalent" = "U";
"Menu Item" = "Tickle Services/Unnumber Lines";
"Receive Type" = "NXAsciiPboardType";
"Send Type" = "NXAsciiPboardType";
"Tcl" = "\
# Remove line numbers which were inserted by Number Lines.

    # Don't you just love long regular expressions? This one could be
    # more easily represented by \"^\\[ 0-9\\]*\\t(\\[^\\n\\]*)(\\n?)\
\".  I left
    # it as the longer form in order to reduce the likelihood of
    # inadvertant application.  [Note that rexp will only match a
    # sequence of 5 spaces or digits, followed by a digit, followed b\
y
    # a tab.  The above expression would match any sequence of spaces
    # or digits followed by a tab.]
set rexp \"^\\[ 0-9\\]\\[ 0-9\\]\\[ 0-9\\]\\[ 0-9\\]\\[ 0-9\\]\\[0-9\\\
]\\t(\\[^\\n\\]*)(\\n?)\" 
if {[regsub -all $rexp [pasteboard read] \"\\\\1\\\\2\" foo]} {
    pasteboard write $foo
}
";
# End TickleServices Data
