# Begin TickleServices Version 1.02mab Data
"Divider" = "217.000000";
"Menu Item" = "!Initialization!";
"Tcl" = "\
    # Splits multi-file Workspace-style lists into TCL-style lists.
proc filenames {} {
    global NXFilenamePboardType
    
	# If there is only one filename, make certain it's a list.
	# Otherwise, split on \\t.
    set filenames [pasteboard read $NXFilenamePboardType]
    if {[string first \"\\t\" $filenames]!=-1} {
	set filenames [split $filenames \"\\t\"]
    } else {
	set filenames [list $filenames]
    }
    return $filenames
}

    # Convert a path of the form /bla/bla/bla/.../tail to
    # \"tail /bla/bla/bla/...\"
proc path2title {path} {
    return \"[file tail $path]  1  [file dirname $path]\"
}

    # Convert a path back from a title.
proc title2path {title} {
    if {[regexp \"(.*)  1  (.*)\" $title barf tail dir]} {
	return $dir/$tail
    } else {
	error \"`$title' invalid\"
    }
}

    # Apply the operation procedure to the each element of list
    # and return the resulting list.  Yeah, it's sort of like LISP's
    # mapcar, but not as useful.
proc mapcar {operation list} {
    set ret {}
    foreach elt $list {
	lappend ret [$operation $elt]
    }
    return $ret
}
    # Apply the operation procedure to the each of the parameter
    # sets in args and return the resulting list.
proc mapfn {operation args} {
    set len [llength [lindex $args 0]]
    foreach arg $args {
	if {$len!=[llength $arg]} {
	    error \"Argument length mismatch in argument [lsearch $arg $args\
]\"
	}
    }
    for {set i 0} {$i<$len} {incr i} {
	set oargs [list $operation]
	foreach arg $args {
	    lappend oargs [lindex $arg $i]
	}
	lappend ret [eval $oargs]
    }
    return $ret
}

    # Pass \"arguments\" as arguments to \"command\".  The arguments
    # are reformed into a list not containing newlines, since
    # otherwise newlines would separate commands.  Thus, passargs
    # can be used as a convenience to make scripts easier to read.
proc passargs {command arguments} {
    return [uplevel $command [join [split $arguments]]]
}

    # Attempt to find the cmdName command and return the path to
    # it.  It is cached in the global Command$cmdName variable for
    # future uses and in the default TickleServices Command$cmdName
    # for future runs.
proc findcmd {cmdName} {
    set varname Command$cmdName
    global $varname env
    
	# I have to say [set $varname] to get the value of $varname,
	# because $$varname won't do it (that attempts to get the
	# value of the variable \"$varname\" instead).  This works
	# because the $varname substitution happens _before_ set
	# is called.
    if {[info exists $varname] && [file executable [set $varname]]} {
	return [set $varname]
    }

	# Get location from defaults.
    set cmd [defaults read TickleServices $varname \"\"]
    
	# If it's not set or not there, try to find it.
    if {$cmd==\"\" || ![file executable $cmd]} {
	set cmd \"\"
	
	    # Attempt to find the user's path.  This is so bizarre
	    # because you want to get the PATH from the user's
	    # command-line shell, note TickleServer's PATH, which
	    # was inherited from Workspace and is not sufficient
	    # to our needs.
	if {[info exists env(SHELL)]} {
	    set shell $env(SHELL)
	} else {
	    set shell /bin/csh
	}
	    # This has only been verified to work with these six
	    # shells.  If the user isn't using one of them, try
	    # csh as a last resort under the assumption that their
	    # shell won't work, or might even do something discourteous
	    # like never exit.  Let me know if other shells work.
	if {[lsearch [file tail $shell] {bash csh rc sh tcsh zsh}]==-1} {
	    set shell /bin/csh
	}
	    # The @/: crud is to differentiate the echo'ed output
	    # from anything the shell puts out in the course of
	    # starting up.  In other words, it is a hack.
	if {![catch {exec $shell -c \"echo \\\"@@:@@:@@\\$PATH@:@@@@:@\\\"\"\
} path]} {
	    if {[regexp \"@@:@@:@@(\\[^#\\]*)@:@@@@:@\" $path barf path]} {
		set path [split $path \":\"]
		foreach dir $path {
		    if {[file executable $dir/$cmdName]} {
			set cmd $dir/$cmdName
			break
		    }
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices.app
	    # wrapper.
	if {$cmd==\"\"} {
	    set apppaths [defaults read Workspace ApplicationPaths {}]
	    if {$apppaths==\"\"} {
		set apppaths {
		    ~/Apps
		    /LocalApps
		    /NextApps
		    /NextDeveloper/Apps
		    /NextAdmin
		    /NextDeveloper/Demos
		}
	    } else {
		set apppaths [split $apppaths \":\"]
	    }
	    foreach path $apppaths {
		if {[file executable $path/TickleServices.app/$cmdName]} {
		    set cmd $path/TickleServices.app/$cmdName
		    break
		}
	    }
	}
	
	    # Still didn't find it.  Go look in the TickleServices
	    # ServicesPaths directories.
	if {$cmd==\"\"} {
	    set servicespaths [defaults read TickleServices ServicesPaths {}\
]
	    if {$servicespaths==\"\"} {
		set servicespaths {
		    ~/Library/TickleServices
		    /LocalLibrary/TickleServices
		}
	    } else {
		set servicespaths [split $servicespaths \":\"]
	    }
	    foreach path $servicespaths {
		if {[file executable $path/$cmdName]} {
		    set cmd $path/$cmdName
		    break
		}
	    }
	}
	
	    # If we've found something, ask the user for verification.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    case [alertpanel \"Find $cmdName executable\" \"Use `$cmd' for $\
cmdName?\" \"OK\" \"Choose\" \"Cancel\"] {
		0	{set cmd \"\"}
		-1	return
	    }
	}
	
	    # If we haven't found it yet, ask the user about it.
	if {$cmd==\"\" || ![file executable $cmd]} {
	    set cmd [passargs openpanel {
		-title \"Select $cmdName executable\"
		-prompt \"Name:\"
	    }]
	}
	
	    # If we've got it, write it to the defaults database.
	if {$cmd!=\"\" && [file executable $cmd]} {
	    defaults write TickleServices $varname $cmd
	} else {
	    return
	}
    }
    set $varname $cmd
    return $cmd
}
";
"Window Size" = "{ 812.000, 384.000}";
|
"Menu Item" = "Workspace/Add To LaunchPaths";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Add an entry to the Workspace LaunchPaths default.

set filenames [filenames]
set launchpaths [split [defaults read Workspace LaunchPaths \"\"] \";\
\"]
foreach filename $filenames {
    lappend launchpaths $filename
}
defaults write Workspace LaunchPaths [join $launchpaths \";\"]
";
|
"Menu Item" = "Workspace/Arrange LaunchPaths...";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXFontPboardType";
"Send Type" = "NXMailPboardType";
"Send Type" = "NXPostScriptPboardType";
"Send Type" = "NXRTFPboardType";
"Send Type" = "NXRulerPboardType";
"Send Type" = "NXTIFFPboardType";
"Send Type" = "NXTabularTextPboardType";
"Tcl" = "\
# Rearrange the Workspace LaunchPaths default.

set launchpaths [split [defaults read Workspace LaunchPaths \"\"] \";\
\"]
if {[llength $launchpaths]==0} {
    alertpanel {Arrange LaunchPaths} {No LaunchPaths entries to arran\
ge.}
    return
} elseif {[llength $launchpaths]==1} {
    alertpanel {Arrange LaunchPaths} {Only one LaunchPaths entry avai\
lable.}
    return
}
while {1} {
    set change [listpanel -multiple -empty {Rearrange LaunchPaths} [m\
apcar path2title $launchpaths] \"Save\" \"Move to front\" \"Move to e\
nd\"]
    if {[lindex $change 0]==1} {
	break
    } else {
	set changes [mapcar title2path [lindex $change 1]]
	foreach choice $changes {
	    set index [lsearch $launchpaths $choice]
	    if {$index!=-1} {
		set launchpaths [lreplace $launchpaths $index $index]
	    }
	}
	if {[lindex $change 0]==0} {
	    set launchpaths [concat $changes $launchpaths]
	} else {
	    set launchpaths [concat $launchpaths $changes]
	}
    }
}
defaults write Workspace LaunchPaths [join $launchpaths \";\"]
";
|
"Menu Item" = "Workspace/Edit LaunchPaths...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Edit the Workspace LaunchPaths default.

set launchpaths [split [defaults read Workspace LaunchPaths \"\"] \";\
\"]
while {1} {
    if {[llength $launchpaths]>0} {
	set change [listpanel -multiple -empty {Edit LaunchPaths} [mapcar pa\
th2title $launchpaths] \"Save\" \"Add\" \"Remove\"]
    } else {
	set change {0 {}}
    }
    if {[lindex $change 0]==1} {
	break
    } elseif {[lindex $change 0]==0} {
	set choices [openpanel -title {LaunchPaths} -multiple]
	if {$choices==\"\" && [llength $launchpaths]==0} {
	    break
	}
	set launchpaths [concat $launchpaths $choices]
    } else {
	foreach choice [mapcar title2path [lindex $change 1]] {
	    set index [lsearch $launchpaths $choice]
	    if {$index!=-1} {
		set launchpaths [lreplace $launchpaths $index $index]
	    }
	}
    }
}
defaults write Workspace LaunchPaths [join $launchpaths \";\"]
";
|
"Menu Item" = "Workspace/Hide Files";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Cause a file to be \"hidden\" in Workspace by adding it to its
# directory's .hidden file.

set filenames [filenames]
set tails {}
set dirname [file dirname [lindex $filenames 0]]
if {[file exists $dirname/.hidden] && ![file writable $dirname/.hidde\
n]} {
    alertpanel {Hide Files} \"Unable to hide files because `$dirname/\
.hidden' is not writable\"
    return
}
set hidden [open $dirname/.hidden a+]
foreach file $filenames {
    puts $hidden [file tail $file]
}
close $hidden
catch [exec [findcmd WSNotify]]
";
|
"Key Equivalent" = "A";
"Menu Item" = "Workspace/Launch Apps...";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXFontPboardType";
"Send Type" = "NXPostScriptPboardType";
"Send Type" = "NXRTFPboardType";
"Send Type" = "NXRulerPboardType";
"Send Type" = "NXTIFFPboardType";
"Send Type" = "NXTabularTextPboardType";
"Tcl" = "\
# (This service used to be in File.ts, but I think it makes more
# sense here -scott)
#
# This is an attempt at a facility for giving reasonably easy access
# to places where applications live. \"We don't need no steenking
# dock-extenders\".
# 
# Originally, I had coded this in two parts.  First, it'd use a listp\
anel
# to ask the user which path to consult, then it would use an openpan\
el
# to choose an application.  Problem was, openpanel would give me eit\
her
# just the .app files, else it would give me all files with .app file\
s
# as directories.  Either wasn't right.  Then, I found that just dump\
ing
# everything into a listpanel only took a couple seconds, so I went
# with it.

    # Get the list of application paths for Workspace.
set apppaths [defaults read Workspace ApplicationPaths {}]
if {$apppaths==\"\"} {
    set apppaths {
	~/Apps
	/LocalApps
	/NextApps
	/NextDeveloper/Apps
	/NextAdmin
	/NextDeveloper/Demos
    }
} else {
    set apppaths [split $apppaths \":\"]
}

set basenames {}
set appnames {}
foreach path $apppaths {
    if {![catch {glob $path/*} files]} {
	foreach file $files {
	    if {[file extension $file]==\".app\"} {
		lappend appnames $file
		lappend basenames [file tail $file]
	    } elseif {[file isfile $file] && [file executable $file]} {
		lappend appnames $file
		lappend basenames [file tail $file]
	    }
	}
    }
}
if {[llength $appnames]} {
    set apps {}
    foreach app [listpanel -sort -multiple {Which Apps?} $basenames] \
{
	if {[set i [lsearch $appnames \"*/$app\"]]!=-1} {
	    lappend apps [lindex $appnames $i]
	}
    }
    if {$apps!=\"\"} {
	eval [concat exec open $apps]
	
	    # Ugly hack time ... when I'm launching these apps, I want one
	    # of them to become the next foreground app, right?  But,
	    # TickleServices is going to try to pass that job off to the
	    # previous foreground app.  So, I added a hack to the
	    # services-handler which will check for the following variable,
	    # and if it's there, it arranges things so that no application
	    # is in the foreground.
	global TSInhibitReactivate
	set TSInhibitReactivate {}
    }
} else {
    alertpanel {Launch} \"No apps to launch in `$apppath'.\"
}
";
"Timeout" = "60000";
|
"Menu Item" = "Workspace/Open in Edit";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# (This service used to be in Tickle Services.ts, but I think it
# makes more sense here -scott)
#
# Open the selected file or filename in Edit.  Originally
# submitted by Mike Dixon <mdixon@parc.xerox.com>

global NXFilenamePboardType
if {[lsearch [pasteboard types] $NXAsciiPboardType]==-1} {
    set fnames [filenames]
} else {
    set fnames [pasteboard read]
	# Run glob in case fnames specifies regex characters.
    if {![catch {glob $fnames} res]} {
	set fnames $res
    }
}
eval [concat exec /usr/bin/open -a Edit $fnames]
set TSInhibitReactivate {}
";
|
"Menu Item" = "Workspace/Toggle Unix Expert";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXTIFFPboardType";
"Tcl" = "\
# Toggle Workspace Unix Expert mode
# The original version of this service was contributed by
# Gordie Freedman <gordie@cyclesoft.com>

if {[string toupper [defaults read System UnixExpert YES]]==\"NO\"} {
    defaults write System UnixExpert YES
} else {
    defaults write System UnixExpert NO
}
catch [exec [findcmd WSNotify]]
";
|
"Key Equivalent" = "U";
"Menu Item" = "Workspace/Unhide Apps...";
"Send Type" = "NXAsciiPboardType";
"Send Type" = "NXFilenamePboardType";
"Send Type" = "NXFontPboardType";
"Send Type" = "NXPostScriptPboardType";
"Send Type" = "NXRTFPboardType";
"Send Type" = "NXRulerPboardType";
"Send Type" = "NXTIFFPboardType";
"Send Type" = "NXTabularTextPboardType";
"Tcl" = "\
# This uses the \"unhide\" command that
# eps@futon.SFSU.EDU (Eric P. Scott) posted to the net to allow
# the user to unhide selected applications.

set processes [exec ps xww]
set apps {}
foreach proc [split $processes \"\\n\"] {
    if {[regexp \"....................(/.*-MachLaunch \\[0-9\\]* \\[0\
-9\\]*.*)\" $proc barf proc]} {
	while {[regexp \"(/.*) -.*\" $proc barf p]} {
	    set proc $p
	}
	set name [file tail $proc]
	if {$name==\"WM\"} {
	    set name Workspace
	} elseif {[string match \"*.daemon\" $name]} {
	    continue
	}
	lappend apps $name
    }
}
set unhide [findcmd unhide]
if {$unhide!=\"\"} {
    eval [concat exec [list $unhide] [listpanel -multiple -sort {Unhi\
de Apps} $apps]]
    global TSInhibitReactivate
    set TSInhibitReactivate {}
}
";
"Timeout" = "60000";
|
"Menu Item" = "Workspace/Unhide Files In...";
"Send Type" = "NXFilenamePboardType";
"Tcl" = "\
# Give the user a list of the files in the selected directory that
# are \"hidden\", and allow them to choose files to unhide.

set dirname [filenames]
if {[llength $dirname]>1} {
    set dirname [file dirname [lindex $dirname 0]]
}
set files {}
if {[file exists $dirname/.hidden]} {
    if {![file writable $dirname/.hidden]} {
	alertpanel {Unhide Files} \"Unable to unhide files because `$dirname\
/.hidden' is not writable\"
	return
    }
    set hidden [open $dirname/.hidden r]
    foreach file [split [read $hidden] \\n] {
	if {$file!=\"\" && [lsearch $files $file]==-1} {
	    lappend files $file
	}
    }
    close $hidden
} elseif {![file writable $dirname]} {
    alertpanel {Unhide Files} \"Unable to unhide files because `$dirn\
ame' is not writable\"
    return
}
if {$files==\"\"} {
    alertpanel {Unhide Files} \"No files hidden in `[file tail $dirna\
me]'.\"
    return
}
set unhide [listpanel -multiple -empty -sort \"Unhide in `[file tail \
$dirname]'\" $files]
if {$unhide!=\"\"} {
    foreach file $unhide {
	set index [lsearch $files $file]
	if {$index!=-1} {
	    set files [lreplace $files $index $index]
	}
    }
    if {$files==\"\"} {
	exec rm -f $dirname/.hidden
    } else {
	set hidden [open $dirname/.hidden w]
	foreach file $files {
	    puts $hidden $file
	}
	close $hidden
    }
    catch [exec [findcmd WSNotify]]
}
";
# End TickleServices Data
